Package: amp.sim
Title: Flexible Simulation Utilities for Pharmacometric Modeling
Version: 0.1.1
Authors@R: c(person("Richard", "Hooijmaijers", email = "richardhooijmaijers@gmail.com", role = c("aut", "cre","cph")),
             person("LAPP Consultants",email = "info@lapp.nl", role=c("fnd","cph")))
Description: The goal of 'amp.sim' is to transform 'NONMEM' models into R syntax so they can be used for
    simulations using the 'deSolve', 'nlmixr2' or 'mrgsolve' package. Additionally, functionality is included to aid simulations performed
    directly in 'NONMEM' and to automatically create shiny apps for simulation models.
URL: https://leidenadvancedpkpd.github.io/amp.sim/,
        https://github.com/LeidenAdvancedPKPD/amp.sim
BugReports: https://github.com/LeidenAdvancedPKPD/amp.sim/issues
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: quarto
Depends: R (>= 4.1.0)
Imports: stats, utils, MASS, whisker, rlang, cli, lifecycle, NMdata (>=
        0.2.3)
Suggests: nonmem2rx, tidyr, shiny, shinyjs, bslib, rstudioapi, deSolve,
        rxode2, mrgsolve, R3port, ggplot2, dplyr, testthat, quarto
NeedsCompilation: no
Packaged: 2026-05-11 07:32:12 UTC; r.hooijmaijers
Author: Richard Hooijmaijers [aut, cre, cph],
  LAPP Consultants [fnd, cph]
Maintainer: Richard Hooijmaijers <richardhooijmaijers@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 08:20:07 UTC
Built: R 4.6.0; ; 2026-05-11 11:27:51 UTC; unix

# Generated by roxygen2: do not edit by hand

export(conv_pow)
export(convert_nmsyntax)
export(convert_nonmem)
export(cut_equal)
export(dose_func)
export(dput2)
export(get_est)
export(get_inits)
export(get_nmblock)
export(get_param)
export(get_parmvar)
export(make_nmsimmodel)
export(mdose)
export(mod2shiny)
export(model_validation)
export(nmlistblock)
export(overlaying)
export(par_delete)
export(pos_clpar)
export(sample_par)
export(sample_sim)
export(settings2df)
export(simdata)
export(split_sim)
export(tmpl_model)
importFrom(lifecycle,deprecated)
importFrom(rlang,.data)

<!-- badges: start -->
[![Codecov test coverage](https://codecov.io/gh/LeidenAdvancedPKPD/amp.sim/graph/badge.svg)](https://app.codecov.io/gh/LeidenAdvancedPKPD/amp.sim)
<!-- badges: end -->

# Introduction <img src="man/figures/logo.png" align="right" alt="" width="120" />

There are quite a few R packages available to simulate (pharmacometric) models in R.
This package is not intended to be yet another simulation package, but rather a package that can be used alongside available simulation frameworks.
This means that the package does not include functionality to perform the actual simulations, but rather **amp**lifies it:

1. Automate and streamline simulations within NONMEM
2. Translate NONMEM models to R simulation frameworks (support for  `deSolve`, `rxode2`, `nlmixr2` and `mrgsolve`)
3. Directly create `shiny` applications of (translated) models

Currently the package is under active development and can be installed using:

```R
devtools::install_github("LeidenAdvancedPKPD/amp.sim")
```

The pkgdown website contains various articles on how to use the package and what 
should be taken into account when translating models.

This package was initially developed as an in-house package at LAP&P, and was started in 2017. 
Various versions were developed, where many people within LAP&P helped in making the package better and more robust. 
Without them this package wouldn't be possible!

# Other packages

The main packages in R to perform simulations for pharmacometrics are `nlmixr2` and `mrgsolve`. The `deSolve` package is a 
more general solution which could require some optimization in case of large/complex simulations.
Besides simulations in R, NONMEM is a tool often used to perform simulations as well. Because NONMEM itself is 
low level, R packages like `NMSim` can make these type of simulations much easier. For translating models, there are solutions
like `nonmem2mrgsolve`, `pharmpy` and `nonmem2rx` (called in this package).

This package only amplifies packages like `nlmixr2` and `mrgsolve`. The functionality for NONMEM simulations differ from the 
`NMSim` package. The `amp.sim` package is more low level and has no advanced functionality to start/run NONMEM. 
Implementation is centered towards the dataset, in which a control stream is tailored towards. 
There is an overlap with the model translation options, although this package aims to combine different translations and 
extends this with other simulation tools and possible `shiny` implementation.

# Model translation test

To test how well this package can translate NONMEM models a separate repository is available [here](https://github.com/RichardHooijmaijers/amp.sim.test).
In here dozens of models from the ddmore repository are tested. This test indicate that almost all translations are successful.
The main reason some models could not be fully translated is mainly caused by a subtle difference in model syntax which can be fixed quite easily.

## -----------------------------------------------------------------------------
#| eval: false
# library(amp.sim)
# convert_nonmem("run1.mod", "example1", type_return = "deSolve")
# # convert_nonmem("run1.mod", "example2", type_return = "rxode2")
# # For rxode2/nlmixr2 it is advised to let the nonmem2rx package
# # do the translation, added value is that the model can also be use for fitting!
# convert_nonmem("run1.mod", "example2", type_return = "nonmem2rx")
# convert_nonmem("run1.mod", "example3", type_return = "mrgsolve")


## -----------------------------------------------------------------------------
#| eval: true
#| echo: false
library(amp.sim)
ret <- convert_nonmem(system.file("example_models/PK.1CMT.ORAL.COV.mod",package = "amp.sim"), 
                      paste0(tempdir(),"/example"), overwrite = TRUE,
                      type_return = "mrgsolve",control="string",mod_return = "CP")
mdl <- readLines(paste0(tempdir(),"/example.cpp"))
ret[grepl("mread",ret)] <- "mod <- mread(\"example.cpp\")"


## -----------------------------------------------------------------------------
#| eval: true
#| echo: false
#| class-output: fortranfree
ret <- readLines(system.file("example_models/PK.1CMT.ORAL.COV.mod",package = "amp.sim"))
cat(ret, sep="\n")


## -----------------------------------------------------------------------------
#| eval: false
# library(amp.sim)
# convert_nonmem(system.file("example_models/PK.1CMT.ORAL.COV.mod",package = "amp.sim"),
#                "example", type_return = "mrgsolve", mod_return = "CP")

## -----------------------------------------------------------------------------
#| eval: true
#| echo: false
#| class-output: cpp
#cat(ret,sep="\n")
cat(mdl,sep="\n")


## -----------------------------------------------------------------------------
#| echo: false
#| fig-width: 6
#| message: false
library(mrgsolve)
library(ggplot2)
mod  <- mread(paste0(tempdir(),"/example.cpp"))
evnt <- ev(amt = 100, ii = 24, addl = 1)
out  <- mod |> ev(evnt) |> mrgsim(end = 48, delta = 0.1, nid=10)
ggplot(out@data,aes(time,CP)) + geom_line() + scale_y_log10() + theme_bw()


## -----------------------------------------------------------------------------
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
library(ggplot2)
parm <- c(WEIGHT = 70)               
mod  <- param(mod,parm)   
out  <- zero_re(mod) |> ev(evnt) |> mrgsim(end = 48, delta = 0.1, nid=1)
ggplot(out@data,aes(time,CP)) + geom_line() + scale_y_log10() + theme_bw()


## -----------------------------------------------------------------------------
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
parm <- c(WEIGHT = 70)               
mod  <- param(mod,parm)   
out  <- mod |> ev(evnt) |> mrgsim(end = 48, delta = 0.1, nid=100)
ggplot(out@data,aes(time,CP, group=ID)) + geom_line(alpha=0.3) + 
  scale_y_log10() + theme_bw()


## -----------------------------------------------------------------------------
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false

library(dplyr)
parm  <- c(WEIGHT = 70)
mod   <- param(mod,parm)   

# sample from uncertainty and apply simulations
extf  <- system.file("example_models/PK.1CMT.ORAL.COV.ext",package = "amp.sim")
covf  <- system.file("example_models/PK.1CMT.ORAL.COV.cov",package = "amp.sim")
parms <- sample_par(ext = extf, covmat = covf, nrepl = 100, uncert = TRUE) |> 
  rename_with(~ sub("S", "", .x))

dosim <- function(df){
  zero_re(mod) |> ev(evnt) |>
    param(c(unlist(df),parm)) |> 
    mrgsim_df(end = 48, delta = 0.1, nid=1) |>
    mutate(ID = unique(df$ID))  
}

out <- parms |> group_by(ID) |>
  group_map(~dosim(.x),.keep=TRUE) |> bind_rows()

ggplot(out,aes(time,CP,group=ID)) + geom_line(alpha=.3) + 
  scale_y_log10() + theme_bw()


## -----------------------------------------------------------------------------
#| eval: false
# parm  <- c(WEIGHT = 70)
# extf  <- system.file("example_models/PK.1CMT.ORAL.COV.ext",package = "amp.sim")
# covf  <- system.file("example_models/PK.1CMT.ORAL.COV.cov",package = "amp.sim")
# 
# parms <- sample_par(ext = extf, covmat = covf, nrepl=10, uncert=TRUE)
# parms <- setNames(parms,sub("S","",names(parms)))
# 
# library(parallel)
# cl  <- makeCluster(getOption("cl.cores", 7))
# clusterExport(cl, c("evnt","parms","mod","parm"))
# out <- parLapply(cl, 1:nrow(parms),function(x){
#   library(mrgsolve)
#   ret     <- mod |> ev(evnt) |> param(c(unlist(parms[x,]),parm)) |>
#                mrgsim_df(end = 48, delta = 0.1, nid=10)
#   ret$REP <- unique(parms$ID[x])
#   ret
# })
# stopCluster(cl)
# 
# out <- do.call(rbind,out)
# ggplot(out,aes(time,CP,group=ID)) + geom_line(alpha=.3) +
#   scale_y_log10() + facet_wrap("REP",labeller = "label_both") +
#   theme_bw()
# 


## -----------------------------------------------------------------------------
#| echo: false
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
# Perform actual sims without parallel, because do not want to compile it (on CRAN) with this package
parm  <- c(WEIGHT = 70)
parms <- sample_par(ext    = system.file("example_models/PK.1CMT.ORAL.COV.ext",package = "amp.sim"),
                    covmat = system.file("example_models/PK.1CMT.ORAL.COV.cov",package = "amp.sim"),
                    nrepl=10, uncert=TRUE)
parms <- setNames(parms,sub("S","",names(parms)))
out <- lapply(1:nrow(parms),function(x){
  ret     <- mod |> ev(evnt) |> param(c(unlist(parms[x,]),parm)) |>
               mrgsim_df(end = 48, delta = 0.1, nid=10)
  ret$REP <- unique(parms$ID[x])
  ret
})
out <- do.call(rbind,out)
ggplot(out,aes(time,CP,group=ID)) + geom_line(alpha=.3) + 
  scale_y_log10() + facet_wrap("REP",labeller = "label_both") + 
  theme_bw()



## -----------------------------------------------------------------------------
tmpl_model()


## -----------------------------------------------------------------------------
#| eval: false
# tmpl_model("ana1CMTivC.tmp")


## -----------------------------------------------------------------------------
#| echo: true
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
library(ggplot2)
ana1CMTiv <- function(Dose,pars,t,dur){
  C <- 1/pars['V']
  L <- pars['CL']/pars['V']
  ifelse(t <= dur,
    (Dose/dur) * C / L * (1-exp(-L * t)),
    (Dose/dur) * C / L * (1-exp(-L * dur)) * exp(-L * (t-dur))
  )
}
pars   <- c(CL=1,V=1)
times  <- seq(0,24,length.out=200)
out    <- mdose(Dose = 10, tau = 24, ndose = 1, t = times,
                func = ana1CMTiv, pars = pars, dur = 2)
ggplot(out,aes(time,y)) + geom_line() + theme_bw()



## -----------------------------------------------------------------------------
#| eval: false
# samp <- data.frame(CL=rnorm(100,10,1),V=rnorm(100,5,0.5))
# out <- lapply(1:nrow(samp), function(x){
#   data.frame(mdose(Dose=10,tau=24,ndose=1,t=times,func=ana1CMTiv,
#                    pars=unlist(samp[x,]),dur=2),ID=x)
# })
# out <- do.call(rbind,out)

## ----eval=FALSE---------------------------------------------------------------
# library(amp.sim)
# mod2shiny(parvector = c(THETA1=0.1,THETA2=0.3),
#           modfile   = 'model.cpp',
#           evnt      = ev(amt = 100, ii = 24, addl = 1),
#           naming    = c(THETA1 = "KA (1/h)", THETA2 = "CL (l/h)"),
#           framework = "mrgsolve"
#           outloc    = "simApp")


## ----echo=FALSE, out.width="100%"---------------------------------------------
#knitr::include_graphics("./shinyscreenshot.PNG")
#knitr::include_graphics(paste0(getwd(),"/shinyscreenshot.PNG"))
#knitr::include_graphics(file.path("..","man", "figures", "shinyscreenshot.PNG"))
#knitr::include_graphics(system.file("shinyscreenshot.PNG",package = "amp.sim"))


## ----eval=FALSE---------------------------------------------------------------
# # part added in ui.r
# numericInput(inputId = 'DOSE', label='Dose (mg):', value=100)
# # part added in server.r
# events <- ev(amt = input$DOSE, ii = 24, addl = 1)

## ----eval=FALSE---------------------------------------------------------------
# # residual error and bio-availability can be added in the des_func e.g.
# CP  <- (A2/V) * F1 * (1 + rnorm(length(A2),0,sqrt(0.01)))
# # lag time should be adapted in the events data frame e.g.
# events      <- dose_func(cmt=1,value=300,ndose=3,tau=24)
# events$time <- events$time + pars['TLAG']
# # infusions should be handled in the event and are supported in dose_func
# # see template models how this can be adapted in the model part (?tmpl_model)
# dose_func(cmt=4,value=300,ndose=3,tau=24, tinf=2)

## -----------------------------------------------------------------------------
library(amp.sim)
simd <- simdata(time = seq(0,120,1), dosetime = 0, doseheight = 100, 
                addl = 2, ii = 24, numid = 10)


## -----------------------------------------------------------------------------
#| eval: false
# # lapply can be used for multiple repetitions
# simd <- lapply(seq(300,600,1200),function(x) {
#   simdata(time = seq(0,120,1), dosetime = 0, doseheight = x,
#           addl = 2, ii = 24, numid = 10)
# })
# simd <- do.call(rbind,simd)
# # .. or rbind to do a few repetitions
# simd <- rbind(cbind(simd,CMT=1),cbind(simd,CMT=2),cbind(simd,CMT=3))


## -----------------------------------------------------------------------------
#| eval: false
# simd$WEIGHT <- 70
# simd$TRT    <- sample(1:3,10,replace=TRUE)
# simd$AGE    <- as.integer(rnorm(10,39,5))


## -----------------------------------------------------------------------------
#| eval: false
# # Simple sampling for 10 subjects with the same THETAs but different ETAs
# samp <- sample_par("run1.ext",inc_eta=TRUE,nrepl=10)
# 
# # We could include sampling with uncertainty and omit ETA values
# # samp <- sample_par("run2.ext","run2.cov",inc_eta=FALSE,nrepl=10,uncert = TRUE)
# 
# # Or have uncertainty and variability
# # samp <- sample_par("run2.ext","run2.cov",inc_eta=TRUE,nrepl=10,uncert = FALSE)
# 
# # Or use the result from a bootstrap to sample uncertainty
# # samp <- sample_par("run2.ext",bootstrap=".", uncert = TRUE)
# 
# # In case of clinical trial simulations where a combination of replicates and subjects
# # should be used, the sample_sim function can be used
# # samp <- sample_sim(ext="run2.ext",cov="run2.cov", type="unc_sameIIV", nrepl=10, nsub=20)


## -----------------------------------------------------------------------------
#| eval: false
# inp    <- merge(simd,samp)
# inp    <- inp[order(inp$ID,inp$TIME),]
# write.csv(inp,csv="sim.input.csv",row.names = FALSE, na = ".")


## -----------------------------------------------------------------------------
#| eval: true
#| echo: false
library(amp.sim)
simd <- simdata(time = seq(0,120,1), dosetime = 0,
                doseheight = 100, addl = 2, ii = 24, numid = 10)
simd$WEIGHT  <- 70
simd$AGE     <- as.integer(rnorm(10,39,5))
simd$STHETA1 <- 0.234
simd$STHETA2 <- 1.25
simd$STHETA3 <- 30.1
simd$SETA1   <- round(rnorm(nrow(simd),sd=0.2),3)
simd$SETA2   <- round(rnorm(nrow(simd),sd=0.1),3)

## -----------------------------------------------------------------------------
#| eval: true
head(simd)


## -----------------------------------------------------------------------------
#| eval: false
# make_nmsimmodel("run1.mod", smod = "sim1.mod", data = "sim.input.csv")


## -----------------------------------------------------------------------------
#| eval: false
# split_sim(data    = "sim.input.csv",
#           model   = "run2sim_final.mod",
#           locout  = "simulation",
#           splitby = "DOSE",
#           numout  = 3)


## -----------------------------------------------------------------------------
#| eval: false
# write.csv(head(inp),csv="sim.input.csv",row.names = FALSE, na = ".")
# make_nmsimmodel("run1.mod", smod = "sim1.mod", data = "sim.input.csv")
# 
# split_sim(data    = inp,
#           model   = "sim1.mod",
#           locout  = "simulation",
#           splitby = "DOSE",
#           numout  = 3)


## -----------------------------------------------------------------------------
#| eval: false
# # NOTE: THIS IS EXAMPLE CODE; run_models IS NOT A FUNCTION AVAILABLE IN THE PACKAGE
# mods <- list.files("simulation",pattern="\\.mod$",full.names = TRUE)
# run_models(mods)

#------------------------------------------ split_sim ------------------------------------------
#' Split a large NONMEM simulation in chunks
#'
#' This function splits a large simulation dataset and accompanying model in chunks
#'  to prevent memory problems within the simulation or to enable simulating over multiple cores
#'
#' @param data character string for input dataset or a dataframe with the simulation data (see details)
#' @param model character string for the simulation model
#' @param locout character string for the location of the split input dataset and models
#' @param splitby character string with the variable in the data to be split on (if this is not ID it could lead to unexpected results)
#' @param numout the number of 'equal length' outputs to be created.
#'
#' @details In general the data can be a character string defining the input dataset (csv file) or a dataframe
#'  it is proposed to use a dataframe as in many cases the splitting should take place for a large problem. In case
#'  a dataframe is used (e.g. from loading an Rdata object)  it is not necessary to import a huge csv file.
#'
#' @export
#' @return split CSV and models files are written to disk
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod   <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' dat     <- simdata(0:24, dosetime = 0, doseheight = 10, addl = 2, ii = 24, 
#'                    numid = 50, STHETA1= 1, STHETA2 = 2, STHETA3 = 1,
#'                    SETA1 = 0, SETA2 = 0)
#' tmp_out <- tempfile(fileext = ".csv")
#' mod_out <- tempfile(fileext = ".mod")
#' 
#' write.csv(dat,file=tmp_out, na=".", quote=FALSE, row.names = FALSE)
#' make_nmsimmodel(nmmod, mod_out, data=tmp_out)
#' 
#' split_sim(data = tmp_out, model = mod_out, locout=tempdir())
#' list.files(tempdir(), pattern="\\.mod$")
#' 
split_sim <- function(data,model,locout,splitby="ID",numout=4){

  # Split the input data
  if(!inherits(data,"data.frame")){
    inp           <- utils::read.csv(data,stringsAsFactors = FALSE, na.strings = ".")
    names(inp)[1] <- sub("^X\\.","",names(inp)[1]) # In case a comment char is used #
    nam <- sub("\\.csv$","",basename(data))
  }else{
    inp <- data
    nam <- deparse(substitute(data))
  }
  inps          <- split(inp,cut_equal(inp[,splitby],numout))
  noret         <- lapply(1:length(inps),function(x){
    utils::write.csv(inps[[x]],paste0(locout,"/",nam,".",x,".csv"),row.names=FALSE,quote=FALSE,na=".")
  })

  # Split the simulation model
  mdl       <- readLines(model)
  doll      <- setdiff(c(grep("\\$",mdl),length(mdl)+1),grep(";.*\\$",mdl))
  ddat      <- unlist(lapply(setdiff(grep("\\$DATA",mdl),grep(";.*\\$DATA",mdl)),function(x) x:(min(doll[doll>x]-1))))
  mdl[ddat] <- paste(";",mdl[ddat])
  noret     <- lapply(1:numout,function(x){
    out <- append(mdl,paste("$DATA",paste0(nam,".",x,".csv"),"IGNORE=@"),ddat[1]-1)
    writeLines(out,paste0(locout,"/",sub("\\.mod$","",basename(model)),".",x,".mod"))
  })
}

context("Test if functions that perform NONMEM simulations work as expected")

#----------------------
# Test simdata function
test_that("simdata correctly makes data for a simulation model", {
  res1 <- simdata(0:24,dosetime=0,doseheight=10,addl=2,ii=24,numid=5)
  res2 <- simdata(0:24,dosetime=c(0,2),doseheight=10,addl=NA,ii=NA,rate=-2,numid=2)
  
  expect_length(unique(res1$ID),5)
  expect_true(all(unique(res1$TIME)==0:24))
  expect_true(res1$ADDL[1]==2)
  expect_true(res1$II[1]==24)
  expect_true(all(is.na(res1$DV)))
  expect_true(nrow(res1)==(25*5)+5) # 25 times 5 subjects and 5 dose lines
  
  expect_length(unique(res2$ID),2)
  expect_true(nrow(res2[!is.na(res2$AMT),])==4)
  expect_true(all(res2$TIME[!is.na(res2$AMT)]==c(0,2)))
  expect_true(all(is.na(res2$ADDL)))
  expect_true(all(is.na(res2$II)))
  expect_true(unique(res2$RATE[!is.na(res2$AMT)])==-2)
})

#------------------------------
# Test make_nmsimmodel function
test_that("make_nmsimmodel correctly makes a NONMEM simulation model", {
  dat         <- simdata(0:24,dosetime=0,doseheight=10,addl=2,ii=24,numid=50)
  dat$WEIGHT  <- 70
  dat$SEX     <- 1
  dat$STHETA1 <- dat$STHETA2 <- dat$STHETA3 <- dat$STHETA4 <- dat$STHETA5 <- 1
  dat$SETA1   <- dat$SETA2   <- 0
  write.csv(dat,file=paste0(tempdir(),"/simdat.csv"),na=".",quote=FALSE,row.names = FALSE)
  nmmod       <- system.file("testfiles/nonmem.mod",package="amp.sim")
  suppressWarnings(make_nmsimmodel(nmmod,paste0(tempdir(),"/simmod.mod"), data=paste0(tempdir(),"/simdat.csv")))
  mod         <- readLines(paste0(tempdir(),"/simmod.mod"))
  modsub      <- mod[grep("\\$PK",mod):length(mod)]
  
  expect_true(any(grepl("\\$SIM",mod)))
  expect_true(substr(mod[grepl("\\$EST",mod)],1,1)==";")
  expect_true(substr(mod[grepl("\\$COV",mod)],1,1)==";")
  expect_true(substr(mod[grepl("\\$THETA",mod)],1,1)==";")
  expect_true(grepl("simdat.csv",mod[grepl("^\\$DATA",mod)]))
  
  expect_true(any(grepl("STHETA1",modsub)))
  expect_true(any(grepl("STHETA5",modsub)))
  expect_true(any(grepl("SETA1",modsub)))
})

#------------------------
# Test split_sim function
# TAKE INTO ACCOUNT THAT THE PREVIOUS TEST SHOULD BE DONE BEFORE RUNNING THE CODE BELOW
test_that("split_sim correctly splits the simulation", {
  split_sim(data=paste0(tempdir(),"/simdat.csv"),model=paste0(tempdir(),"/simmod.mod"),
            locout=tempdir())
  nfo  <- file.info(list.files(tempdir(),full.names = TRUE))
  dats <- nfo$size[grepl(paste0("simdat.[[:digit:]].csv"),row.names(nfo))]
  
  expect_true(all(paste0("simmod.",1:4,".mod")%in%list.files(tempdir())))
  expect_true(all(paste0("simdat.",1:4,".csv")%in%list.files(tempdir())))
  expect_true(sd(dats)/mean(dats)<0.1)
})

#------------------------------------------ mdose ------------------------------------------
#' Performs multiple dosing in case of analytical solution of model
#'
#' This function takes a function that defines a model in analytical solution and performs multiple dosing for it
#'
#' @param Dose numeric vector with the dosing height
#' @param tau numeric vector with the tau of dosing
#' @param ndose numeric vector with the number of doses
#' @param t numeric vector with the time-points that should be outputted
#' @param func name of the function for which multiple dosing should be applied
#' @param ... arguments for func
#'
#' @details This function will create a list that can be used to perform superposition
#'   which is necessary in case of an analytical solution in a multiple dose setting.
#'   The function will check if there is an overlap in arguments and will use the arguments
#'   given to \code{mdose} for the function given in \code{func} if applicable (e.g. it is
#'   likely that \code{func} has an argument for Dose, in this case it will use the Dose argument
#'   provide in \code{mdose})
#'   The function can have any number of arguments that can be passed using "...". However there
#'   should at least be an argument \code{t} which is the time vector for which simulations are
#'   necessary.
#'
#' @export
#' @return a data frame with the superimposed results
#' @author Richard Hooijmaijers
#' @examples
#'
#' ana1CMTiv <- function(Dose,pars,t){
#'   Dose * pars['C'] * exp(-pars['L']*t)
#' }
#' res <- mdose(10, tau = 24, ndose = 5, t = 0:240, 
#'              func = ana1CMTiv, pars = c(L=.01,C=5))
#' plot(res$time,res$y, type="l")
#'
mdose <- function(Dose,tau,ndose,t,func,...){
  # Make sure the overlapping arguments are passed to the function
  args1        <- lapply(ls(),function(x) get(x))
  names(args1) <- ls(pattern="[^args1]")
  args2        <- c(args1,list(...))
  args2        <- args2[names(formals(func))]
  args2        <- args2[names(args2)[names(args2)!="t"]]

  # Create time list for superposition, perform function multiple times and output dfrm
  timlst <- lapply(t, function(x) {
    n.tau <- ceiling( x / (sum.tau <- sum( tau)))
    t     <- c( 0, rep( 0:(n.tau-1) * sum.tau, each = length( tau)) + rep( cumsum( tau), n.tau))
    utils::head(x - t[ t <= x],n=ndose)
  })
  y <- sapply(timlst,function(y){  sum(do.call(deparse(substitute(func)),c(args2,t=list(y)))) })
  return(data.frame(time=t,y=y))
}

context("Test miscelaneous functions")

#------------------------
# Test cut_equal function
test_that("cut_equal cuts in general equal bins", {
  expect_equal(length(table(cut_equal(1:101,5))),5)
  expect_equal(round(mean(sapply(gregexpr("\\+",unique(cut_equal(1:101,5))),length)+1)),20)
  expect_equal(round(mean(sapply(gregexpr("\\+",unique(cut_equal(1:143,4,type=2))),length)+1)),36)
  expect_lt(sd(sapply(gregexpr("\\+",unique(cut_equal(1:1134,13))),length)+1),0.5)
})

#------------------------
# Test dose_func function
test_that("dose_func creates valid dosing", {
  dos1 <- dose_func(1,100,times=0)
  dos2 <- dose_func(1,100,ndose=3,tau=24)
  dos3 <- dose_func(2,100,ndose=3,tau=24,tinf=2)
  dos4 <- dose_func(2,100,times=c(0,24,96))
  
  expect_equal(nrow(dos1),1)
  expect_equal(dos1$var,"A1")
  expect_equal(dos1$method,"add")
  expect_equal(dos1$value,100)
  expect_equal(nrow(dos2),3)
  expect_equal(dos2$time,c(0,24,48))
  expect_equal(dos3$time[1:4],c(0,2,24,26))
  expect_equal(dos3$method[1:2],c("add","rep"))
  expect_equal(dos3$value[1:2],c(50,0))
  expect_equal(nrow(dos4),3)
  expect_equal(dos4$time,c(0,24,96))
})

#-------------------------------
# Test model_validation function
test_that("model_validation creates valid output", {
  res  <- model_validation(system.file("testfiles/compareParfile",package="amp.sim"),
                           system.file("testfiles/compareModel.cpp",package="amp.sim"),out=NULL)
  # Omit latex compilation is difficult in covr and CRAN
  # suppressWarnings(model_validation(system.file("testfiles/compareParfile",package="amp.sim"),
  #                                   system.file("testfiles/compareModel.cpp",package="amp.sim"),
  #                                   out=paste0(tempdir(),"/compare.tex")))                           
    
  expect_true("ggplot"%in%class(res$plot_sim_est))
  expect_true("ggplot"%in%class(res$hist_diff))
  expect_true("ggplot"%in%class(res$hist_reldiff))
  expect_true("data.frame"%in%class(res$summary))
  expect_true("data.frame"%in%class(res$alldif))
  
  expect_true(abs(res$summary$mean[2])<0.001)
  expect_true(abs(mean(as.numeric(res$alldif$reldiff)) - res$summary$mean[2])<1e-5) # small differences due to rounding can occur
  #expect_true(file.exists(paste0(tempdir(),"/compare.tex")))
  #expect_true(file.exists(paste0(tempdir(),"/compare.pdf")))
})

#-------------------------
# Test sample_par function
test_that("sample_par samples correctly", {
  
  extf   <- system.file("testfiles/ext_sampling.ext",package="amp.sim")
  covf   <- system.file("testfiles/covariance_sampling.cov",package="amp.sim")
  bsd    <- system.file("testfiles/bootstrap_testing",package="amp.sim")
  samp1  <- suppressWarnings(sample_par(extf,covf,inc_eta=TRUE,uncert=TRUE, seed=1234))
  samp2  <- suppressWarnings(sample_par(extf,covf,inc_eta=TRUE,uncert=TRUE, seed=1234))
  samp3  <- suppressWarnings(sample_par(extf,covf,inc_eta=TRUE,inc_theta = FALSE, uncert = FALSE, seed=1234))
  samp4  <- suppressWarnings(sample_par(extf,covf,restheta = "THETA3",uncert = TRUE))
  samp5  <- suppressWarnings(sample_par(extf,bootstrap = bsd,uncert = TRUE))
  
  mancov <- matrix(c(3.97955E-02,-3.61431E-02,-3.61431E-02,3.46211E-01),nrow=2)
  set.seed(1234)
  omsamp <- data.frame(MASS::mvrnorm(10,c(0,0),Sigma = mancov))
  rext   <- read.table(extf,skip=1,header=TRUE)
  rcov   <- read.table(covf,skip=1,header=TRUE)
  set.seed(1234)
  thsamp <- data.frame(MASS::mvrnorm(10,mu = unlist(rext[rext$ITERATION==-1e9,-c(1,11)]),Sigma = rcov[,-1]))
  allbs  <- list.files("bootstrap_testing",pattern="\\.ext$",full.names=TRUE)
  allbs  <- do.call(rbind,lapply(allbs,read.table,skip=1,header=TRUE))
  allbs  <- allbs[allbs$ITERATION==-1e9,]
  
  expect_mapequal(samp1,samp2)                      # reproducible through seed
  expect_setequal(samp3$SETA1,omsamp$X1)            # omega sampling same as manual
  expect_setequal(samp3$SETA2,omsamp$X2)            # omega sampling same as manual
  expect_true(length(unique(samp4$STHETA3))==1)     # fixing theta for sigma
  expect_setequal(round(samp1$STHETA1,10),round(thsamp$THETA1,10))      # theta sampling same as manual
  expect_setequal(round(samp1$STHETA3,10),round(thsamp$THETA3,10))      # theta sampling same as manual
  expect_setequal(round(samp1$STHETA5,10),round(thsamp$THETA5,10))      # theta sampling same as manual
  expect_true(all(allbs$THETA2%in%samp5$STHETA2))   # theta sampling same as manual bootstrap
  expect_true(all(allbs$THETA4%in%samp5$STHETA4))   # theta sampling same as manual bootstrap
  expect_warning(sample_par(extf,covf,uncert=TRUE, seed=123)) # warning regarding negative values
})

#-------------------------
# Test sample_sim function
test_that("sample_sim samples correctly", {
  extf   <- system.file("testfiles/ext_sampling.ext",package="amp.sim")
  covf   <- system.file("testfiles/covariance_sampling.cov",package="amp.sim")
  samp1  <- suppressWarnings(sample_sim(ext=extf,covmat=covf,type="noIIV",nrepl=3,nsub=4))
  samp2  <- suppressWarnings(sample_sim(ext=extf,covmat=covf,type="sameIIV",nrepl=3,nsub=4))
  samp3  <- suppressWarnings(sample_sim(ext=extf,covmat=covf,type="varIIV",nrepl=3,nsub=4))
  samp4  <- suppressWarnings(sample_sim(ext=extf,covmat=covf,type="unc_noIIV",nrepl=2,nsub=3))
  samp5  <- suppressWarnings(sample_sim(ext=extf,covmat=covf,type="unc_sameIIV",nrepl=2,nsub=3))
  samp6  <- suppressWarnings(sample_sim(ext=extf,covmat=covf,type="unc_varIIV",nrepl=2,nsub=3))
  
  expect_true(length(unique(samp1$REP))==3)              # correct no of subjects and replicates
  expect_true(nrow(samp1)==3*4)                          # correct no of subjects and replicates
  expect_true(length(unique(samp1$STHETA1))==1)          # no uncertainty
  expect_named(samp1,c("REP","ID",paste0("STHETA",1:5))) # only REP, ID and THETAs for noIIV
  expect_true(length(unique(samp2$SETA1))==4)            # same ETA for sameIIV setting
  expect_true(length(unique(samp2$STHETA3))==1)          # no uncertainty for sameIIV setting
  expect_true(length(unique(samp3$SETA1))==12)           # same ETA for varIIV setting
  expect_true(length(unique(samp3$STHETA4))==1)          # no uncertainty for varIIV setting
  expect_named(samp4,c("REP","ID",paste0("STHETA",1:5))) # only REP, ID and THETAs for unc_noIIV
  expect_true(length(unique(samp4$STHETA5))==2)          # uncertainty for unc_noIIV setting (diff sample per replicate)
  expect_true(length(unique(samp5$SETA1))==3)            # same ETA for unc_sameIIV setting
  expect_true(length(unique(samp5$STHETA5))==2)          # uncertainty for unc_sameIIV setting (diff sample per replicate)
  expect_true(length(unique(samp6$SETA2))==3*2)          # diff ETA for unc_varIIV setting
  expect_true(length(unique(samp6$STHETA5))==2)          # uncertainty for unc_varIIV setting (diff sample per replicate)
})

#-------------------------
# Test tmpl_model function
test_that("tmpl_model gets correct template models", {
  tfil  <- tmpl_model()
  afil  <- list.files(system.file(package="amp.sim"),pattern="\\.tmp$")
  mod1  <- tmpl_model("ana1CMTbolusC.tmp", ret="string")
  mod1m <- readLines(system.file("ana1CMTbolusC.tmp",package="amp.sim"))
  expect_setequal(tfil,afil)
  expect_setequal(mod1,mod1m)  
})

#--------------------
# Test mdose function
test_that("mdose performs correct superposition", {
  simmod <- function(Dose,pars,t) Dose * pars['C'] * exp(-pars['L']*t)
  pars   <- c(L=.01,C=5)
  res    <- mdose(10,tau=24,ndose=5,t=0:240,func=simmod, pars=pars)
  
  res2    <- lapply(1:5,function(x){data.frame(time=(0:240)+((x-1)*24),y=simmod(10,pars,0:240))})
  res2    <- suppressWarnings(Reduce(function(x, y) merge(x, y, all=TRUE,by="time"), res2))
  res2$y  <- rowSums(res2[,-1],na.rm = TRUE)
  res2    <- res2[res2$time<=240,c("time","y")]
  
  expect_mapequal(res,res2) 
})

#------------------------------------------ make_nmsimmodel ------------------------------------------
#' Create a simulation model from original model
#'
#' This function creates a simulation model from an original NONMEM model where dollar blocks are replaced and/or
#'  commented based on input file and other simulation functions within package
#'
#' @param omod character string for the original model
#' @param smod character string for the simulation model (for writing the model to disk)
#' @param data character string for the input dataset for the simulation
#' @param subprobs numeric indicating the number of subproblems in simulation model
#' @param table character of length 1 with the items for the dollar table block (if null it will use items in input dataset)
#' @param sigma_ext character with the name of the ext file which includes the sigma value (in case residual error is coded in the dollar sigma block)
#'
#' @details The function will adapt an original model in such a way that it can be used directly for simulations
#'   the assumptions are that an input dataset is created using [sample_par] and [simdata].
#'   This means that all THETA and ETA values are available in the simulation dataset as respectively STHETAN and SETAN (simulated THETA/ETA n).
#'   Furthermore it is assumed that the input dataset is a csv file.
#'   The function will first comment all applicable dollar blocks. Then a OMEGA is added with 0 FIX and the INPUT, DATA and
#'   TABLE blocks are appended with information based on the input dataset. Finally all THETAs and ETAs are replaced with
#'   the items in the dataset and the simulation model is written to  disk.
#'
#' @export
#' @return a simulation model (file) is created
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod   <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' dat     <- simdata(0:24, dosetime = 0, doseheight = 10, addl = 2, ii = 24, 
#'                    numid = 50, STHETA1= 1, STHETA2 = 2, STHETA3 = 1,
#'                    SETA1 = 0, SETA2 = 0)
#' tmp_out <- tempfile(fileext = ".csv")
#' mod_out <- tempfile(fileext = ".mod")
#' 
#' write.csv(dat,file=tmp_out, na=".", quote=FALSE, row.names = FALSE)
#' make_nmsimmodel(nmmod, mod_out, data=tmp_out)
#' readLines(mod_out) |> head(n=15) |> cat(sep="\n")
#' 
make_nmsimmodel <- function(omod,smod,data,subprobs=1,table=NULL,sigma_ext=NULL){

  # get model and indices of dollar blocks
  omod  <- readLines(omod)
  dblck <- get_nmblock(omod,c("INPUT","DATA","THETA","OMEGA","EST","COV","TABLE","SIGMA"),ret="index")

  # comment dollar blocks
  out   <- omod
  out[unname(unlist(dblck[names(dblck)!="SIGMA"]))] <- paste(";",out[unname(unlist(dblck[names(dblck)!="SIGMA"]))])
  if(!is.null(sigma_ext)) out[dblck$SIGMA] <- paste(";",out[dblck$SIGMA])
  out   <- append(out,"$OMEGA 0 FIX",dblck$OMEGA[1]-1)

  # correct $INPUT, $DATA and $TABLE
  dnames <- utils::read.csv(data,header=FALSE,nrow=1,stringsAsFactors = FALSE)
  out    <- append(out,paste("$INPUT\n",sub("#","",paste(dnames,collapse=" "))),dblck$INPUT[1]-1)
  out    <- append(out,paste("$DATA",basename(data),"IGNORE=@"),dblck$INPUT[1]-1)
  out    <- append(out,paste0("$SIM (123) SUBPROBLEMS=",subprobs," ONLYSIM"),length(out))
  tbln   <- ifelse(!is.null(table),table,sub("#","",paste(dnames,collapse=" ")))
  out    <- append(out,paste("$TABLE\n",tbln,"\nNOPRINT NOAPPEND FILE=par"),length(out))

  # replace THETA/ETA with simulated ones in input and write model
  thnum  <- as.numeric(sub("STHETA","",dnames[grep("STHETA",dnames)]))
  etnum  <- as.numeric(sub("SETA","",dnames[grep("SETA",dnames)]))
  for(i in thnum) out <- gsub(paste0("THETA\\(",i,"\\)"),paste0("STHETA",i),out)
  for(i in etnum) out <- gsub(paste0("ETA\\(",i,"\\)"),paste0("SETA",i),out)

  # Adapt sigma if indicated
  if(!is.null(sigma_ext)){
    ext  <- utils::read.table(sigma_ext,skip=1,header=TRUE)
    ext  <- ext[ext$ITERATION==-1000000000,grep("SIGMA",names(ext)),drop=FALSE]
    out  <- append(out,paste0("$SIGMA\n",paste(unlist(ext),collapse="\n")),grep("\\$SIGMA",out)[1])
  }
  message("Check residual error; in case coded as sigma use 'sigma_ext' otherwise check if the correct THETA is used icw uncertainty")
  writeLines(out,smod)
}

#------------------------------------------ dput2 ------------------------------------------
#' Wrapper function around dput to provide more options
#'
#' This function wraps around the dput function and provide more options to save result
#' in a vector
#'
#' @param x An object passed to \code{dput}
#' @param comment logical indicating if comment characters should be prepended to results
#' @param obj character of length one indicating the name of the object that should be prepended to result
#' @param collapse character of length one with the collapse character. If provided the result will be pasted with this collapse character
#' @param ... additional arguments passed to dput
#'
#' @export
#' @return a vector with the result from dput
#' @author Richard Hooijmaijers
#' @examples
#'
#' dput2(setNames(runif(5),letters[1:5]))
#'
dput2 <- function(x,comment=FALSE,obj=NULL,collapse=NULL,...){
  ret <- utils::capture.output(dput(x,...))
  if(!is.null(obj)) ret <- c(paste(obj,"<-",ret[1]),ret[-1])
  if(comment) ret <- paste("#", ret)
  if(!is.null(collapse)) ret <- paste(ret,collapse=collapse)
  return(ret)
}
#------------------------------------------ pos_clpar ------------------------------------------
#' Get position of closing parenthesis
#'
#' This function gets the position of the closing parenthesis after the first opening one
#'
#' @param x character string for which the parenthesis should be searched
#'
#' @export
#' @return a numeric with the position of the closing parenthesis (will be -1 if no match is present)
#' @author Richard Hooijmaijers
#' @examples
#'
#' tst <- "IF (test == A(1)) a(1)=(1*5)/2"
#' pos_clpar(tst)
#' substring(tst,1,pos_clpar(tst))
#'
pos_clpar <- function(x){
  if(grepl("'|\"",x)) stop("character comparison not supported")  
  ob  <- gregexpr("\\(",x)[[1]] 
  cb  <- gregexpr("\\)",x)[[1]] 
  names(ob) <- rep("+1",length(ob))
  names(cb) <- rep("-1",length(ob))
  ab  <- sort(c(ob, cb))
  cab <- as.numeric(names(ab)) |> cumsum()
  return(ab[which(cab==0)[1]])
}

#------------------------------------------ get_est ------------------------------------------
#' Get model estimates from NONMEM ext or model file
#'
#' This function gets the estimates (THETA, ETA and OMEGA) from a NONMEM ext or model file to be used
#' within the simulations in R. The model file is included as option as the names of THETAs can be
#' obtained in case this is set as comment in the model file
#'
#' @param from the model or ext file (or data.frame/model text string from results object) to be read in to obtain estimates.
#'  extension or class of object of file determines the actions to be taken
#'
#' @details the function will return a list with theta, eta, omega and naming of theta values. In case a model is used as input, the values represent
#'  the initial values from a model. In case the ext file is used, the final estimates are taken. The eta values
#'  are all set to 0. The omega values are returned as a matrix so it can be used for sampling (e.g. using mvrnorm).
#'  naming of thetas is taken from the model comments in the THETA block or in case an ext file is used naming is
#'  set to THETA1:n.
#'  In case the model is used as input, there are some assumptions within the function on how the model is coded. For the
#'  omega block the value of omega must always be placed on a separate line (e.g. $OMEGA 0.1 is not permitted as 0.1 should be placed
#'  on the next line. Also covariance within the omega block should be placed on the same line separated by spaces (e.g. for a BLOCK(2)
#'  the first line should state variance eta1 and the second line should state covariance eta1, eta2 followed by variance eta2).
#'  For the theta block, it is assumed that in case lower and upper boundaries are available they are separated by commas.
#'
#' @export
#' @return a list with theta, eta and omega values
#' @author Richard Hooijmaijers
#' @examples
#'
#' # get the initial estimates from the model or final estimates from ext file
#' mod  <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' ext  <- system.file("example_models","PK.1CMT.ORAL.ext", package = "amp.sim")
#' get_est(mod)
#' get_est(ext)
#' 
get_est <- function(from){
  if(inherits(from,"data.frame")){
    # In the new structure we can no longer support data frames for ext files 
    lifecycle::deprecate_stop("0.1.0","get_est(from = 'must be a location to ext or model file')")
  }
  if(length(grep("\\.ext$",from))!=0){
    # actions for ext data
    est    <- NMdata::NMreadExt(from)
    theta  <- est[est$par.type=="THETA","value"]
    theta  <- stats::setNames(theta, est[est$par.type=="THETA","parameter"])
    thetan <- names(theta)
    ome    <- est[est$par.type=="OMEGA",] |> NMdata::dt2mat(col.value = "value")
    sigm   <- est[est$par.type=="SIGMA",] |> NMdata::dt2mat(col.value = "value")
  }else{
    # actions for model file (be aware sigma is not mandatory!)
    if(length(from)==1 && file.exists(from))  from <- readLines(from)
    from   <- iconv(from, "latin1", "ASCII", sub="") # make sure there are no non-ASCII characters
    est    <- NMdata::NMreadInits(lines=from, return = "all")
    theta  <- est$pars$init[est$pars$par.type=="THETA"]
    theta  <- stats::setNames(theta, est$pars$parameter[est$pars$par.type=="THETA"])
    thetan <- merge(est$elements[!duplicated(est$elements$linenum),c("linenum","parameter","par.type")],
                    est$lines[est$lines$par.type=="THETA",c("linenum","text.after")])
    thetan <- trimws(thetan[order(thetan$linenum),][thetan$par.type=="THETA","text.after"])
    ome    <- NMdata::dt2mat(est$pars[est$pars$par.type=="OMEGA",], col.value = "init")
    sigm   <- est$pars[est$pars$par.type=="SIGMA",]
    if(nrow(sigm)>0) sigm <- NMdata::dt2mat(sigm, col.value = "init") else sigm   <- NULL
  }
  # create result list
  eta    <- stats::setNames(rep(0,nrow(ome)), paste0("ETA",1:nrow(ome)))
  retlst <- list(THETA = theta, THETAN = thetan, OMEGA = ome, ETA = eta, SIGMA = sigm)
  return(retlst)
}

context("Test if functions that read and convert models work as expected")

#--------------------------
# Test get_nmblock function
test_that("get_nmblock correctly gets all blocks from a NONMEM models", {
  mdl   <- c("$PROB dummy model","$DES\n res = 1 +2","res2= 2+3","$PK variable=1")
  conv1 <- get_nmblock(mdl,"PROB")                  # get single item
  conv2 <- get_nmblock(mdl,"PROBLEM")               # set an alias
  conv3 <- get_nmblock(mdl,c("PROBLEM","DES","PK")) # get multiple blocks
  conv4 <- get_nmblock(mdl,"DES",ret="index")       # get index only
  conv5 <- get_nmblock(mdl,"PK",omitbn=FALSE)       # include dollar block name in content
  
  expect_equal(conv1[[1]],"dummy model")
  expect_equal(conv2[[1]],"dummy model")
  expect_length(conv3, 3)
  expect_named(conv3, c("PROBLEM","DES","PK"))
  expect_equal(conv4[[1]],2:3)
  expect_true(grepl("\\$PK",conv5[[1]]))
})

#--------------------------
# Test nmlistblock function
test_that("nmlistblock correctly lists all different parts of a NONMEM model", {
  mdl   <- c("$PROB dummy model ; comment","$DES\n DADT(1) = -KA * A(1)","$PK","IF(DUMM.EQ.1) KA = 1","A_0(1) = 0")
  lst   <- nmlistblock(get_nmblock(mdl,c("PROBLEM","DES","PK"))) 

  expect_length(lst, 3)
  expect_length(lst$PK, 3)
  expect_named(lst$PK[[1]], c("orig", "type", "LHS", "RHS", "comm", "cntrl", "dupl"))
  
  expect_equal(lst$DES[[1]]$orig,"DADT(1) = -KA * A(1)")
  expect_equal(lst$DES[[1]]$type,"formula")
  expect_equal(lst$DES[[1]]$LHS,"DADT(1)")
  expect_equal(trimws(lst$DES[[1]]$RHS),"-KA * A(1)")
  
  expect_equal(lst$PK[[2]]$type,"control+formula")
  expect_equal(lst$PK[[2]]$cntrl,"IF(DUMM.EQ.1)")
  expect_equal(lst$PK[[3]]$type,"init")
  
  expect_equal(trimws(lst$PROB[[1]]$comm),"comment")
})

#-------------------------------
# Test convert_nmsyntax function
test_that("convert_nmsyntax correctly convert NONMEM specific syntax", {
  mdl   <- c("IF(DUMM.EQ.1.OR.DUMM.GE.3) KA = 1","CL = LOG(2) * EXP(1) + MAX(10)")
  res1  <- convert_nmsyntax(mdl) 
  res2  <- convert_nmsyntax(mdl,type="deSolve") 
  
  expect_equal(res1[1],"if(DUMM == 1 | DUMM >= 3) KA = 1")
  expect_true(grepl("log",res1[2]) & grepl("exp",res1[2]))
  expect_true(grepl("std::max",res1[2]))
})

#-----------------------
# Test conv_pow function
test_that("conv_pow correctly convert the power to syntax used in mrgsolve", {
  res   <- conv_pow("y = par1*(par2/par3)^xy + a - par4**(2/par5) + 3**4") 
  
  expect_true(grepl("pow\\(\\(par2/par3\\), xy\\)",res))
  expect_true(grepl("pow\\(par4, \\(2/par5\\)\\)",res))
  expect_true(grepl("pow\\(3, 4\\)",res))
})

#----------------------
# Test get_est function
test_that("get_est correctly gets the estimate from a model or ext output", {
  extf   <- system.file("testfiles/ext_sampling.ext",package="amp.sim")
  covf   <- system.file("testfiles/covariance_sampling.cov",package="amp.sim")
  mdl    <- system.file("testfiles/nonmem.mod",package="amp.sim")
  extr <- get_est(extf)
  modr <- get_est(mdl)
  
  expect_equal(names(extr), names(modr))
  expect_true(is.matrix(extr$OMEGA))
  expect_true(is.matrix(modr$OMEGA))
  expect_true(is.matrix(extr$SIGMA))
  expect_named(extr$THETA)
  expect_named(extr$ETA)
  expect_equal(round(unname(extr$THETA),2), c(0.09, 2.67, 1.38, 0.41, 1.61))
  expect_equal(unname(extr$ETA), c(0,0))
  expect_equal(round(extr$OMEGA,2), matrix(c(0.04,-0.04,-0.04,0.35),nrow=2))
  expect_equal(unname(modr$THETA), c(0.08, 2, 1, 0.2, 1.2))
  expect_equal(unname(modr$ETA), c(0,0))
  expect_equal(modr$OMEGA, matrix(c(0.2,0.01,0.01,0.1),nrow=2))
  expect_equal(extr$THETAN, paste0("THETA",1:5))
  expect_equal(trimws(modr$THETAN), c("KA (1/h)", "CL (l/h)", "V (l)", "effect of WT", "effect of SEX"))
})

#--------------------------
# Test get_parmvar function
test_that("get_parmvar gets all correct parameter variables from model", {
  mdl   <- c("$DES\n DADT(1) = -KA * A(1)","$PK","IF(DUMM.EQ.1) THEN","KA=1","ELSE","KA=2","ENDIF","A_0(1) = 0",
             "CL = LOG(2) * EXP(1) + MAX(10)","V2=THETA(1)","$THETA","10")
  lst   <- nmlistblock(get_nmblock(mdl,c("DES","PK"))) 
  res1  <- get_parmvar(lst) 
  res2  <- get_parmvar(lst,returnall=TRUE) 
  
  expect_equal(res1,"DUMM")
  expect_equal(res2,c("DUMM","KA","A_0(1)","CL","V2","DADT(1)"))
})

#------------------------
# Test get_param function
test_that("get_param returns the valid parameters and matrices", {
  
  extf   <- system.file("testfiles/ext_sampling.ext",package="amp.sim")
  covf   <- system.file("testfiles/covariance_sampling.cov",package="amp.sim")
  mdl    <- system.file("testfiles/nonmem.mod",package="amp.sim")
  
  lst   <- nmlistblock(get_nmblock(mdl,c("PK","THETA","OMEGA","SIGMA"))) 
  res1  <- get_param(mdl,lst,extf) 
  res2  <- get_param(mdl,lst) 
  res3  <- get_param(mdl,lst,addparam = FALSE) 
  
  expect_true(length(res1)==length(res2) & length(res1)==length(res3))             # length is the same for each setting
  expect_true(all(c("WEIGHT","SEX")%in%names(res1$params)))                        # additional parameter added
  expect_true(all(names(res3$params)%in%paste0("THETA",1:5)))                      # additional parameter not added
  expect_true(res1$params[1]==0.0858609)                                           # THETA value from ext
  expect_true(res2$params[1]==0.08)                                                # THETA value from model
  expect_true(grepl("theta_names.*KA.*CL",paste(res1$theta_names, collapse=" ")))  # creation of names
  expect_true(is.matrix(res1$omega_matrix) & is.matrix(res1$sigma_matrix))
  expect_true(all(trimws(res1$omega_string)==c("0.0397955","-0.0361431 0.346211" )))
  expect_true(res2$sigma_string=="0.1")
})

#------------------------
# Test get_inits function
test_that("get_inits returns the valid initial states", {
  mdl   <- c("$DES\n DADT(1) = -KA * A(1)","DADT(2) = 0","DADT(3) = 0","$PK","A_0(2) = BL","A_0(3) = 100")
  lst   <- nmlistblock(get_nmblock(mdl,c("DES","PK")))
  res   <- get_inits(lst) 
  
  expect_true(length(res)==3) 
  expect_true(as.numeric(res['A1'])==0)
  expect_true(res['A2']=='BL')
  expect_true(as.numeric(res['A3'])==100)
})

#--------------------------
# Test nm2mrgsolve function
test_that("nm2mrgsolve returns the correctly tranformed model", {
  mdl   <- c("$PROB  example","$PK","KA = THETA(1) * EXP(ETA(1))","CL = THETA(2) * EXP(ETA(2))","V  = THETA(3)","S2  = V","K20 = CL/V","F1  = 1",
             "$MODEL","COMP = (ABS)","COMP = (CENTRAL)",
             "$DES","CP = A(2)/V","DADT(1) = - KA*A(1)","DADT(2) =   KA*A(1) - K20*A(2)","$THETA (0,.1) ; KA (1/h)","(0,2) ; CL (l/h)","(0,1) ; V (l)",
             "$OMEGA",".01 ; ETA KA",".02 ; ETA CL","$ERROR","Y = F * (1 + EPS(1))","IPRED = F","$SIGMA",".1 ; Prop. error")
  lst   <- nmlistblock(get_nmblock(mdl,c("DES","PK","OMEGA","SIGMA","PROB","THETA","MODEL","ERROR")))
  res   <- nm2mrgsolve(lst,mdl) 
  
  expect_true(all(c("modname", "problem", "control2mod", "cmt", "pkblock", "init", 
                    "desblock", "predblock", "errorblock", "param", "randstruct", 
                    "sigmablock", "modtype", "mdl_ret", "control")%in%names(res))) 
  
  expect_true(res$modname==".cpp")
  expect_true(res$problem=="example")
  expect_false(res$control2mod)
  expect_true(grepl("CMT.*A1.*A2",res$cmt))
  expect_true(grepl("PK.*KA.*=.*THETA\\(1\\) \\* exp\\(ETA\\(1\\)\\)",res$pkblock))
  expect_true(grepl("DES.*DADT\\(1\\).*=.*-.*KA\\*A\\(1\\)",res$desblock))
  expect_true(res$predblock=="")
  expect_true(grepl("F = A\\(2\\)/S2",res$errorblock))
  expect_true(grepl("Y =  F \\* \\(1 +.*EPS\\(1\\)\\)",res$errorblock))
  expect_true(res$param=="THETA1 = 0.1, THETA2 = 2, THETA3 = 1")
  expect_true(grepl("OMEGA @block.*0.01.*0.*0.02",res$randstruct))
  expect_true(grepl("SIGMA @block.*0.1",res$sigmablock))
  expect_true(res$modtype=="ode")
  expect_true(grep("mrgsolve",res$control)==1 & grep("mread",res$control)==2)
  expect_true(any(grepl("ev\\(",res$control)))
  expect_true(any(grepl("mrgsim\\(",res$control)))
  
  res2 <- nm2mrgsolve(lst,mdl,mod_return = "Y") 
  expect_true(res2$mdl_ret=="$CAPTURE Y")
  res3 <- nm2mrgsolve(lst,mdl,out = "test") 
  expect_true(res3$modname=="test.cpp")
})

#------------------------
# Test nm2rxode2 function
test_that("nm2rxode2 returns the correctly tranformed model", {
  mdl   <- c("$PROB  example","$PK","KA = THETA(1) * EXP(ETA(1))","CL = THETA(2) * EXP(ETA(2))","V  = THETA(3)","S2  = V","K20 = CL/V","F1  = 1",
             "$MODEL","COMP = (ABS)","COMP = (CENTRAL)",
             "$DES","CP = A(2)/V","DADT(1) = - KA*A(1)","DADT(2) =   KA*A(1) - K20*A(2)","$THETA (0,.1) ; KA (1/h)","(0,2) ; CL (l/h)","(0,1) ; V (l)",
             "$OMEGA",".01 ; ETA KA",".02 ; ETA CL","$ERROR","Y = F * (1 + EPS(1))","IPRED = F","$SIGMA",".1 ; Prop. error")
  lst   <- nmlistblock(get_nmblock(mdl,c("DES","PK","OMEGA","SIGMA","PROB","THETA","MODEL","ERROR")))
  res   <- nm2rxode2(lst,mdl) 
  
  expect_true(all(c("modname", "problem", "control2mod", "cmt", "pkblock", "init", 
                    "desblock", "predblock", "errorblock", "param", "randstruct", 
                    "sigmablock", "modtype", "mdl_ret", "control")%in%names(res))) 
  
  expect_true(res$modname==".r")
  expect_true(res$problem=="example")
  expect_false(res$control2mod)
  expect_true(res$cmt=="" & res$predblock=="" & res$param=="" & res$randstruct=="" & res$sigmablock=="" & res$mdl_ret=="")
  expect_true(grepl("KA.*=.*THETA1.*exp\\(ETA1\\)",res$pkblock))
  expect_true(grepl("A1\\(0\\).*=.*0",res$init))
  expect_true(grepl("d/dt\\(A1\\).*=.*-.*KA\\*A1",res$desblock))
  expect_true(grepl("F = A2/S2",res$errorblock))
  expect_true(grepl("Y =  F \\* \\(1 +.*EPS1\\)",res$errorblock))
  expect_true(res$modtype=="ode")
  expect_true(grep("rxode2",res$control)==1 & grep("source",res$control)==2)
  expect_true(any(grepl("c\\(THETA1 = 0.1.*2.*1",res$control)))
  expect_true(any(grepl("ome.*structure.*0.01.*0.*0.*0.02",res$control)))
  expect_true(any(grepl("sigm.*structure.*0.1.*dim",res$control)))
  expect_true(any(grepl("et\\(",res$control)))
  expect_true(any(grepl("rxSolve\\(",res$control)))
  
  res2 <- nm2rxode2(lst,mdl,control = "model") 
  expect_true(res2$control2mod)
  expect_false(any(grepl("rxode2|source",res2$control)))
  res3 <- nm2rxode2(lst,mdl,out = "test") 
  expect_true(res3$modname=="test.r")
})

#------------------------
# Test nm2nonmem2rx function
test_that("nm2nonmem2rx returns the correctly tranformed model", {
  if(require(nonmem2rx)){
    mdl   <- c("$PROB  example","$PK","KA = THETA(1) * EXP(ETA(1))","CL = THETA(2) * EXP(ETA(2))","V  = THETA(3)","S2  = V","K20 = CL/V","F1  = 1",
               "$MODEL","COMP = (ABS)","COMP = (CENTRAL)",
               "$DES","CP = A(2)/V","DADT(1) = - KA*A(1)","DADT(2) =   KA*A(1) - K20*A(2)","$THETA (0,.1) ; KA (1/h)","(0,2) ; CL (l/h)","(0,1) ; V (l)",
               "$OMEGA",".01 ; ETA KA",".02 ; ETA CL","$ERROR","Y = F * (1 + EPS(1))","IPRED = F","$SIGMA",".1 ; Prop. error")
    tdir   <- tempdir()
    writeLines(mdl,paste0(tdir,"/testmodel.mod"))
    #lst   <- nmlistblock(get_nmblock(mdl,c("DES","PK","OMEGA","SIGMA","PROB","THETA","MODEL","ERROR")))
    res   <- nm2nonmem2rx(paste0(tdir,"/testmodel.mod")) 
    
    expect_true(all(c("modelfun","modname","control")%in%names(res))) 
    
    expect_true(res$modname==".r")
    expect_true(grepl("KA.*exp\\(ETA.*\\)",res$modelfun))
    expect_true(grepl("d/dt\\(ABS\\).*<-.*-.*ka",res$modelfun))
    expect_true(grepl("f\\(ABS\\)",res$modelfun))
    expect_true(any(grepl("parm.*<-.*\\(.*\\)",res$control)))
    expect_true(any(grepl("ome.*structure.*0.01.*0.*0.*0.02",res$control)))
    expect_true(any(grepl("sigm.*structure.*0.1.*dim",res$control)))
    expect_true(any(grepl("et\\(",res$control)))
    expect_true(any(grepl("rxSolve\\(",res$control)))
    
    res2 <- nm2nonmem2rx(paste0(tdir,"/testmodel.mod"),control = "model") 
    expect_false(any(grepl("rxode2|source",res2$control)))
    res3 <- nm2nonmem2rx(paste0(tdir,"/testmodel.mod"),out = "test") 
    expect_true(res3$modname=="test.r")
  }
})

#------------------------
# Test nm2deSolve function
test_that("nm2deSolve returns the correctly tranformed model", {
  mdl   <- c("$PROB  example","$PK","KA = THETA(1) * EXP(ETA(1))","CL = THETA(2) * EXP(ETA(2))","V  = THETA(3)","S2  = V","K20 = CL/V","F1  = 1",
             "$MODEL","COMP = (ABS)","COMP = (CENTRAL)",
             "$DES","CP = A(2)/V","DADT(1) = - KA*A(1)","DADT(2) =   KA*A(1) - K20*A(2)","$THETA (0,.1) ; KA (1/h)","(0,2) ; CL (l/h)","(0,1) ; V (l)",
             "$OMEGA",".01 ; ETA KA",".02 ; ETA CL","$ERROR","Y = F * (1 + EPS(1))","IPRED = F","$SIGMA",".1 ; Prop. error")
  lst   <- nmlistblock(get_nmblock(mdl,c("DES","PK","OMEGA","SIGMA","PROB","THETA","MODEL","ERROR")))
  res   <- nm2deSolve(lst,mdl) 
  
  expect_true(all(c("modname", "problem", "control2mod", "cmt", "pkblock", "init", 
                    "desblock", "predblock", "errorblock", "param", "randstruct", 
                    "sigmablock", "modtype", "mdl_ret", "control")%in%names(res))) 
  
  expect_true(res$modname==".r")
  expect_true(res$problem=="example")
  expect_false(res$control2mod)
  expect_true(res$cmt=="" & trimws(res$predblock)=="" & res$randstruct=="" & res$sigmablock=="" & res$errorblock=="")
  expect_true(grepl("KA.*<-.*THETA1.*exp\\(ETA1\\)",res$pkblock))
  expect_true(grepl("DADT1.*<-.*-.*KA\\*A1",res$desblock))
  expect_true(res$modtype=="ode")
  expect_true(all(names(res$init)==c("A1","A2")))
  expect_true(sum(res$init)==0)
  expect_true(all(names(res$param)==c("THETA1","THETA2","THETA3","ETA1","ETA2")))
  expect_true(all(unname(res$param)==c(0.1,2,1,0,0)))
  expect_true(grepl("list.*DADT1.*DADT2",res$mdl_ret))
  
  expect_true(grep("library.*deSolve",res$control)==2 & grep("source",res$control)==4)
  expect_true(any(grepl("c\\(THETA1 = 0.1.*2.*1",res$control)))
  expect_true(any(grepl("THETA1 = 0.1.*ETA2 = 0",res$control)))
  expect_true(any(grepl("A1 = 0.*A2 = 0",res$control)))
  expect_true(any(grepl("dose_func\\(",res$control)))
  expect_true(any(grepl("lsoda\\(",res$control)))
  
  res2 <- nm2deSolve(lst,mdl,control = "model") 
  expect_true(res2$control2mod)
  expect_false(any(grepl("library.*deSolve|source",res2$control)))
  res3 <- nm2deSolve(lst,mdl,out = "test") 
  expect_true(res3$modname=="test.r")
})

#-----------------------------
# Test convert_nonmem function
test_that("convert_nonmem returns the correctly tranformed model", {
  mdl   <- c("$PROB  example","$PK","KA = THETA(1) * EXP(ETA(1))","CL = THETA(2) * EXP(ETA(2))","V  = THETA(3)","S2  = V","K20 = CL/V","F1  = 1",
             "$MODEL","COMP = (ABS)","COMP = (CENTRAL)",
             "$DES","CP = A(2)/V","DADT(1) = - KA*A(1)","DADT(2) =   KA*A(1) - K20*A(2)","$THETA (0,.1) ; KA (1/h)","(0,2) ; CL (l/h)","(0,1) ; V (l)",
             "$OMEGA",".01 ; ETA KA",".02 ; ETA CL","$ERROR","Y = F * (1 + EPS(1))","IPRED = F","$SIGMA",".1 ; Prop. error")
  tdir   <- tempdir()
  writeLines(mdl,paste0(tdir,"/testmodel.mod"))
  
  cntrl1 <- convert_nonmem(paste0(tdir,"/testmodel.mod"),control="string",out=paste0(tdir,"/testmrgsolve"),overwrite=TRUE) 
  mod1   <- readLines(paste0(tdir,"/testmrgsolve.cpp"))
  
  expect_true(length(cntrl1)>0 & length(mod1)>0) 
  expect_true(grep("mrgsolve",cntrl1)[1]==1 & grep("mread",cntrl1)==2)
  expect_true(any(grepl("\\$PLUGIN",mod1))  & any(grepl("\\$DES",mod1)) & any(grepl("\\$PK",mod1)))
  
  cntrl2 <- convert_nonmem(paste0(tdir,"/testmodel.mod"),control="string",out=paste0(tdir,"/testrxode2"),overwrite=TRUE,type_return="rxode2") 
  mod2   <- readLines(paste0(tdir,"/testrxode2.r"))
  
  expect_true(length(cntrl2)>0 & length(mod2)>0) 
  expect_true(grep("rxode2",cntrl2)[1]==1 & grep("source",cntrl2)==2)
  expect_true(any(grepl("model.*rxode2\\(",mod2))  & any(grepl("d/dt\\(A1\\)",mod2)))
  
  cntrl3 <- convert_nonmem(paste0(tdir,"/testmodel.mod"),control="string",out=paste0(tdir,"/testdesolve"),overwrite=TRUE,type_return="deSolve") 
  mod3   <- readLines(paste0(tdir,"/testdesolve.r"))
  
  expect_true(length(cntrl3)>0 & length(mod3)>0) 
  expect_true(grep("deSolve",cntrl3)[1]==2 & grep("source",cntrl3)==4)
  expect_true(any(grepl("model.*function\\(",mod3))  & any(grepl("DADT1",mod3)))
  
  cntrl4 <- convert_nonmem(paste0(tdir,"/testmodel.mod"),control="string",out=paste0(tdir,"/testreturn1"),overwrite=TRUE,mod_return="CP") 
  mod4   <- readLines(paste0(tdir,"/testreturn1.cpp"))
  expect_true(any(grepl("\\$CAPTURE CP",mod4)))
  
  cntrl5 <- convert_nonmem(paste0(tdir,"/testmodel.mod"),control="string",out=paste0(tdir,"/testnm2rx"),
                           overwrite=TRUE,type_return="nonmem2rx") |> suppressMessages()
  mod5   <- readLines(paste0(tdir,"/testnm2rx.r"))
  expect_true(any(grepl("ini\\(",mod5)))
  
  convert_nonmem(paste0(tdir,"/testmodel.mod"),control="file",out=paste0(tdir,"/testcontrol"),overwrite=TRUE, verbose=FALSE) 
  expect_true(file.exists(paste0(tdir,"/testcontrol_control.r")))
  expect_message(convert_nonmem(paste0(tdir,"/testmodel.mod"),control="console",out=paste0(tdir,"/testcontrol"),overwrite=TRUE, verbose=FALSE))
  
  expect_error(convert_nonmem(paste0(tdir,"/testmodel.mod"),type_return="dummy"))
  convert_nonmem(paste0(tdir,"/testmodel.mod"),control="file",out=paste0(tdir,"/testoverwrite"),overwrite=TRUE, verbose=FALSE) 
  expect_warning(convert_nonmem(paste0(tdir,"/testmodel.mod"),control="file",out=paste0(tdir,"/testoverwrite"),overwrite=FALSE, verbose=FALSE) )
  expect_message(convert_nonmem(paste0(tdir,"/testmodel.mod"),control="console",out=paste0(tdir,"/testcontrol"),overwrite=TRUE, verbose=TRUE))
})

#------------------------------------------ nm2deSolve ------------------------------------------
#' Convert NONMEM model to deSolve syntax
#'
#' This function converts a NONMEM model to syntax useable in deSolve simulations
#'
#' @param lstblock structured list with information of the model that was read-in, usually obtained from the [nmlistblock] function
#' @param model character vector with the model content
#' @param ext character with the name of the NONMEM ext file (if not provided estimates are read directly from control stream)
#' @param mod_return a character vector indicating which items should be returned from the model function (see [convert_nonmem] for more details)
#' @param out character with the name of the output file without a file extension
#' @param control character with the type of control to bre returned (see [convert_nonmem] for more details)
#'
#' @noRd
#' @return a list is returned inluding all building blocks to create a model
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' lst   <- get_nmblock(nmmod, block = c("PROB","SUB","MODEL","PK","DES","PRED",
#'                                       "THETA","OMEGA","ERROR","SIGMA","EST"))
#' lst   <- nmlistblock(lst)
#' nm2deSolve(lst, readLines(nmmod))
#'
nm2deSolve <- function(lstblock,model,ext=NULL,mod_return=NULL,out=NULL,control=""){ # ,mod_return=NULL
  # Define a translator function to rewrite specific parts for mrgsolve
  translator     <- function(block){
    sapply(block,function(x){
      if(x$type=="comment"){
        rets <- paste0("#",x$comm)
      }else if(x$type=="empty"|x$type==""){
        rets <- ""
      }else{
        if(grepl("controlflow",x$type)){
          rets <- x$cntrl
          if(x$comm!="") rets <- paste0(rets,"#",x$comm)
        }else{
          #if(grepl("DADT\\(.*\\)",x$orig)) x$LHS <- sub("DADT\\(","d/dt(A",x$LHS)
          #x$RHS <- par_delete(x$RHS)
          rets  <- paste(x$LHS,"=",x$RHS)
          rets <- gsub("="," <- ",rets)
          if(x$type=="init")  rets <- ""
          if(x$comm!="") rets <- paste(rets,"#",x$comm)
          if(x$type=="control+formula") rets <- paste(x$cntrl,rets)
          rets <- par_delete(rets)
        }
        rets <- convert_nmsyntax(rets,type="deSolve")
      }
      rets
    })
  }

  params            <- get_param(model,lstblock,ext=ext,addparam = TRUE) # check if we want the addparam as argument (maybe always true?)
  sclpar            <- params$all_params[grepl("^S\\d+$",params$all_params)]
  adderr            <- ifelse(!"F"%in%params$all_params,"",ifelse(length(sclpar)>0,paste0("F = A",sub("S","",sclpar[1]),"/",sclpar[1],";"),"F = A1;"))

  retlst             <- list()
  retlst$problem     <- lstblock$PROB[[1]]$orig
  retlst$pkblock     <- paste(paste0("    ",translator(lstblock$PK)),collapse="\n")
  retlst$desblock    <- paste(paste0("    ",translator(lstblock$DES)),collapse="\n")
  retlst$predblock   <- paste(paste0("    ",translator(lstblock$PRED)),collapse="\n")
  retlst$param       <- c(params$params,stats::setNames(rep(0,nrow(params$omega_matrix)),paste0("ETA",1:nrow(params$omega_matrix))))
  retlst$init        <- get_inits(lstblock)
  # inits are defined outside model function, meaning that some values are not available in parameter vector. Try to obtain it by evaluating the PK block
  tempenv  <- new.env()
  list2env(as.list(retlst$param), envir = tempenv)
  parsedpk <- try(eval(parse(text=retlst$pkblock),envir = tempenv))
  if("try-error"%in%class(parsedpk)){
    warning("could not parse for retrieval of intial values, check intials before simulating")
  }else{
    for(i in 1:length(retlst$init)) if(is.na(suppressWarnings(as.numeric(retlst$init[i])))) try(retlst$init[i] <- get(retlst$init[i],envir = tempenv))
  }
  retlst$init        <- stats::setNames(as.numeric(retlst$init),names(retlst$init))
  retlst$modname     <- paste0(out,".r")
  retlst$control2mod <- ifelse(control=="model",TRUE,FALSE)
  retlst$mdl_ret     <- paste0("c(",paste0("DADT",1:sum(grepl("DADT[[:digit:]]",translator(lstblock$DES))),collapse=","),")")
  if(!is.null(mod_return)) retlst$mdl_ret <- paste0("    list(",retlst$mdl_ret,",c(",paste(mod_return,mod_return,sep="=",collapse=", "),"))")
  if(is.null(mod_return))  retlst$mdl_ret <- paste0("    list(",retlst$mdl_ret,")")
  retlst$modtype     <- ifelse(trimws(retlst$desblock)!="","ode",ifelse(trimws(retlst$predblock)!="","pred","other"))
  # These parts do not go into the model and can therefore be omitted (keep in list to return uniform list across packages)
  retlst$errorblock  <- retlst$cmt <- retlst$randstruct <- retlst$sigmablock <- ""

  retlst$control <- ifelse(retlst$modtype=="ode","","# MODEL IS NOT DEFINED AS ODE TAKE INTO ACCOUNT THAT THE MODEL AND CONTROL FILE NEED EXTENSIVE ADAPTATIONS TO MAKE IT WORK!")
  retlst$control <- c(retlst$control,"library(deSolve)","library(ggplot2)",paste0("source(\"",out,".r\")"))
  retlst$control <- c(retlst$control, params$theta_names)
  retlst$control <- c(retlst$control, dput2(retlst$param,FALSE,"parm","\n"))
  retlst$control <- c(retlst$control, dput2(retlst$init,FALSE,"init","\n"))
  retlst$control <- c(retlst$control, "times <- seq(0,48,.1)")
  retlst$control <- c(retlst$control,"evnt <- amp.sim::dose_func(cmt=1,value=100,tau=24,ndose=2)")
  retlst$control <- c(retlst$control,"out  <- data.frame(deSolve::lsoda(init,times,model,parm,events=list(data=evnt)))")
  retlst$control <- c(retlst$control,"out  <- tidyr::pivot_longer(out,cols=!contains(\"time\"))")
  retlst$control <- c(retlst$control,"ggplot(out,aes(time,value)) + geom_line() + facet_wrap(\"name\",scales=\"free\")")
  if(control=="model") retlst$control <- retlst$control[-c(2,4)]

  return(retlst)
}

library(testthat)
library(amp.sim)

test_check("amp.sim")

# devtools::install() 
# devtools::load_all() 
# devtools::test() 
# test_dir("tests/testthat/")

# devtools::test_active_file(file = "tests/testthat/test_misc.R")
# devtools::test_active_file(file = "tests/testthat/test_nonmemsim.R")
# devtools::test_active_file(file = "tests/testthat/test_readconvert.R")
# devtools::test_active_file(file = "tests/testthat/test_shinypart.R")
# install.packages("covr",repos = "https://cloud.r-project.org")
# covr::report()
# covr::package_coverage()
# covr::codecov()
# covr::package_coverage(type="tests")
# devtools:::test_coverage()

#------------------------------------------ model_validation ------------------------------------------
#' Validates NONMEM estimation model with an mrgsolve simulation model
#'
#' This function uses the estimates from a NONMEM run and compares this with the results of a simulation
#' run. This function is inspired by a blog post from mrgsolve and mainly looks at the differences in population predictions
#'
#' @param nmtable either a character with a file or a data frame including the NONMEM table output
#' @param simmodel character with the file including the mrgsolve model
#' @param rounding numeric with the rounding applied for comparing
#' @param comppred character with the variable in mrgsolve model that should be compared with PRED variable in NONMEM
#' @param out character with the name of the output to create
#' @param ... additional arguments passed through to \code{mrgsim} function
#'
#' @details For a correct comparison, the nmtable should include all variables related to dosing (e.g. AMT/CMT/EVID).
#'   The simulation model should be available as a separate file that can be read in using \code{mrgsolve::mread}.
#'   To use the function, the packages \code{R3port}, \code{ggplot2}, \code{mrgsolve} and \code{dplyr} should be installed.
#'   Be aware that no variables are renamed in $TABLE in the NONMEM control stream (e.g. AMT2=AMT). This can have unexpected results when comparing.
#'
#' @export
#' @return a file with a PDF report is returned
#' @author Richard Hooijmaijers
#' @examples
#'
#' \donttest{
#'   res  <- model_validation(system.file("testfiles/compareParfile",package="amp.sim"),
#'                            system.file("testfiles/compareModel.cpp",package="amp.sim"),
#'                            out=NULL)
#' }
model_validation <- function(nmtable,simmodel,rounding=4,comppred="CP",out=NULL,...){

  if(length(find.package("ggplot2", quiet = TRUE))==0) stop("the ggplot2 package should be installed to use this function")
  if(length(find.package("R3port", quiet = TRUE))==0) stop("the R3port package should be installed to use this function")
  if(length(find.package("mrgsolve", quiet = TRUE))==0) stop("the mrgsolve package should be installed to use this function")
  if(length(find.package("dplyr", quiet = TRUE))==0) stop("the dplyr package should be installed to use this function")

  # read in nonmem table file
  if(inherits(nmtable,"data.frame")){
    parf <- nmtable
  }else{
    parf <- try(utils::read.table(nmtable,header=TRUE,comment.char = "",skip=1))
    if("try-error"%in%class(parf)) stop("Could not read in NONMEM table file")
  }
  if(!all(c("PRED","ID","TIME","AMT")%in%names(parf))) stop("The variables 'PRED', 'TIME', 'ID' and 'AMT' should at least be available for comparison")

  # read in model and simulate
  mod    <- try(mrgsolve::mread(simmodel))
  if("try-error"%in%class(mod)) stop("Could not read in mrgsolve model")
  simres <- try(mrgsolve::mrgsim_df(mrgsolve::zero_re(mod), data=parf, carry_out = "PRED", digits = rounding,...))
  if("try-error"%in%class(simres)) stop("Could not simulate mrgsolve model")
  if(!comppred%in%names(simres))   stop("Selected comppred is not available in model output, make sure it is captured in mrgsolve model")

  simres$diff    <- round(simres[,comppred],rounding) - round(simres$PRED,rounding)
  simres$reldiff <- 100 * (round(simres[,comppred],rounding) - round(simres$PRED,rounding))/round(simres$PRED,rounding)

  # Create results and write to report if applicable
  simrest1 <- dplyr::rename(simres,"value"="diff") |> dplyr::mutate(variable="Absolute difference") |> dplyr::select(!dplyr::all_of("reldiff"))
  simrest2 <- dplyr::rename(simres,"value"="reldiff") |> dplyr::mutate(variable="Relative difference (%)") |> dplyr::select(!dplyr::all_of("diff"))
  simrest  <- rbind(simrest1,simrest2)
  sumtbl   <- dplyr::summarise(dplyr::group_by(simrest,dplyr::across(dplyr::all_of("variable"))), n = dplyr::n(), min=min(.data$value,na.rm=TRUE),
                               p5=stats::quantile(.data$value,0.05,na.rm=TRUE), median=stats::median(.data$value,na.rm=TRUE),
                               p9=stats::quantile(.data$value,0.95,na.rm=TRUE), max=max(.data$value,na.rm=TRUE),
                               mean=mean(.data$value,na.rm=TRUE), sd=stats::sd(.data$value,na.rm=TRUE))

  top10            <- simres[!is.na(simres$reldiff) & simres$reldiff!=0,]
  top10$absreldiff <- formatC(abs(top10$reldiff),2,format="fg",flag="#")
  top10$reldiff    <- formatC(top10$reldiff,2,format="fg",flag="#")
  top10$diff       <- formatC(top10$diff,2,format="fg",flag="#")

  pl1 <- ggplot2::ggplot(simres,ggplot2::aes(.data[["PRED"]],.data[[comppred]])) + ggplot2::geom_point(alpha=.2) + ggplot2::geom_abline(slope=1,intercept = 0) +
    ggplot2::labs(x="Results estimation",y="Results simulation") + ggplot2::theme_bw(base_size = 9)
  pl2a <- ggplot2::ggplot(simres,ggplot2::aes(.data$diff)) + ggplot2::geom_histogram() +
    ggplot2::labs(x="Difference bewteen estimation and simulation") + ggplot2::theme_bw(base_size = 9)
  pl2b <- ggplot2::ggplot(simres,ggplot2::aes(.data$reldiff)) + ggplot2::geom_histogram() +
    ggplot2::labs(x="Relative difference bewteen estimation and simulation (%)") + ggplot2::theme_bw(base_size = 9)

  if(!is.null(out)){
    dir.create(dirname(out),showWarnings = FALSE)
    writeLines(paste("\\section{Results}","This report shows the results for comparing the NONME results and the results obtained",
                     "after simulations by \\texttt{mrgsolve} for model ",simmodel,".\\\\ \\listoftables \\listoffigures\n",sep="\n"),
               paste0(dirname(out),"/01.res.tex.rawtex"))
    R3port::ltx_list(sumtbl,out = paste0(dirname(out),"/02.res.tex"), show=FALSE,
                     title="Summary statistics of the differences between estimation and simultion model")
    R3port::ltx_list(utils::head(top10[order(top10$absreldiff,decreasing = TRUE),],10),porder=FALSE,xrepeat=TRUE, show=FALSE,
                     out = paste0(dirname(out),"/03.res.tex"),title="Top 10 highest relative differences between estimation and simultion model")
    R3port::ltx_plot(pl1,out = paste0(dirname(out),"/04.res.tex"),pwidth = 7, show=FALSE,
                     title="Graphical comparison between estimation and simultion model")
    R3port::ltx_plot(list(pl2a,pl2b,ncol=1),out = paste0(dirname(out),"/05.res.tex"),orientation="portrait",linebreak=FALSE, show=FALSE,
                     pwidth = 7,pheight=4, title="Graphical difference between estimation and simultion model")
    R3port::ltx_combine(dirname(out),out=basename(out),orientation='portrait',clean=2, show=FALSE)
  }else{
    return(list(summary=sumtbl,alldif=top10,plot_sim_est=pl1,hist_diff=pl2a,hist_reldiff=pl2b))
  }
}

#------------------------------------------ par_delete ------------------------------------------
#' Delete parenthesis including a numeric within a vector
#'
#' This function deletes all parenthesis from a character vector that has a numeric value inside.
#' This is applicable as THETA(n), ETA(n), DADT(n) is not always allowed in R coding
#'
#' @param vect the vector that should be scanned for parenthesis
#' @param excfun exclude functions from deleting parenthesis (taken from the globally defined function list in the package)
#'
#' @export
#' @return a vector with the stripped parenthesis
#' @author Richard Hooijmaijers
#' @examples
#'
#' par_delete(c("LOG(1)","ETA(1)","EXP(2)/ETA(3)+THETA(4)"))
par_delete <- function(vect,excfun=TRUE){
  avect <- NULL
  for(i in 1:length(vect)){
    numx  <- gregexpr("\\([[:digit:]]*\\)",vect[i])[[1]]
    if(min(numx)<0){
      avect <- c(avect,vect[i])
    }else{
      if(excfun){
        funcs <- get("nmfuncs",envir=.simenv)
        numx2 <- gregexpr(paste(names(funcs),collapse="|"),vect[i])[[1]]
        if(min(numx2)>=0){
          numx2 <- numx2+attr(numx2,"match.length")-1
          tatt  <- attr(numx,"match.length")[!numx%in%numx2]
          numx  <- numx[!numx%in%numx2];attr(numx,"match.length") <- tatt
        }
        #print(numx)
      }
      if(length(numx)==0){
        avect <- c(avect,vect[i])
      }else{
        for(j in length(numx):1){
          vect[i] <- paste0(substr(vect[i],1,numx[j]-1),
                            substr(vect[i],numx[j]+1,numx[j]+attr(numx,"match.length")[j]-2),
                            substr(vect[i],numx[j]+attr(numx,"match.length")[j],nchar(vect[i])))
        }
        avect <- c(avect,vect[i])
      }
    }
  }
  return(avect)
}

#------------------------------------------ sample_par ------------------------------------------
#' Sample model parameters from a multivariate normal distribution
#'
#' This function samples model parameters from NONMEMs covariate matrix (*.cov) and final parameter
#'  estimates (*.ext). Furthermore etas can also be sampled where eta blocks are taken into account (see details)
#'
#' @param ext character string with location of ext file with final model parameters 
#' @param covmat character string with location of cov file with covariance matrix or data.frame of cov file
#' @param bootstrap character string with location of ext files from bootrstrap or vector with ext files from bootstrap
#'    (relevant in case ext files should be excluded because of minimization issues)
#' @param seed a numeric with the seed number used in set.seed to enable reproducibility, when not provided 
#'  the seed from the global environment will be used (e.g. using [base::set.seed])
#' @param nrepl numeric with the number of replicates to sample
#' @param inc_theta logical indicating if THETAs should be added to result
#' @param inc_eta logical indicating if ETAs should be added to result
#' @param verbose logical indicating if additional information should be added to result (e.g. OMEGA/SIGMA values)
#' @param dropfixed logical indicating if parameters that are fixed should be dropped (can only be done in case covmat is provided)
#' @param uncert logical indicating if the uncertainty should be sampled
#' @param restheta character with the theta that describes residual error (e.g. "THETA3") in case uncertainty is sampled
#'   this parameter will be set to the population value
#'
#' @details In general the function can be used to sample from covariance matrix so the different parameters can be
#'   added to a simulation dataset or model to enable uncertainty simulations. In most cases it is not necessary to
#'   include OMEGAs or SIGMAs in these type of simulations. It can be convenient to add ETAs in the simulation dataset
#'   to perform a simulation where no $THETA or $OMEGA information is necessary. In case inc_eta is TRUE, the ETAs
#'   from the ext files are used and placed in a matrix to take into account covariance or
#'   'BLOCKS'. A matrix from the ext file is constructed based on the naming of OMEGA values (e.g. OMEGA.2.1. will be
#'   added to row 2, column 1 and column 1, row 2). The matrix is used as Sigma for the mvrnorm function with mu=0.
#'
#' @seealso [MASS::mvrnorm]
#' @export
#' @return a dataframe with sampled values
#' @author Richard Hooijmaijers
#' @examples
#'
#' ext <- system.file("example_models","PK.1CMT.ORAL.COV.ext", package = "amp.sim")
#' cov <- system.file("example_models","PK.1CMT.ORAL.COV.cov", package = "amp.sim")
#' sample_par(ext, inc_eta = TRUE, nrepl = 5)
#' sample_par(ext, cov, uncert = TRUE, nrepl = 5)
#'
sample_par <- function(ext,covmat=NULL,bootstrap=NULL,seed=NULL,nrepl=10,inc_theta=TRUE,inc_eta=FALSE,verbose=FALSE,dropfixed=FALSE,uncert=FALSE,restheta=NULL){
  # set seed and read data (if applicable)
  if(!is.null(seed)){
    set.seed(seed)
  }

  if(inherits(ext,"character"))    extf   <- utils::read.table(ext,skip=1,header=TRUE) else extf <- ext
  if(inherits(covmat,"character")) covmat <- try(utils::read.table(covmat,skip=1,header=TRUE))

  # sample from covariance matrix or bootstrap (for uncertainty only!)
  extf     <- extf[extf$ITERATION==-1e9,-c(1,ncol(extf))]
  if(uncert){
    if(is.null(bootstrap)){
      if(is.null(covmat)) return("In case of sampling with uncertainty, a covmat should be provided (or bootstrap location)")
      fixcov  <- names(covmat[,apply(covmat,2,function(x) all(x==0))])
      if(nrepl!=1)  sampl   <- data.frame(ID=1:nrepl,MASS::mvrnorm(n = nrepl,mu = unlist(extf),Sigma=covmat[,-1]))
      if(nrepl==1)  sampl   <- data.frame(ID=1:nrepl,as.list(MASS::mvrnorm(n = nrepl,mu = unlist(extf),Sigma=covmat[,-1])))
      for(i in fixcov) sampl[,i] <- extf[,i]
      if(dropfixed) sampl   <- sampl[,setdiff(names(sampl),fixcov)]
      #print(sampl)
    }else{
      if(is.null(bootstrap)) return("In case of sampling with uncertainty, a bootstrap location should be provided (or covmat)")
      bsres <- if(length(bootstrap)==1 && file.info(bootstrap)$isdir) list.files(bootstrap,pattern="\\.ext$",full.names=TRUE) else bootstrap
      bsres <- lapply(bsres,utils::read.table,skip=1,header=TRUE)
      bsres <- lapply(bsres,function(x) x[x$ITERATION==-1e9,])
      bsres <- do.call(rbind,bsres)

      if(nrow(bsres)>=nrepl){
        sampl <- bsres[sample(1:nrow(bsres),nrepl),]
      }else{
        sampl <- bsres[c(1:nrow(bsres),sample(1:nrow(bsres),nrepl-nrow(bsres),replace=TRUE)),]
      }
      sampl <- sampl[,!names(sample)%in%c("ITERATION","OBJ")] 
      sampl <- cbind(ID=1:nrepl,sampl)
    }
    if(!is.null(restheta)) sampl[,restheta] <- extf[,restheta]
  }else{
    sampl   <- cbind(ID=1:nrepl,extf[rep(1,nrepl),])
  }

  if(!inc_theta)  sampl <- sampl[,!grepl("THETA",names(sampl)),drop=FALSE]
  if(!verbose){
    sampl <- sampl[,!grepl("OMEGA",names(sampl)),drop=FALSE]
    sampl <- sampl[,!grepl("SIGMA",names(sampl)),drop=FALSE]
  }

  if(any(sampl<0)) warning("Negative values present, take into account when using results within model")

  names(sampl)[-1] <- paste0("S",names(sampl))[-1]

  if(inc_eta){
    ommat <- get_est(ext)
    etas  <- data.frame(ID=1:nrepl,MASS::mvrnorm(n=nrepl,mu=rep(0,length(ommat$ETA)),Sigma=ommat$OMEGA))
    if(nrepl==1) etas  <- data.frame(ID=1,t(data.frame(MASS::mvrnorm(n=nrepl,mu=rep(0,length(ommat$ETA)),Sigma=ommat$OMEGA))))
    names(etas)[-1] <- paste0("SETA",1:length(ommat$ETA))
    sampl <- merge(sampl,etas)
  }
  return(sampl)
}

#------------------------------------------ nm2rxode2 ------------------------------------------
#' Convert NONMEM model to rxode2 syntax
#'
#' This function converts a NONMEM model to syntax useable in rxode2 simulations
#'
#' @param lstblock structured list with information of the model that was read-in, usually obtained from the \code{\link{nmlistblock}} function
#' @param model character vector with the model content
#' @param ext character with the name of the NONMEM ext file (if not provided estimates are read directly from control stream)
#' @param out character with the name of the output file without a file extension
#' @param control character with the type of control to bre returned (see \code{\link{convert_nonmem}} for more details)
#'
#' @noRd
#' @return a list is returned inluding all building blocks to create a model
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' lst   <- get_nmblock(nmmod, block = c("PROB","SUB","MODEL","PK","DES","PRED",
#'                                       "THETA","OMEGA","ERROR","SIGMA","EST"))
#' lst   <- nmlistblock(lst)
#' nm2rxode2(lst, readLines(nmmod))
#'
nm2rxode2 <- function(lstblock,model,ext=NULL,out=NULL,control=""){ # ,mod_return=NULL
  # Define a translator function to rewrite specific parts for mrgsolve
  translator     <- function(block){
    sapply(block,function(x){
      if(x$type=="comment"){
        rets <- paste0("#",x$comm)
      }else if(x$type=="empty"|x$type==""){
        rets <- ""
      }else{
        if(grepl("controlflow",x$type)){
          rets <- x$cntrl
          if(x$comm!="") rets <- paste0(rets,"#",x$comm)
        }else{
          if(grepl("DADT\\(.*\\)",x$orig)) x$LHS <- sub("DADT\\(","d/dt(A",x$LHS)
          x$RHS <- par_delete(x$RHS)
          rets  <- paste(x$LHS,"=",x$RHS)
          if(x$type=="init")  rets <- ""
          rets <- paste(rets,";")
          if(x$comm!="") rets <- paste(rets,"#",x$comm)
          if(x$type=="control+formula") rets <- paste(x$cntrl,rets)
        }
        rets <- convert_nmsyntax(rets,type="rxode2")
      }
      rets
    })
  }
  params            <- get_param(model,lstblock,ext=ext,addparam = TRUE) # check if we want the addparam as argument (maybe always true?)
  sclpar            <- params$all_params[grepl("^S\\d+$",params$all_params)]
  adderr            <- ifelse(!"F"%in%params$all_params,"",ifelse(length(sclpar)>0,paste0("F = A",sub("S","",sclpar[1]),"/",sclpar[1],";"),"F = A1;"))

  retlst             <- list()
  retlst$modname     <- paste0(out,".r")
  retlst$problem     <- lstblock$PROB[[1]]$orig
  retlst$control2mod <- ifelse(control=="model",TRUE,FALSE)
  retlst$cmt         <- ""
  retlst$pkblock     <- paste(translator(lstblock$PK),collapse="\n")
  retlst$init        <- get_inits(lstblock)
  retlst$init        <- paste(paste0("  ",names(retlst$init),"(0) = ",retlst$init),collapse="\n")
  retlst$desblock    <- paste(paste0("  ",translator(lstblock$DES)),collapse="\n")
  retlst$predblock   <- paste(paste0("  ",translator(lstblock$PRED)),collapse="\n")
  retlst$errorblock  <- translator(lstblock$ERROR)
  retlst$errorblock  <- paste(paste0("  ",c(adderr,retlst$errorblock)),collapse="\n")
  retlst$modtype     <- ifelse(trimws(retlst$desblock)!="","ode",ifelse(trimws(retlst$predblock)!="","pred","other"))
  # These parts do not go into the model and can therefore be omitted (keep in list to return uniform list across packages)
  # (It seems that rxode2 always returns all outputs/variables created in the function)
  retlst$mdl_ret     <- retlst$sigmablock  <- retlst$randstruct <- retlst$param <- ""

  if(retlst$modtype=="pred")  {retlst$init <- retlst$pkblock <- retlst$cmt <- retlst$desblock <- retlst$errorblock <- ""}
  if(retlst$modtype=="other") {retlst$init <- retlst$cmt <- retlst$desblock <- retlst$predblock <- ""}
  if(retlst$modtype=="ode")   {retlst$predblock <- ""}

  retlst$control    <- c("library(rxode2)",paste0("source(\"",out,".r\")"))
  retlst$control    <- c(retlst$control, params$theta_names)
  retlst$control    <- c(retlst$control, dput2(params$params,FALSE,"parm","\n"))
  retlst$control    <- c(retlst$control, dput2(params$omega_matrix,FALSE,"ome","\n"))
  retlst$control    <- c(retlst$control, dput2(params$sigma_matrix,FALSE,"sigm","\n"))
  retlst$control    <- c(retlst$control, "evnt <- et(amt = 100, ii = 24, addl = 1)")
  retlst$control    <- c(retlst$control, "out  <- rxSolve(model,parm,evnt,omega=ome,sigma=sigm,nSub=10)","plot(out)")
  if(control=="model") retlst$control <- retlst$control[-1:-2]

  return(retlst)
}

#------------------------------------------ conv_pow ------------------------------------------
#' Convert infix power notation to prefix notation
#'
#' This function will transform the power notation that can be used in R to the one needed by mrgsolve
#'
#' @param x character vector with the formulas to convert
#'
#' @export
#' @return a vector with the transformed power notations
#' @author Richard Hooijmaijers
#' @examples
#'
#' conv_pow("y = par1*(par2/par3)^xy + a - par4**(2/par5)")
conv_pow <- function(x){
  sapply(x,function(y){
    if(!grepl("=",y) | !grepl("\\*\\*|\\^",y)){
      return(y)
    }else{
      ret   <- sub("=","~",y)
      ret   <- paste0("substitute(",ret,",list(`^`='pow',`**`='pow'))")
      ret   <- try(eval(parse(text=ret)),silent=TRUE)
      if("try-error"%in%class(ret)){
        warning(paste("Could not correctly convert powers (",y,"), returned original string\n"))
        return(y)
      }
      ret   <- paste(trimws(deparse(ret)),collapse="")
      ret   <- gsub("\"","",ret)
      return(sub("~","=",ret))
    }

  },USE.NAMES = FALSE)
}

#------------------------------------------ convert_nmsyntax ------------------------------------------
#' Convert NONMEM specific syntax to R syntax
#'
#' This function converts a NONMEM syntax to R syntax, mainly for functions and operators
#'
#' @param x character vector with the syntax to be converted
#' @param type character with the type of syntax to convert to (currently "deSolve" and "mrgsolve" and "rxode2" are supported)
#'
#' @export
#' @return character vector with the converted syntax
#' @author Richard Hooijmaijers
#' @examples
#'
#' convert_nmsyntax("IF(VAR.GT.0) VAR2 = PHI(1)")
convert_nmsyntax <- function(x,type="mrgsolve"){
  # list of NONMEM functions and operators and the R version of if
  funcs <- get("nmfuncs",envir=.simenv)
  oper  <- get("nmoper",envir=.simenv)
  for(i in 1:length(oper))  x <- gsub(names(oper)[i], oper[i], x, ignore.case = TRUE)
  for(i in 1:length(funcs)){
    if(type=="mrgsolve" & funcs[i]%in%c('min(','max(')){
      x <- gsub(names(funcs)[i], paste0("std::",funcs[i]), x, ignore.case = TRUE)
    }else if(type=="mrgsolve" & funcs[i]=='ceiling('){
      x <- gsub("CEILING\\(","ceil(", x, ignore.case = TRUE)
    }else if(type=="mrgsolve" & funcs[i]=='phi('){
      #x <- gsub("PHI\\((.*)\\)", "R::pnorm(\\1, 0.0, 1.0, 1, 0)", x, ignore.case = TRUE)
      x <- gsub("PHI\\(([^)]*)\\)", "R::pnorm(\\1, 0.0, 1.0, 1, 0)", x, ignore.case = TRUE)
    }else{
      x <- gsub(names(funcs)[i], funcs[i], x, ignore.case = TRUE)
    }
  }
  return(x)
}

#------------------------------------------ cut_equal ------------------------------------------
#' cut function for (appr.) equal intervals
#'
#' This function uses cut to make categories based on approximately equal number of bins
#'
#' @param x the vector that should be cut into equal bins
#' @param n the number of bins that should be used
#' @param type the type of algorithm to be used (see details)
#' @param ntries number of samples/tries for type 2
#'
#' @details Generating equal bins can be quite difficult. There are multiple ways of
#'   assessing if bins are equal. This function provides a method based on quantiles
#'   (type=1) or a method based on sampling proposed by M. Ruppert (type=2). In general
#'   type 1 is faster but less accurate, while type 2 is slower and more accurate
#'   (in case of a reasonable ntries)
#'
#' @export
#' @return  a character vector with the categories/bins
#' @author Richard Hooijmaijers
#' @examples
#'
#'  table(cut_equal(1:20,5))
cut_equal <- function(x,n,type=1,ntries=1000){
  if(type==1){
    brk <- 1
    x2  <- as.numeric(as.factor(x))
    qnt <- stats::quantile(x2,probs=seq(0,1,length.out = brk))
    while(length(unique(qnt)) < (n+1)){
      qnt <- stats::quantile(x2,probs=seq(0,1,length.out = brk))
      brk <- brk+1
    }
    ret   <- as.character(cut(x2,unique(qnt),include.lowest = TRUE,right=FALSE))
    reval <- sapply(unique(ret),function(re) paste(unique(x[which(ret==re)]),collapse="+"))
    #return(plyr::revalue(ret,reval))
    return(unname(reval[match(ret,names(reval))]))
  }else if(type==2){
    uniques <- length(unique(x))
    n       <- min(n, uniques)
    tv      <- table(x)

    m <- t(replicate(ntries, sample(1 : n, min(n, uniques), replace = FALSE)))
    if (uniques - n > 0) m <- cbind(m, matrix(replicate( ntries, sample(replace = TRUE, 1 : n, max(0, uniques - n))), nrow = ntries))
    colnames(m) <- names(tv)

    counts  <- apply(m, 1, function(r) sapply(1 : n, function(i) sum(tv[which(r == i)])))
    score   <- apply(counts, 2, stats::var)
    best    <- which.min(score)
    assignm <- m[best,  ]

    NAMES <- sapply(1 : n, function(i) paste(names(which(assignm == i)), collapse = "+"))
    return(NAMES[assignm[x]])
  }
}

#------------------------------------------ tmpl_model ------------------------------------------
#' Get coding for template models
#'
#' This function returns the code for some template models
#'
#' @param tmpl character with template
#' @param ret character indicating how the result should be returned. Currently "console", "string" and "script" are accepted
#'
#' @details There are templates available for 1-2 CMT PK models for IV/bolus/oral administration
#'   as both analytical as closed form and parameterized with constants or as CL/V.
#'   To see which templates are available in the package
#'   run the function without arguments
#'
#' @export
#' @return model syntax is returned either to console or script (within Rstudio) or a character string
#' @author Richard Hooijmaijers
#' @examples
#'
#' tmpl_model()
#' tmpl_model("ana1CMTbolusK.tmp")
#'
tmpl_model <- function(tmpl, ret="console"){
  if (missing(tmpl)) {
    list.files(system.file(package = "amp.sim"), pattern = "\\.tmp$")
  }else{
    tmplm <- readLines(paste0(system.file(package = "amp.sim"),"/",tmpl))
    if(ret=="script"){
      if(rstudioapi::isAvailable()){
        range <- rstudioapi::getSourceEditorContext()$selection[[1]]$range
        nrang <- rstudioapi::document_position(range$end[1] + 1, 0)
        if(all(range$start==range$end)) nrang <- nrang - c(1,0)
        rstudioapi::insertText(nrang,paste(c(tmplm,"\n\n"),collapse="\n"))
      }else{
        cli::cli_text("code can only be used for output in Rstudio environment (use 'string' or 'console' for ret argument outside Rstudio)")
      }
    }else if(ret=="console"){
      message(paste(tmplm,collapse="\n"))
    }else if(ret=="string"){
      return(tmplm)
    }
  }
}

#------------------------------------------ overlaying ------------------------------------------
#' Performs overlaying of simulations
#'
#' This function basically appends the simulation output and add this to a reactiveValue
#'
#' @param input list with the input elements from a shiny app
#' @param out dataframe with the results from a simulation to be appended for overlaying
#' @param savedsims reactiveValue or list that contains the saved simulations
#'
#' @export
#' @return a list with a dataframe with the appended simulations and settings
#' @author Richard Hooijmaijers
#' @examples
#' if(requireNamespace("shiny")){
#'   out   <- data.frame(time=0:4, A1=rnorm(5), numsim=1)
#'   input <- shiny::reactiveValues(CL=2, KA=0.3, updOpts="appsim")
#'   overlayres <- overlaying(out,input)
#'   savedsims  <- list(res = overlayres$results, sett = overlayres$settings)
#'   overlayres <- overlaying(input=input,out=out,savedsims=savedsims)
#'   head(overlayres$result)
#' }
#'
overlaying <- function(out,input,savedsims=NULL){
  if(is.null(input)) stop("no input object can be found")
  inpl <- shiny::isolate(shiny::reactiveValuesToList(input))
  if(!exists("out")) stop("Make sure a dataframe is available with simulation results")
  if(inpl$updOpts=="appsim" & !is.null(savedsims$res)){
    resn            <- out
    resn$numsim     <- max(savedsims$res$numsim) + 1
    resn$Simulation <- paste0("sim ",resn$numsim)
    resn$Simulation <- factor(resn$Simulation,levels=unique(resn$Simulation))
    res             <- rbind(savedsims$res,resn)
    sett            <- c(savedsims$sett,list(vals=inpl))
    names(sett)[length(sett)] <- unique(as.character(resn$Simulation))
  }else{
    res            <- out
    res$numsim     <- 1
    res$Simulation <- paste0("sim ",res$numsim)
    res$Simulation <- factor(res$Simulation,levels=unique(res$Simulation))
    sett           <- list(`sim 1`=inpl)
  }
  return(list(results=res,settings=sett))
}

context("Test if functions for shiny work as expected")

#--------------------------
# Test mod2shiny function
test_that("mod2shiny correctly creates a shiny app", {
  nmmod <- system.file("testfiles/nonmem.mod",package="amp.sim")
  tst   <- convert_nonmem(nmmod,out=paste0(tempdir(),"/shinytest"),control="string",overwrite = TRUE)
  prm   <- c(THETA1 = 0.08, THETA2 = 2, THETA3 = 1, THETA4 = 0.2, THETA5 = 1.2, WEIGHT = 70, SEX = 1)
  evnt  <- mrgsolve::ev(amt = 100, ii = 24, addl = 1)
  nams  <- c(THETA1 = "KA (1/h)", THETA2 = "CL (l/h)", THETA3 = "V (l)", THETA4 = "effect of WT", THETA5 = "effect of SEX") 
  
  expect_message(mod2shiny(prm,modfile=paste0(tempdir(),"/shinytest.cpp"),evnt=evnt,naming=nams,
                           framework="mrgsolve",outloc=tempdir()))
  
  uip  <- try(readLines(paste0(tempdir(),"/ui.r")))
  srvp <- try(readLines(paste0(tempdir(),"/server.r")))
  
  expect_false("try-error"%in%class(uip))
  expect_false("try-error"%in%class(srvp))
  expect_true(all(c("etc","www")%in%list.files(tempdir())))
  expect_true(any(grepl("numericInput.*THETA1.*KA",uip)))
  expect_true(any(grepl("parm.*WEIGHT",srvp)))
})

#--------------------------
# Test settings2df function
test_that("settings2df correctly creates dataframe with settings", {
  sett <- list(`sim 1`=list(THETA1=0.5,THETA2=1,alllabs=c("THETA1%=%DUMMY1","THETA2%=%DUMMY2")),`sim 2`=list(THETA1=0.9,THETA2=1))
  res  <- settings2df(sett)
  
  expect_true("data.frame"%in%class(res))
  expect_true(all(c("name","sim 1","sim 2")%in%names(res)))
  expect_true(all(c("DUMMY1","DUMMY2")%in%res$name))
  expect_true(all(c(0.5,0.9)%in%as.numeric(res[1,2:3])))
})

#--------------------------
# Test overlaying function
test_that("overlaying correctly adds information", {
  out         <- data.frame(time=0:4, A1=rnorm(5), numsim=1)
  input       <- shiny::reactiveValues(CL=2, KA=0.3, updOpts="appsim")
  overlayres1 <- overlaying(out,input)
  savedsims   <- list(res = overlayres1$results, sett = overlayres1$settings)
  overlayres2 <- overlaying(input=input,out=out,savedsims=savedsims)
  
  expect_type(overlayres1, "list")
  expect_true(unique(overlayres1$results$numsim)==1)
  expect_equal(overlayres1$settings$`sim 1`$CL,2)
  
  expect_equal(unique(overlayres2$results$numsim),1:2)
  expect_equal(length(overlayres2$settings), 2)
  expect_equal(overlayres2$settings[[1]], overlayres2$settings[[2]])
})

#------------------------------------------ nm2rxode2 ------------------------------------------
#' Convert NONMEM model to nonmem2rx syntax
#'
#' This function converts a NONMEM model to syntax useable in rxode2 simulations fully based on the nonmem2rx package
#'
#' @param mod character with the model file to be read in and converted
#' @param out character with the name of the output file without a file extension
#' @param control character with the type of control to be returned (see \code{\link{convert_nonmem}} for more details)
#'
#' @noRd
#' @return a list is returned inluding all building blocks to create a model
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' nm2nonmem2rx(nmmod) |> suppressMessages()
#'
nm2nonmem2rx <- function(mod, out=NULL,control=""){
  res         <- nonmem2rx::nonmem2rx(file=mod)
  theta_names <- stats::setNames(res$iniDf$label[!is.na(res$iniDf$ntheta)],res$iniDf$name[!is.na(res$iniDf$ntheta)]) 
  covs        <- res$allCovs[!res$allCovs%in%unique(unlist(dimnames(res$sigma)))]
  covs        <- stats::setNames(rep(-999,length(covs)),covs)
  parm        <- c(res$theta,covs)
  retlst      <- list()
  retlst$modelfun  <- deparse(body(res$fun))
  retlst$modelfun  <- c("model <- function()",retlst$modelfun)
  retlst$modelfun  <- paste(retlst$modelfun,collapse="\n")
  retlst$modname   <- paste0(out,".r")
  
  retlst$control    <- c("library(rxode2)",paste0("source(\"",out,".r\")"))
  retlst$control    <- c(retlst$control, dput2(theta_names,TRUE,"theta_names"))
  retlst$control    <- c(retlst$control, dput2(parm,FALSE,"parm","\n"))
  retlst$control    <- c(retlst$control, dput2(res$omega,FALSE,"ome","\n"))
  retlst$control    <- c(retlst$control, dput2(res$sigma,FALSE,"sigm","\n"))
  retlst$control    <- c(retlst$control, "evnt <- et(amt = 100, ii = 24, addl = 1)")
  retlst$control    <- c(retlst$control, "out  <- rxSolve(model,parm,evnt,omega=ome,sigma=sigm,nSub=10)","plot(out)")
  if(control=="model") retlst$control <- retlst$control[-1:-2]
  
  return(retlst)
}

#------------------------------------------ dose_func ------------------------------------------
#' Create data.frame with dosing for usage in simulation with deSolve
#'
#' This function creates a data.frame that can be used for events of a deSolve simulation
#'
#' @param cmt number of compartment or differential equation where the dosing should be given
#' @param value the value of the dosing that should be given
#' @param tinf in case value is set an infusion is assumed with tinf as infusion time
#' @param tau dosing interval to be used
#' @param ndose number of doses to be used
#' @param times In case tau and ndose cannot be used (unequal dosing), this parameter can be
#'   used to set times of dosing (e.g c(0,24,168))
#'
#' @export
#' @return a data frame that can be used as an event dataset within deSolve
#' @author Richard Hooijmaijers
#' @examples
#'
#'  dose_func(8,100,tau=48,ndose=5,tinf=2)
dose_func <- function(cmt,value,tinf,tau,ndose,times){
  if(missing(times)){
    timing <- seq(0,(ndose-1)*tau,tau)
  }else{
    timing <- times
  }
  dose <- data.frame(var=paste0("A",cmt),time=timing,value=value,method="add")
  if(!missing(tinf)){
    dose2 <- data.frame(var=paste0("A",cmt),time=timing+tinf,value=0,method="rep")
    dose  <- rbind(dose,dose2)
    dose  <- dose[order(dose$time),]
    dose$value[dose$value!=0] <- dose$value[dose$value!=0] / tinf
  }
  return(dose)
}

#------------------------------------------ get_parmvar ------------------------------------------
#' Get the variables within a NONMEM model that should be passed to the simulation model
#'
#' This function go through all formulas and control flows to check if a parameter is created ad-hoc
#' or taken from the input file. This is important as these variables need to be set for simulation
#'
#' @param lstblock list with each item being a separate structured dollar block, usually obtain from \code{\link{nmlistblock}}
#' @param returnall logical indicating if all variables should be returned or just the ones that are not defined
#'
#' @export
#' @return a vector with model variables for the simulation model
#' @author Richard Hooijmaijers
#' @examples
#'
#' mod    <- system.file("example_models","PK.1CMT.ORAL.COV.mod", package = "amp.sim")
#' mdll   <- get_nmblock(mod,block=c("PK","DES"))
#' mdlls  <- nmlistblock(mdll)
#' get_parmvar(mdlls)
#' 
get_parmvar <- function(lstblock,returnall=FALSE){
  alllst   <- c(lstblock$PK,lstblock$DES,lstblock$ERROR,lstblock$PRED)
  if(is.null(alllst)) stop("There is no PK, DES, PRED or error block in the provided model")
  allvars   <- lapply(alllst,function(x){if(x$type%in%c("formula","control+formula","controlflow","init")) return(x)})
  allvars   <- allvars[!sapply(allvars,is.null)]
  addvars   <- vector("character") # return vector with variables to be added
  noaddvars <- vector("character") # temp vector to check if variable should be added
  for(i in 1:length(allvars)){
    if(allvars[[i]]$type=="controlflow"){
      # if(!grepl("ELSE|END IF|ENDIF",allvars[[i]]$cntrl))  rhsvars <- all.vars(parse(text=paste(gsub(".*if","if",convert_nmsyntax(allvars[[i]]$cntrl)),"}")))
      # The preceding line has issues in case if is also in the name (e.g. IF(CMedif.GT.0)), because a controlflow always start with if we do not need the gsub
      if(!grepl("ELSE|END IF|ENDIF",allvars[[i]]$cntrl))  rhsvars <- all.vars(parse(text=paste(convert_nmsyntax(allvars[[i]]$cntrl),"}")))
    }else if(allvars[[i]]$type=="control+formula"){
      rhsvars  <- c(all.vars(parse(text=allvars[[i]]$RHS)),all.vars(parse(text=paste(convert_nmsyntax(allvars[[i]]$cntrl),"{}"))))
    }else{
      rhsvars   <- all.vars(parse(text=convert_nmsyntax(allvars[[i]]$RHS)))
    }
    # LHS variables are never obtained from data but logically always assigned
    noaddvars <- unique(c(noaddvars,allvars[[i]]$LHS))
    if(length(rhsvars)>0){
      for(j in rhsvars){
        if(j%in%noaddvars)  noaddvars <- unique(c(noaddvars,j))
        if(!j%in%noaddvars) addvars   <- unique(c(addvars,j))
      }
    }
  }
  if(!returnall)  return(addvars[addvars!='']) else return(c(addvars,noaddvars)[c(addvars,noaddvars)!=''])
}

#------------------------------------------ settings2df ------------------------------------------
#' Add settings to dataframe
#'
#' This function adds settings or input elements to a dataframe for displaying in app
#'
#' @param savedsims reactiveValue that contains the saved simulations
#' @param leaveout character vector with the the elements that should be left out the dataframe
#'
#' @export
#' @return a dataframe with settings
#' @author Richard Hooijmaijers
#' @examples
#'
#' if(requireNamespace("tidyr")){
#'   sim1 <- list(THETA1  = 0.5, THETA2 = 1,
#'                alllabs =c("THETA1%=%DUMMY1","THETA2%=%DUMMY2"))
#'   sim2 <- list(THETA1  = 0.9, THETA2 = 1.5)
#'   settings2df(list(sim1 = sim1, sim2 = sim2))
#' } 
#'
settings2df <- function(savedsims,leaveout=c("go","updOpts","sett","refr","tabsel")){
  if (!requireNamespace("tidyr", quietly = TRUE)) stop("Package \"tidyr\" needed for this function to work", call. = FALSE)
  sett          <- lapply(savedsims,function(x) x[names(x)!="alllabs"])
  sett          <- lapply(sett,function(x) lapply(x,paste,collapse=", ")) # take into account sliders
  sett          <- do.call(rbind,lapply(sett,"as.data.frame"))
  sett$sim      <- row.names(sett)
  sett          <- tidyr::pivot_longer(sett,cols=!tidyr::contains("sim"))
  sett          <- tidyr::pivot_wider(sett,names_from = "sim",values_from="value")
  labsr         <- savedsims[[1]]$alllabs
  labs          <- sub(".*%=%","",labsr)
  names(labs)   <- sub("%=%.*","",labsr)
  sett          <- sett[!sett$name%in%leaveout,]
  sett$name     <- labs[match(sett$name,names(labs))]
  return(sett)
}

#------------------------------------------ mod2shiny ------------------------------------------
#' Creates a basic simulation app for a given R model
#'
#' This function fills in a default template for simulations within shiny
#'
#' @param parvector named vector with the model parameters, should contain all parameters used by the model
#' @param modfile A script with the model defined in it, it is assumed that this file is created using \code{\link[amp.sim]{convert_nonmem}}
#' @param evnt dataframe with the events used by the model (this is saved as rds together with the app)
#' @param init vector with the compartment initialization (only applicable for deSolve framework)
#' @param naming named vector with the names in parvector and the new values to use within the shiny app
#' @param apptitle string with the title to be used for the app
#' @param outloc character with location where the resulting shiny app should be saved
#' @param omega vector with the omega matrix for the model (only applicable for rxode2 framework)
#' @param sigma vector with the sigma matrix for the model (only applicable for rxode2 framework)
#' @param delloc logical indicating if the location should be deleted first
#' @param framework character indicating the simulation framework that was used, currently "deSolve", "rxode2", "nonmem2rx" and "mrgsolve" are supported
#' @param logo  character with a png of the company logo added in the header of the shiny app
#' @param times  character vector with the times to simulate or set to NULL to use a default time vector
#'
#' @details This function creates a default shiny app that can be used as a starting point for further
#'   development. There are already some basic features available but it is intended to be adapted when used 
#'   for production. For the automatic creation of an app it is assumed that the model is created using the [convert_nonmem]
#'   function. Although it is not strictly necessary, the information provided to this function will be in the correct format
#'   when this function is used. Some of the arguments in this function are only necessary in case a certain conversion framework
#'   is used. In general a single subject is simulated, but be aware that for the deSolve and rxode2 framework OMEGA/ETA is implemented.
#'
#' @export
#' @return creates necessary app files for a shiny simulation app
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' outf  <- tempfile()
#' convert_nonmem(nmmod, out=outf, verbose = FALSE)
#' prm   <- c(THETA1 = 0.3, THETA2 = 2, THETA3 = 5)
#' evnt  <- mrgsolve::ev(amt = 100, ii = 24, addl = 1)
#' nams  <- c(THETA1 = "KA (1/h)", THETA2 = "CL (l/h)", THETA3 = "V (l)") 
#' mod2shiny(prm, modfile= paste0(outf,".cpp"), evnt = evnt,
#'           naming = nams, framework = "mrgsolve", outloc=tempdir())
#' 
mod2shiny <- function(parvector,modfile,evnt,init=NULL,naming=NULL,apptitle="Shiny app title",outloc=".",omega=NULL,sigma=NULL,
                      delloc=FALSE,framework="deSolve",logo=paste0(system.file(package="amp.sim"),"/logo.png"),times=NULL){
  
  if(framework=="nonmem2rx") framework="rxode2" # nonmem2rx can be handled in the same way as rxode2
  if(framework=="deSolve" && is.null(init)) stop("Provide 'init' when the deSolve framework is used")
  if(framework=="rxode2" && (is.null(omega) | is.null(sigma))) stop("Provide 'omega' and 'sigma' when the rxode2 framework is used")
  if(delloc) try(unlink(outloc,recursive=TRUE))
  # Fill in ui template
  parvector2 <- parvector
  if(!is.null(naming)){
    if(any(names(naming)%in%names(parvector))) names(parvector2)[names(parvector)%in%names(naming)] <- naming
  }
  uiIn    <- readLines(paste0(system.file(package="amp.sim"),"/ui.tmpl"))
  #inpe    <- paste0("numericInput(inputId = '",names(parvector),"', label='",names(parvector),":', value=",round(parvector,2),")")
  inpe    <- paste0("numericInput(inputId = '",names(parvector),"', label='",names(parvector2),":', value=",signif(parvector,3),")")
  #sourcef <- ifelse(framework=="mrgsolve",paste0("assign('model',mread('etc/",basename(modfile),"'),envir = .GlobalEnv)"),paste0("source('etc/",basename(modfile),"')"))
  uilist  <- list(apptitle=apptitle,inputElements=paste(inpe,collapse=",\n        "),packages=paste0("library(",framework,")"))
  uiOut   <- whisker::whisker.render(uiIn, uilist)
  
  # Fill in server template - decided to place in the entire parm output including changes for input elements
  serverIn   <- readLines(paste0(system.file(package="amp.sim"),"/server.tmpl"))
  sourcef    <- ifelse(framework=="mrgsolve",paste0("model <- mread('etc/",basename(modfile),"')"),paste0("source('etc/",basename(modfile),"')"))
  parserv1   <- paste0("parm <- c(",paste(paste(names(parvector),"=",round(parvector,2)),collapse=", "),")")
  parserv2   <- paste(paste0("    parm['",names(parvector),"'] <- input$",names(parvector)),collapse="\n")
  parserv    <- paste(parserv1,parserv2,sep="\n")
  usede      <- ifelse(framework=="deSolve",TRUE,FALSE)
  userxode2  <- ifelse(framework=="rxode2",TRUE,FALSE)
  if(is.null(times)){
    times  <- "times <- seq(0,max(events$time)+24,length.out=1000)"
    if("addl"%in%names(evnt)) times <- "times <- seq(0,max(events$time) + ((max(events$addl)+1)*max(events$ii)) ,length.out=1000)"
  }else{
    times <- dput2(times,obj="times",collapse="")
  }
  if(!is.null(init))   initcode   <- dput2(init,obj="inits",collapse="")  else initcode <- ""
  if(!is.null(omega))  omegacode  <- dput2(omega,obj="omega",collapse="") else omegacode <- ""
  if(!is.null(sigma))  sigmacode  <- dput2(sigma,obj="sigma",collapse="") else sigmacode <- ""
  if(framework=="deSolve")  simcode <- "out <- as.data.frame(lsoda(inits,times,model, parm,events=list(data=events)))"
  if(framework=="mrgsolve") simcode <- "model <- param(model,parm)\nout  <- zero_re(model) %>% ev(events) %>% mrgsim(tgrid=times)"
  if(framework=="rxode2")   simcode <- "out   <- rxSolve(model,parm,events,omega=omega,sigma=sigma,nSub=1)"
  #simcode    <- paste(simcode,"out  <- tidyr::pivot_longer(out,cols=!contains('time'))",sep="\n")
  simcode    <- paste(simcode,"out  <- tidyr::pivot_longer(as.data.frame(out),cols=!contains(c('time','ID')))",sep="\n")
  servlist   <- list(sourcefunc=sourcef,changeParm=parserv,usedesolve=usede,initcode=initcode,simcode=simcode,timesv=times,
                     userxode2=userxode2,omegacode=omegacode,sigmacode=sigmacode)
  #print(servlist)
  serverOut  <- whisker::whisker.render(serverIn, servlist)
  
  # Make folders and write files
  dir.create(paste0(outloc,"/etc"),recursive = TRUE, showWarnings = FALSE)
  dir.create(paste0(outloc,"/www"),recursive = TRUE, showWarnings = FALSE)
  saveRDS(evnt,paste0(outloc,"/etc/events.rds"))
  writeLines(uiOut,paste0(outloc,"/ui.r"))
  writeLines(serverOut,paste0(outloc,"/server.r"))
  file.copy(logo,paste0(outloc,"/www/logo.png"))
  file.copy(paste0(system.file(package="amp.sim"),"/modelscheme.png"),paste0(outloc,"/www/modelscheme.png"))
  file.copy(modfile,paste0(outloc,"/etc/",basename(modfile)))
  message(paste0("Shiny app created in location '",outloc,"'. It can be submitted using:\n",cli::style_bold("shiny::runApp('",normalizePath(outloc, winslash = "/"),"',launch.browser=TRUE)")))
}

# Set up and environment to hold some global variables
.simenv <- new.env(parent=emptyenv())

# NONMEM functions and the corresponding translation (mrsolve has some deviations for std::min/max which is handled in convert_nmsyntax)
assign("nmfuncs", c("EXP\\("="exp(","LOG\\("="log(","ABS\\("="abs(","SIN\\("="sin(","COS\\("="cos(", "PHI\\(" = "phi(",
                  "SQRT\\("="sqrt(","TANH\\("="tan(","MIN\\("="min(","MAX\\("="max(","FLOOR\\("="floor(","CEILING\\("="ceiling("), envir=.simenv)

# NONMEM operators and the corresponding translation
assign("nmoper", c("\\.LT\\."=" < ","\\.LE\\."=" <= ","\\.EQ\\."=" == ","\\.GE\\."=" >= ","\\.GT\\."=" > ",
                 "\\.NE\\."=" != ","\\.AND\\."=" & ","\\.OR\\."=" | ","IF\\(|IF[[:blank:]]*\\("="if(","THEN"="{","ENDIF"="}",
                 "ELSE IF"="}else if","ELSE$"="}else{","END IF$"="}"), envir=.simenv)

# reserved keywords, for now listed all reserved keywords from mrgsolve:::reserved(). If necessary extend for rxode2 or NONMEM (e.g. reserved2, etc)
assign("reserved1", c("ID", "amt", "cmt", "ii", "ss", "evid", "addl", "rate", "time","SOLVERTIME", "table", "ETA", "EPS", "AMT", "CMT", "ID", "TIME",
                     "EVID", "simeps", "self", "simeta", "NEWIND", "DONE", "CFONSTOP", "DXDTZERO", "CFONSTOP", "INITSOLV", "_F", "_R", "_ALAG", "SETINIT",
                     "report", "_VARS_", "VARS", "SS_ADVANCE", "AMT", "CMT", "II", "SS", "ADDL", "RATE", "THETA", "pred_CL", "pred_VC", "pred_V",
                     "pred_V2", "pred_KA", "pred_Q", "pred_VP", "pred_V3", "double", "int", "bool", "capture", "T"), envir=.simenv)

#------------------------------------------ nm2mrgsolve ------------------------------------------
#' Convert NONMEM model to mrgsolve syntax
#'
#' This function converts a NONMEM model to syntax useable in mrgsolve simulations.
#'
#' @param lstblock structured list with information of the model that was read-in, usually obtained from the [nmlistblock] function
#' @param model character vector with the model content
#' @param ext character with the name of the NONMEM ext file (if not provided estimates are read directly from control stream)
#' @param mod_return a character vector indicating which items should be returned from the model function (see [convert_nonmem] for more details)
#' @param out character with the name of the output file without a file extension
#'
#' @noRd
#' @return a list is returned inluding all building blocks to create a model
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' lst   <- get_nmblock(nmmod, block = c("PROB","SUB","MODEL","PK","DES","PRED",
#'                                       "THETA","OMEGA","ERROR","SIGMA","EST"))
#' lst   <- nmlistblock(lst)
#' nm2mrgsolve(lst, readLines(nmmod))
#'
nm2mrgsolve <- function(lstblock,model,ext=NULL,mod_return=NULL,out=NULL){
  # Define a translator function to rewrite specific parts for mrgsolve
  wrn <- msg <- ""
  translator     <- function(block){
    sapply(block,function(x){
      if(x$type=="comment"){
        rets <- paste0("//",x$comm)
      }else if(x$type=="empty"|x$type==""){
        rets <- ""
      }else{
        if(grepl("controlflow",x$type)){
          rets <- x$cntrl
          if(x$comm!="") rets <- paste0(rets,"//",x$comm)
        }else{
          #if(grepl("A[[:digit:]]+$",x$LHS) & grepl("A\\([[:digit:]]+\\)$",trimws(x$RHS))){
          if(grepl("^A[[:digit:]]+$",trimws(x$LHS)) & grepl("A\\([[:digit:]]+\\)$",trimws(x$RHS))){
            assign("msg",c("Assignment of compartment with same name (commented out)",msg),envir = parent.frame(n = 4))
            x$LHS <- paste0("//",x$LHS)
          } 
          #if(grepl("A[[:digit:]]+$",x$LHS) & !grepl("A\\([[:digit:]]+\\)$",trimws(x$RHS))){
          if(grepl("^A[[:digit:]]+$",trimws(x$LHS)) & !grepl("A\\([[:digit:]]+\\)$",trimws(x$RHS))){
            assign("wrn",c("Assignment of compartment (commented out), check code to ensure this is correct",wrn),envir = parent.frame(n = 4))
            x$LHS <- paste0("//",x$LHS)
          }
          if(identical(trimws(x$LHS),gsub("\\(|\\)","",trimws(x$RHS)))) acomm <- "//" else acomm <- ""
          rets <- paste(acomm,x$LHS,"=",x$RHS)  
          #rets <- paste(x$LHS,"=",x$RHS)
          if(x$type=="init")  rets <- ""
          rets <- conv_pow(rets)
          rets <- paste(rets,";")
          if(x$comm!="") rets <- paste(rets,"//",x$comm)
          if(x$type=="control+formula") rets <- paste(x$cntrl,rets)
        }
        rets <- convert_nmsyntax(rets,type="mrgsolve")
      }
      rets
    })
  }
  params            <- get_param(model,lstblock,ext=ext,addparam = TRUE) # check if we want the addparam as argument (maybe always true?)
  sclpar            <- params$all_params[grepl("^S\\d+$",params$all_params)]
  adderr            <- ifelse(!"F"%in%params$all_params,"",ifelse(length(sclpar)>0,paste0("F = A(",sub("S","",sclpar[1]),")/",sclpar[1],";"),"F = A(1);"))
  
  retlst             <- list()
  retlst$modname     <- paste0(out,".cpp")
  retlst$problem     <- lstblock$PROB[[1]]$orig
  retlst$control2mod <- FALSE
  retlst$cmt         <- gsub(";.*","",unlist(lapply(lstblock$MODEL,"[[","orig")))
  if(any(grepl("^NCOMPARTMENTS\\s*=|^NCM\\s*=|^NCOMPS\\s*=|^NCOMP\\s*=",retlst$cmt))){
    #retlst$cmt         <- retlst$cmt[grepl("^NCOMPARTMENTS=|^NCM=|^NCOMPS=|^NCOMP=",retlst$cmt)][1]
    #retlst$cmt         <- paste(paste0("A",1:as.numeric(sub("^NCOMPARTMENTS=|^NCM=|^NCOMPS=|^NCOMP=","",retlst$cmt))),collapse=" ") 
    retlst$cmt         <- sub(".*\\b(NCOMP|NCOMPARTMENT|NCOMPARTMENTS|NCM|NCOMPS|NCOMP)\\b\\s*=\\s*([0-9]+).*", "\\2", paste(retlst$cmt, collapse=" "))
    retlst$cmt         <- paste(paste0("A",1:as.numeric(retlst$cmt)),collapse=" ") 
  }else{
    #retlst$cmt         <- unlist(regmatches(retlst$cmt,gregexpr("\\(.*?\\)",retlst$cmt)))
    retlst$cmt         <- unlist(regmatches(retlst$cmt,gregexpr("COMPARTMENT.*=|COMP.*=",retlst$cmt))) # no necessarily brackets but we need at least "COMP="
    retlst$cmt         <- paste(paste0("A",1:length(retlst$cmt)),collapse=" ") # Do not use names for CMTs
  }
  retlst$cmt         <- paste(c("$CMT",retlst$cmt),collapse="\n")
  retlst$pkblock     <- paste(c("$PK",translator(lstblock$PK)),collapse="\n")
  retlst$init        <- get_inits(lstblock)
  cinit              <- sub("tmp = ","",conv_pow(paste0("tmp = ",retlst$init))) # convert powers for initials
  retlst$init        <- paste(paste0("A_0(",gsub("A","",names(retlst$init)),") = ",cinit,";"),collapse="\n")
  retlst$desblock    <- paste(c("$DES",translator(lstblock$DES)),collapse="\n")
  retlst$predblock   <- paste(c("$PRED",translator(lstblock$PRED)),collapse="\n")
  retlst$errorblock  <- translator(lstblock$ERROR)
  retlst$errorblock  <- paste(c("$ERROR",adderr,retlst$errorblock),collapse="\n")
  retlst$param       <- paste(paste(names(params$params),"=",params$params),collapse=", ")
  retlst$randstruct  <- paste("$OMEGA @block",paste(params$omega_string, collapse="\n"),sep="\n") # we do not need annotations, so keep it as simple as possible
  if(!is.null(params$sigma_string)) retlst$sigmablock  <- paste("$SIGMA @block",paste(params$sigma_string, collapse="\n"),sep="\n") else retlst$sigmablock  <- ""
  retlst$modtype     <- ifelse(retlst$desblock!="$DES\n","ode",ifelse(retlst$predblock!="$PRED\n","pred","other"))
  
  if(retlst$modtype=="pred")  {retlst$init <- retlst$pkblock <- retlst$cmt <- retlst$desblock <- retlst$errorblock <- ""}
  if(retlst$modtype=="other") {retlst$init <- retlst$cmt <- retlst$desblock <- retlst$predblock <- ""}
  if(retlst$modtype=="ode")   {retlst$predblock <- ""}
  if(!is.null(mod_return)) retlst$mdl_ret <- paste(c("$CAPTURE",mod_return),collapse=" ") else retlst$mdl_ret <- ""
  
  retlst$control  <- "library(mrgsolve)"
  retlst$control  <- c(retlst$control,paste0("mod <- mread(\"",out,".cpp\")"))
  retlst$control  <- c(retlst$control,params$theta_names)
  retlst$control  <- c(retlst$control,dput2(params$params,TRUE,"parm"))
  retlst$control  <- c(retlst$control,"# mod  <- param(mod,parm)","evnt <- ev(amt = 100, ii = 24, addl = 1)")
  retlst$control  <- c(retlst$control,"# To simulate single subject use zero_re(mod) in combination with nid=1")
  retlst$control  <- c(retlst$control,"out  <- mod %>% ev(evnt) %>% mrgsim(end = 48, delta = 0.1, nid=10)","plot(out)")
  retlst$wrn      <- unique(wrn[wrn!=""])
  retlst$msg      <- unique(msg[msg!=""])
  return(retlst)
}

#------------------------------------------ convert_nonmem ------------------------------------------
#' Convert NONMEM model to R syntax
#'
#' This function converts a NONMEM model to syntax useable in R simulations. Currently deSolve, rxode2 (nonmem2rx) and mrgsolve are available syntaxes to use
#' Additionally a code to control the simulations is created to directly test the simulations
#'
#' @param model character with the model file to be read in and converted
#' @param out character with the name of the output file without a file extension
#' @param ext character with the name of the NONMEM ext file (if not provided estimates are read directly from control stream)
#' @param mod_return a character vector indicating which items should be returned from the model function. For more information see details
#' @param type_return character indicating the type of model that should be created. Currently "deSolve", "rxode2", "nonmem2rx" and "mrgsolve" are accepted
#' @param overwrite logical indicating if the output model should be overwritten
#' @param control character indicating how the model control code should be returned. Currently "file", "console", "string", "script" and "model" (only for rxode2/DeSolve) are accepted
#' @param verbose logical indicating if additional information is written to the console
#'
#' @details For the mod_return argument, the additional variables are added to the output in case the type_return is either mrgsolve or deSolve.
#'
#' @export
#' @return a converted file is generated and a message is returned
#' @author Richard Hooijmaijers
#' @examples
#'
#' mod  <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' outf <- tempfile()
#'
#' convert_nonmem(mod, outf, verbose = FALSE)
#' readLines(paste0(outf,".cpp")) |> head(n=15) |> cat(sep="\n")
#'
#' convert_nonmem(mod, outf, verbose = FALSE, type_return = "nonmem2rx") |> suppressMessages()
#' readLines(paste0(outf,".r")) |> head(n=15) |> cat(sep="\n")
#'
convert_nonmem <- function(model,out, ext=NULL, mod_return=NULL,type_return="mrgsolve",overwrite=FALSE, control="file",verbose=TRUE){
  # removed 'addparam=TRUE' from arguments, decided to always use this (cannot think of a reason why not to)
  mdl    <- readLines(model)
  mdll   <- get_nmblock(mdl,block=c("PROB","SUB","MODEL","PK","DES","PRED","THETA","OMEGA","ERROR","SIGMA","EST"))
  mdlls  <- nmlistblock(mdll)
  if(type_return=="mrgsolve"){
    tmplst <- nm2mrgsolve(mdlls,model=mdl,ext=ext, mod_return = mod_return, out=out)
  }else if(type_return=="rxode2"){
    tmplst <- nm2rxode2(mdlls,model=mdl,ext=ext, out=out, control=control)
  }else if(type_return=="deSolve"){
    tmplst <- nm2deSolve(mdlls,model=mdl,ext=ext, out=out, control=control)
  }else if(type_return=="nonmem2rx"){
    if(length(find.package("nonmem2rx", quiet = TRUE))==0) stop("the nonmem2rx package should be installed to use this option")
    tmplst <- nm2nonmem2rx(model,out=out, control=control)
  }else{
    stop("make sure a valid type_return is given")
  }
  # fill in template and write to disk
  # tomod <- ifelse(control=="model",TRUE,FALSE)
  tmpl  <- readLines(paste0(system.file(package="amp.sim"),"/",tolower(type_return),".tmpl"))
  if(!is.null(out)){
    if(!overwrite && file.exists(tmplst$modname)){
      warning("File already present and not overwritten (set overwrite to TRUE if file should be overwritten)")
    }else{
      writeLines(whisker::whisker.render(tmpl,tmplst),tmplst$modname)
    }
  }
  # handle the control part of the simulations
  if(control=="console")  message(paste(tmplst$control,collapse="\n"))
  if(control=="file")     writeLines(paste(tmplst$control,collapse="\n"), paste0(out,"_control.r"))
  if(control=="string")   return(tmplst$control)
  if(control=="script"){
    if(rstudioapi::isAvailable()){
      range <- rstudioapi::getSourceEditorContext()$selection[[1]]$range
      nrang <- rstudioapi::document_position(range$end[1] + 1, 0)
      if(all(range$start==range$end)) nrang <- nrang - c(1,0)
      rstudioapi::insertText(nrang,paste(c(tmplst$control,"\n\n"),collapse="\n"))
    }else{
      cli::cli_text("script can only be used for output in Rstudio environment (use 'file', 'string' or 'console' for control argument outside Rstudio)")
    }
  }
  # Handle messages (and verbose), check how we want to do this (could add messages in nm2package or have a single message?)
  if(verbose){
    cli::cli_text(paste0("The model is converted to {.pkg ",type_return,"} syntax!"))
    cli::cli_text("However take into account that this function tries to create a good starting point for simulations in R. It is advised though to double check and test the resulting model code to see if everything is handled correctly. At least the following should be taken into account:")
    cli::cli_bullets(c("*"="Be aware of usage of TIME/TALD in model code. In certain cases this works, however rewriting might be necessary (likely when used in des block)"))
    cli::cli_bullets(c("*"="When converting to {.pkg deSolve}, items like IV dosing, bio-availability, lag time and residual error needs to be set manually"))
    cli::cli_bullets(c("*"="When converting to {.pkg deSolve}, initialization of compartments might need additional checking when model parameters are used (e.g. A_0(1)=BSL)"))
    cli::cli_text("Additional information can be found in the package vignettes.")
  }
  if(length(tmplst$wrn>0)) for(i in tmplst$wrn){cli::cli_alert_warning(i)} 
  if(length(tmplst$msg>0)) for(i in tmplst$msg){cli::cli_alert_info(i)} 
}

#------------------------------------------ simdata ------------------------------------------
#' Create a simulation dataset for NONMEM simulation
#'
#' This function creates a simulation dataset including information
#'  for dosing and sampling. The function is setup to return a data frame
#'  to be used within NONMEM or other frameworks
#' @param time a vector with all sampling times to be used
#' @param dosetime a vector with the different dosing times
#' @param doseheight a vector with the different dosing heights (to be added to AMT)
#' @param addl a vector with the additional dose levels (must be same length as doseheight)
#' @param ii a vector with the interdose interval (must be same length as doseheight)
#' @param rate a vector with the dosing rate (must be same length as doseheight)
#' @param numid a vector with the number of IDs to be created
#' @param ... additional variables to be added to the resulting dataset
#'
#' @export
#' @return a dataframe that can be used for NONMEM simulations
#' @author Richard Hooijmaijers
#' @examples
#'
#' # Include additional variables
#' sim1 <- simdata(seq(0,24,1),0.5,100,10,12,NA,2, WEIGHT=70, ETA=0)
#'
#' # unequal dosing scheme
#' sim2 <- simdata(seq(0,24,1), dosetime = c(0.5,1), doseheight = c(100,200),
#'                 addl = c(10,5), ii = c(12,24), numid = 2)
#'
#' # Directly create a sequence of different dose levels
#' sim3 <- lapply(seq(25,60,5),function(x){
#'   simdata(time=1:12,dosetime=0,doseheight=x,addl=139,ii=120,rate=0,numid=10)
#' }) 
#' sim3 <- do.call(rbind,sim3)
#'
simdata <- function(time,dosetime,doseheight,addl,ii,rate=NA,numid=5,...){
  dose     <- data.frame(TIME=dosetime,DV=NA,AMT=doseheight,ADDL=addl,II=ii,RATE=rate)
  obs      <- data.frame(TIME=time,DV=NA,AMT=NA,ADDL=NA,II=NA,RATE=NA)
  out      <- rbind(dose,obs)
  out$DOSE <- doseheight[1]
  out      <- out[rep(1:nrow(out),numid),]
  out$ID   <- rep(1:numid,each=nrow(out)/numid)
  out      <- out[,c("ID", "DOSE", "TIME", "AMT", "ADDL", "II", "RATE", "DV")]
  out      <- out[order(out$ID,out$TIME),]
  if(all(is.na(out$RATE))) out <- out[,names(out)!="RATE"]  
  addarg <- list(...)
  if(length(addarg)>0){
    for(i in seq_along(addarg)) out[,names(addarg)[i]] <- addarg[[i]]
  }
  return(out)
}

#------------------------------------------ get_param ------------------------------------------
#' Get parameter values from model or ext file
#'
#' This function gets the parameter values from a NONMEM model or ext file including naming and additional
#' variables if present
#'
#' @param model character vector with the model content
#' @param lstblock list with each item being a separate structured dollar block, usually obtain from \code{\link{nmlistblock}}
#' @param ext character with the name of the NONMEM ext file (if not provided estimates are read directly from the list block)
#' @param addparam logical indicating if the function should try to add parameters (besides THETAs and the ones defined in covariates)
#'  the additional parameters are always returned so it can be used for warnings
#'
#' @export
#' @return a list with parameters, names and matrices
#' @author Richard Hooijmaijers
#' @examples
#'
#' mod    <- system.file("example_models","PK.1CMT.ORAL.COV.mod", package = "amp.sim")
#' mdll   <- get_nmblock(mod,block=c("PK","DES"))
#' mdlls  <- nmlistblock(mdll)
#' get_param(mod, mdlls)
#' 
get_param <- function(model,lstblock,ext=NULL,addparam=TRUE){
  modst   <- lapply(lstblock,function(x) paste(unlist(x)[names(unlist(x))=="orig"],collapse="\n"))
  modst   <- unlist(strsplit(paste0("$",names(modst)," ",modst),"\n"))
  if(is.null(ext)) mpars <- model else mpars <- ext
  estm    <- get_est(mpars)
  estmm   <- try(get_est(modst),silent=TRUE)
  estma   <- estm$THETA
  ome     <- estm$OMEGA
  dimnames(ome)  <- list(paste0("ETA",1:nrow(ome)),paste0("ETA",1:nrow(ome)))
  omestr  <- ome
  omestr[upper.tri(omestr)] <- ""
  omestr  <- apply(omestr,1,paste,collapse=" ")

  if(!is.null(estm$SIGMA)){
    sigm    <- estm$SIGMA
    dimnames(sigm) <- list(paste0("EPS",1:nrow(sigm)),paste0("EPS",1:nrow(sigm)))
    sigmstr <- sigm
    sigmstr[upper.tri(sigmstr)] <- ""
    sigmstr <- apply(sigmstr,1,paste,collapse=" ")
  }else{
    sigm <- sigmstr <- NULL
  }

  # Leave out the covariates argument, this should be entirely handled using addparam
  # estma   <- c(estma,covariates)
  allpar  <- get_parmvar(lstblock,returnall=TRUE)
  addpar  <- setdiff(get_parmvar(lstblock),names(estma))
  addpar  <- addpar[!addpar%in%c(get("reserved1",envir=.simenv),"F")] # F is reserved in nm error block
  addparv <- stats::setNames(rep(-999,length(addpar)),addpar) # set unkown values to -999, more likely to fail to ensure user changes this!
  if(addparam) estma <- c(estma,addparv)
  if("try-error"%in%class(estmm) || length(estmm$THETA)!=length(estmm$THETAN)){ # any(is.na(estmm$THETA)): although not perfect NAs can be present for naming
    nams <- ""
  }else{
    nams <- dput2(stats::setNames(trimws(estmm$THETAN),names(estmm$THETA)),TRUE,"theta_names")
  }
  return(list(params=estma,theta_names=nams,omega_matrix=ome,omega_string=omestr,sigma_matrix=sigm,sigma_string=sigmstr,add_params=addpar,all_params=allpar))
}

#------------------------------------------ get_inits ------------------------------------------
#' get the initial states for the differential equations
#'
#' This function will extracts the initial states for the differential equations from a NONMEM model
#'
#' @param lstblock list with each item being a separate structured dollar block, usually obtain from \code{\link{nmlistblock}}
#'
#' @export
#' @return a named vector with the state values
#' @author Richard Hooijmaijers
#' @examples
#'
#' mod    <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' mdll   <- get_nmblock(mod,block=c("PK","DES"))
#' mdlls  <- nmlistblock(mdll)
#' get_inits(mdlls)
#'
get_inits <- function(lstblock){
  inits1 <- unlist(sapply(lstblock$DES,function(x) return(x$LHS[grepl("DADT\\(.*\\)",x$LHS)])))
  inits1 <- paste0("A",gsub("DADT\\(|\\)","",inits1))
  inits1 <- stats::setNames(rep(0,length(inits1)),inits1)
  inits2 <- unlist(lapply(lstblock$PK,function(x) stats::setNames(trimws(x[x$type=="init"][['RHS']]),x[x$type=="init"][['LHS']])))
  if(!is.null(inits2)){
    names(inits2) <- gsub("_0\\(|\\)","",names(inits2))
    inits <- inits1
    inits[match(names(inits2),names(inits))] <- convert_nmsyntax(inits2)
  }else{
    inits <- inits1
  }
  return(inits)
}

#------------------------------------------ sample_sim ------------------------------------------
#' Sample model parameters for a simulation
#'
#' This function samples parameters for a simulation. It wraps around the [sample_par] function on the background
#'
#' @param nrepl Number of replicates for the simulation
#' @param nsub Number of subjects for the simulation
#' @param type character with the type of simulation to perform (see details)
#' @param ... Additional arguments passed to [sample_par]
#'   mainly for passing information for ext and cov files
#'
#' @details This function is a high level function wrapper for the \code{sample_par} function specified
#' for different types of simulations that might occur. Currently the following situations can be sampled:
#' - `noIIV`: Sample without uncertainty and without IIV
#' - `sameIIV`: Sample without uncertainty and with the same IIV values within subjects
#' - `varIIV`: Sample without uncertainty and with different IIV values within subjects
#' - `unc_noIIV`: Sample with uncertainty and without IIV
#' - `unc_sameIIV`: Sample with uncertainty and and with the same IIV values within subjects
#' - `unc_varIIV`: Sample with uncertainty and with different IIV values within subjects
#' 
#' In all the cases above where uncertainty is sampled, this is done only for THETA values.
#'
#' @export
#' @return a dataframe with sampled values
#' @author Richard Hooijmaijers
#' @examples
#'
#' ext <- system.file("example_models","PK.1CMT.ORAL.COV.ext", package = "amp.sim")
#' cov <- system.file("example_models","PK.1CMT.ORAL.COV.cov", package = "amp.sim")
#' sample_sim(nrepl=2,nsub=3,type="unc_varIIV", ext=ext,cov=cov)
#' sample_sim(nrepl=2,nsub=3,type="sameIIV", ext=ext)
#'
sample_sim <- function(nrepl=2,nsub=3,type="noIIV",...){
  if(type=="noIIV"){
    ret     <- sample_par(...,nrepl=nsub*nrepl)
    ret$REP <- rep(1:nrepl,each=nsub)
    ret$ID  <- rep(1:nsub,nrepl)
  }else if(type=="sameIIV"){
    ret     <- sample_par(...,inc_eta = TRUE,nrepl=nsub)
    ret     <- ret[rep(1:nsub,nrepl),]
    ret$REP <- rep(1:nrepl,each=nsub)
  }else if(type=="varIIV"){
    ret     <- sample_par(...,inc_eta = TRUE,nrepl=nsub*nrepl)
    ret$REP <- rep(1:nrepl,each=nsub)
    ret$ID  <- rep(1:nsub,nrepl)
  }else if(type=="unc_noIIV"){
    ret     <- sample_par(...,uncert=TRUE,nrepl=nrepl)
    ret     <- ret[rep(1:nrow(ret),nsub),]
    ret$REP <- ret$ID
    ret     <- ret[order(ret$REP),]
    ret$ID  <- rep(1:nsub,nrepl)
  }else if(type=="unc_sameIIV"){
    reps    <- sample_par(...,uncert=TRUE,nrepl=nrepl)
    reps    <- reps[rep(1:nrow(reps),each=nsub),]
    subs    <- sample_par(...,inc_theta=FALSE,inc_eta=TRUE,nrepl=nsub)
    subs    <- subs[rep(1:nrow(subs),nrepl),]
    ret     <- cbind(reps,subs[,!names(subs)%in%"ID"])
    ret$REP <- ret$ID
    ret$ID  <- rep(1:nsub,nrepl)
  }else if(type=="unc_varIIV"){
    reps    <- sample_par(...,uncert=TRUE,nrepl=nrepl)
    reps    <- reps[rep(1:nrow(reps),each=nsub),]
    subs    <- sample_par(...,inc_theta=FALSE,inc_eta=TRUE,nrepl=nsub*nrepl)
    ret     <- cbind(reps, subs[,!names(subs)%in%"ID"])
    ret$REP <- ret$ID
    ret$ID  <- rep(1:nsub,nrepl)
  }
  return(ret[,c("REP","ID",names(ret)[!names(ret)%in%c('REP','ID')])])
}

#------------------------------------------ get_nmblock ------------------------------------------
#' Get information from dollar blocks inside NONMEM control streams
#'
#' This function returns a list with indices or content of dollar blocks
#'
#' @param model character vector of length 1 with filename of model, in case length is greater than 1 it is assumed to be a vector with model code
#' @param block character vector with names of the model blocks. Take into account that grep is used with respect to partial matching
#' @param ret character with the type of return value can be either "content" or "index"
#' @param omitbn logical indicating if the name of the block should be omitted when return (has only effect if ret="content")
#'
#' @export
#' @return a list with either a numeric vector with the indices or a character vector with the content of the dollar blocks
#' @author Richard Hooijmaijers
#' @examples
#'
#' mod  <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' get_nmblock(mod,"OMEGA")
get_nmblock <- function(model,block,ret="content",omitbn=TRUE){
  aliases <- list(c("PROBLEM", "PROB"), c("SUBROUTINES", "SUBS","SUB"), c("ESTIMATE","ESTM", "EST"))
  if(length(model)==1) {rmdl <- readLines(model)}else{rmdl <- model}
  rmdl <- iconv(rmdl, "latin1", "ASCII", sub="") # want to get rid of non-ASCII characters
  alldoll  <- c(grep("^[^;]*\\$",rmdl),length(rmdl)+1)
  ret      <- lapply(block,function(x){
    aliaspres <- sapply(aliases, function(nm) any(nm%in%x))
    if(any(aliaspres)) forsub <- paste0("\\$",unlist(aliases[aliaspres]),collapse="|") else forsub <- paste0("\\$",x)
    retblock <- unlist(lapply(grep(paste0("^[^;]*",forsub),rmdl),function(y) y:(min(alldoll[alldoll>y]-1))))
    if(ret=="content") {
      retblock <- rmdl[retblock]
      if(omitbn){trimws(sub(forsub,"",retblock))}else{retblock}
    }else if(ret=="index"){
      retblock
    }
  })
  names(ret) <- block
  return(ret)
}

#------------------------------------------ nmlistblock ------------------------------------------
#' convert a block to a list
#'
#' This function will convert a NONMEM block to a list including the type and separate formula parts
#'
#' @param dollmodel list with each item being a separate dollar block, usually obtain from \code{\link{get_nmblock}}
#'
#' @export
#' @return a list with the structured code
#' @author Richard Hooijmaijers
#' @examples
#'
#' nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
#' lst   <- get_nmblock(nmmod, block = "PROB")
#' nmlistblock(lst)
#' 
nmlistblock <- function(dollmodel){
  finret <- lapply(1:length(dollmodel),function(blck){
    if(length(dollmodel[[blck]])==0){
      ret <- list(list(orig="",type="",LHS="",RHS="",comm="",cntrl="",dupl=FALSE))
    }else if(names(dollmodel)[blck]=="EST"){
      ret <- list(list(orig=paste(dollmodel[[blck]],collapse=" "),type="",LHS="",RHS="",comm="",cntrl="",dupl=FALSE))
    }else{
      ret  <- lapply(dollmodel[[blck]],function(x){
        type <- ""
        xnc  <- sub(";.*","",x) # strip comments for accurate obtaining type of line
        if(grepl("=",xnc)) type <- "formula"
        if(grepl("A_0\\([[:digit:]]*\\)",xnc)) type <- "init"
        if(grepl("IF\\(|IF[[:blank:]]*\\(|ENDIF|ELSE IF|ELSE$|END IF$",xnc)) type <- "controlflow"
        #if(grepl("IF\\(|IF[[:blank:]]*\\(",xnc) & grepl("=",xnc)) type <- "control+formula"
        if(grepl("IF\\(|IF[[:blank:]]*\\(",xnc) & grepl("(?<![=])=(?![=])",xnc, perl=TRUE)) type <- "control+formula"
        # Take into account verbatim starting with " handle is at a comment for now
        if(grepl("^;",trimws(x)))  type <- "comment"
        if(grepl("^\"",trimws(x))) type <- "comment"
        if(trimws(x)=="")  type <- "empty"
        comm  <- strsplit(x,";")[[1]]
        comm  <- ifelse(length(comm)>1,paste(comm[2:length(comm)],collapse=";"),"")
        if(grepl("^\"",trimws(x))) comm <- paste(x, "-- verbatim code commented by amp.sim")
        cntrl <- ""
        if(type=="controlflow")     cntrl <- gsub(";.*","",x)
        if(type=="control+formula"){
          clbr  <- pos_clpar(xnc) 
          cntrl <- substr(xnc,1,clbr)
          #cntrl <- gsub("=.*","",xnc)
          #cntrl <- sub("\\)([^\\)]*)$",")",cntrl)
        }
        if(type%in%c("formula","control+formula","init")){
          if(type=="control+formula") xnc <- trimws(substring(xnc,clbr+1))
          LHS   <- gsub("=.*","",xnc)
          #LHS   <- gsub("IF.*\\(.*\\)","",LHS)
          LHS   <- trimws(LHS)
          RHS   <- gsub(".*=","",xnc)
          RHS   <- gsub("ERR(\\([[:digit:]]*\\))","EPS\\1",RHS) # Make sure residual is always coded as EPS
        }else{
          LHS <- RHS <- ""
        }
        if(type=="") type <- "comment"
        return(list(orig=x,type=type,LHS=LHS,RHS=RHS,comm=comm,cntrl=cntrl,dupl=FALSE))
      })
      retd <- duplicated(sapply(ret,"[[","LHS"))
      ret <- lapply(1:length(ret), function(x){
        cur      <- ret[[x]]
        cur$dupl <- retd[x]
        cur
      })
    }
    return(ret)
  })
  names(finret) <- names(dollmodel)
  return(finret)
}

---
title: "Shiny apps"
format: 
  html: 
    toc: true
vignette: >
  %\VignetteIndexEntry{Shiny apps}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

## Introduction

Shiny apps are a great way of visualizing a (translated) model. The main advantage of a shiny app for a simulation model is that you can easily test the effect of different parameters on the outcome of a simulation. When translating a model using the `amp.sim` package, there is a standardized way models are created. This makes it possible to automatically create a shiny app for such a model.

## Basics

To create a shiny app for a model, you will need a translated model and a single function call:

```{r eval=FALSE}
library(amp.sim)
mod2shiny(parvector = c(THETA1=0.1,THETA2=0.3),
          modfile   = 'model.cpp',
          evnt      = ev(amt = 100, ii = 24, addl = 1),
          naming    = c(THETA1 = "KA (1/h)", THETA2 = "CL (l/h)"),
          framework = "mrgsolve"
          outloc    = "simApp")
```

The most important parameters for the function are:

- parvector: vector with the parameter values used in the model with the initial values to set in the app
- modfile: The translate model available in a separate file
- evnt: The events that should be used for the simulations. The app uses default dosing provided here, manual adaptations to the app for this are possible afterwards
- naming: by default the names from the parvector argument are used, when naming is provided more sensible names for parameters can be provided
- framework: because of differences between the simulation packages, the framework in which the modfile is created should be provided here
- outloc: File path where the resulting app files should be written to

Once the function is called, the appropriate app files are created (e.g. ui.r/server.r and directories to store app files).
Instructions are shown on the screen how you can directly run the shiny app, which is basically just a call to `shiny::runApp`.

## App appearance

When the app is created an app will be created that will look something like:

![example screenshot](shinyscreenshot.PNG){#fig-shiny}

<!-- ```{r echo=FALSE, out.width="100%"} -->
```{r echo=FALSE, out.width="100%"}
#knitr::include_graphics("./shinyscreenshot.PNG")
#knitr::include_graphics(paste0(getwd(),"/shinyscreenshot.PNG"))
#knitr::include_graphics(file.path("..","man", "figures", "shinyscreenshot.PNG"))
#knitr::include_graphics(system.file("shinyscreenshot.PNG",package = "amp.sim"))
```
<!-- ![](./shinyscreenshot.PNG) -->

The app is basic but already has some features build into it:

- Overlaying; When looking at the outcome of different parameter values you often want to compare different simulations. For this reason there is an option to overlay simulations. This way you can directly compare what would happen if you simulate with different settings
- settings; When overlaying simulations it can become difficult to see what the different settings were for the various simulations. The app provide a way to display the settings for all the various simulations
- Refreshing; although an entire shiny app can be refreshed manually, this option will only reset the parameter values to their original values.
- General customization; The app will contain placeholders that you can use to make the app more usable. There is a tab available where you can provide information for the user and where a model scheme can be added as background information

## Manual adaptations

Once the app is created you can directly use it for simple type of simulations. However in many cases you would want to make manual adaptations the app. Because the `mod2shiny` function creates the applicable app files, it is very easy to make manual adaptations to these files.
In the chunk below we can see how we can implement different dose heights by adjusting only two lines of code:

```{r eval=FALSE}
# part added in ui.r
numericInput(inputId = 'DOSE', label='Dose (mg):', value=100)
# part added in server.r
events <- ev(amt = input$DOSE, ii = 24, addl = 1)
```

To add information regarding the app, you can adapt the ui.r file and adapt the part that reads "Placeholder for ...".
Finally, you can add your own logo to the app either by providing this as an argument to `mod2shiny` or by replacing the file in the www directory (using the same name). This can also be done for the model scheme, just replace the file in this directory with a same named file.
In case you want to create a more complex app, this method can be used to create an initial starting point.

---
title: "Simulations using NONMEM"
format: 
  html: 
    toc: true
vignette: >
  %\VignetteIndexEntry{Simulations using NONMEM}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---


## Introduction

This vignette describes the steps to simulate a NONMEM model using the `amp.sim` package. Although the example is relatively simple, it should showcase the 
general principle that can be applied to any type of model. The `amp.sim` package include various functions that can be used to help 
in the otherwise manual tasks when simulating solely with NONMEM. 
The functions are set-up in way that all sampling is done within R, and that all model parameters are added to the input file. 
This method is chosen because the flexibility it provides to perform different kind of sampling methods (e.g. including uncertainty, variability or a combination). 
Once a standardized input data set is available, the creation of simulation model from any kind of NONMEM model can be automated fairly easy.

## Simulation dataset

### The basics

The basis of a NONMEM simulation is creating a valid dataset. Within NONMEM all information regarding dosing, output time-points, covariates and parameters 
can be added to a data set. The following chunk shows how the `simdata` function can be used to create a starting point for a simulation data set:

```{r}
library(amp.sim)
simd <- simdata(time = seq(0,120,1), dosetime = 0, doseheight = 100, 
                addl = 2, ii = 24, numid = 10)
```

This function will create by default a data set for one dose level (or compound, compartment, etc), if needed for multiple subjects. 
In case multiple dose levels, compartments, etc. are necessary there are different possibilities. The following example shows a couple of examples 
using some simple base R functions:

```{r}
#| eval: false
# lapply can be used for multiple repetitions
simd <- lapply(seq(300,600,1200),function(x) {
  simdata(time = seq(0,120,1), dosetime = 0, doseheight = x,
          addl = 2, ii = 24, numid = 10)
})
simd <- do.call(rbind,simd)
# .. or rbind to do a few repetitions 
simd <- rbind(cbind(simd,CMT=1),cbind(simd,CMT=2),cbind(simd,CMT=3))
```

### Additional variables

The example above creates a basic dataset with only the bare minimum for a simulation. In many cases additional variables need to be added. 
These variables can be covariates or just variables used within the control stream. There are many different ways that variables can be added 
to the data and depends on the type of simulation to perform. Some important considerations are:

1. Should all subjects receive the same value(s)?
2. Should the same subjects be present in each strata/treatment/dose group/etc.?
3. Are there correlations that should be taken into account?
4. How should time-varying covariates be handled?

This vignette will not go through all possibilities and leave it up to the reader to add variables based on the type of simulation.
However, be aware to add all variables used in a control stream in this step to prevent NONMEM from crashing. Some simple examples are 
included here for the creation of additional variables:

```{r}
#| eval: false
simd$WEIGHT <- 70
simd$TRT    <- sample(1:3,10,replace=TRUE)
simd$AGE    <- as.integer(rnorm(10,39,5))
```

### Inclusion of model parameters

Within the last step the model parameters are added to the data set. To do so the `sample_par` function can be used. 
The function takes the NONMEM ext file, which include the final parameter estimates, as input. 
It is possible to sample from the covariance matrix to take into account parameter uncertainty. 
In this case the covariance matrix (NONMEM cov file) should also be provided to the function. The following chunk show some examples on how the function can be used:

```{r}
#| eval: false
# Simple sampling for 10 subjects with the same THETAs but different ETAs
samp <- sample_par("run1.ext",inc_eta=TRUE,nrepl=10)

# We could include sampling with uncertainty and omit ETA values
# samp <- sample_par("run2.ext","run2.cov",inc_eta=FALSE,nrepl=10,uncert = TRUE)

# Or have uncertainty and variability
# samp <- sample_par("run2.ext","run2.cov",inc_eta=TRUE,nrepl=10,uncert = FALSE)

# Or use the result from a bootstrap to sample uncertainty
# samp <- sample_par("run2.ext",bootstrap=".", uncert = TRUE)

# In case of clinical trial simulations where a combination of replicates and subjects
# should be used, the sample_sim function can be used 
# samp <- sample_sim(ext="run2.ext",cov="run2.cov", type="unc_sameIIV", nrepl=10, nsub=20)
```

The resulting data frame will include $\theta$ and/or $\eta$ values based on the settings. The naming will always be prepended with the 
letter 'S' (for simulated or sampled) which is important for subsequent steps.

In case residual error should be included in the simulations it might be necessary to adapt the sampled dataset within this step. 
This is the case when residual error is coded as $\theta$ and uncertainty is included. 
In these cases the $\theta$ value should likely be set to a single value (population estimate). 
In case residual is coded as $\sigma$ the simulation model might need adaptation but more on that later.

At this point the simulation dataset can be combined with the parameter estimates to create a final dataset and export the result to a csv file:

```{r}
#| eval: false
inp    <- merge(simd,samp)
inp    <- inp[order(inp$ID,inp$TIME),]
write.csv(inp,csv="sim.input.csv",row.names = FALSE, na = ".")
```

The resulting dataset would look something like this:
```{r}
#| eval: true
#| echo: false
library(amp.sim)
simd <- simdata(time = seq(0,120,1), dosetime = 0,
                doseheight = 100, addl = 2, ii = 24, numid = 10)
simd$WEIGHT  <- 70
simd$AGE     <- as.integer(rnorm(10,39,5))
simd$STHETA1 <- 0.234
simd$STHETA2 <- 1.25
simd$STHETA3 <- 30.1
simd$SETA1   <- round(rnorm(nrow(simd),sd=0.2),3)
simd$SETA2   <- round(rnorm(nrow(simd),sd=0.1),3)
```
```{r}
#| eval: true
head(simd)
```

## Simulation model

In the previous step we saw that sampling of $\theta$ and $\eta$ values was done in R and added to the dataset. 
This means that it is not necessary to do sample $\theta$ or $\eta$ values in NONMEM. 
One important characteristic of this method is that we do not use certain dollar blocks. 
Furthermore, the original model needs to be rewritten so that the parameters are read from the dataset. 
These things are fairly easy to automate and for this reason the `make_nmsimmodel` function is available:

```{r}
#| eval: false
make_nmsimmodel("run1.mod", smod = "sim1.mod", data = "sim.input.csv")
```

In the example above the original model is read-in (run1.mod), adapted within R and finally written under a different name as simulation model. 
This new model should be directly ready for simulation. The only exception might be handling of residual error:

1. In case residual error should be taken into account and is coded as $\sigma$; The true sigma value should be manually replaced in the simulation model
2. In case residual error should be taken into account, and uncertainty simulation should be performed and is coded as $\theta$; 
   Within the `sample_par` function the "restheta" should be provided. In these cases the residual error is set to the population estimate to use one value for all subjects.

## Split simulations

The main idea behind setting up the simulation using this package is that there is one dataset and one simulation model. 
However in many cases the output of the simulations can be quite big, especially when uncertainty is taken into account or 
in case of a full clinical trial simulation (CTS). To overcome size problems or very long runtimes, it is possible to split the simulation 
in multiple chunks using the `split_sim` function:

```{r}
#| eval: false
split_sim(data    = "sim.input.csv",
          model   = "run2sim_final.mod",
          locout  = "simulation",
          splitby = "DOSE",
          numout  = 3)
```

Within this function, the dataset and model should be provided so that multiple copies can be made for both. Also, a variable to split on should be given and the number of outputs/models. The function will then try, as good as possible, to create equally sized chunks.
An important note here, is that you can use a data frame for `data` instead of a csv file. This is convenient because in that case you do not need to use a huge csv file that should be split. Although, You do need a csv file in order to make the initial simulation model (e.g. 'run2sim_final.mod' above). It is suggested to do this with only the head of the simulation input and then do the splitting, for example:

```{r}
#| eval: false
write.csv(head(inp),csv="sim.input.csv",row.names = FALSE, na = ".")
make_nmsimmodel("run1.mod", smod = "sim1.mod", data = "sim.input.csv")

split_sim(data    = inp,
          model   = "sim1.mod",
          locout  = "simulation",
          splitby = "DOSE",
          numout  = 3)
```

## Run the simulation

At this point, everything should be in place to perform the simulation. The only thing left to do is to run the model(s) in NONMEM. 
Obviously there are many different ways to do this, and based on the infrastructure in your organization. Therefore, the `amp.sim` package does not have any functionality for this. **Below pseudo code for a general R function to run all models within a certain folder is shown**:

```{r}
#| eval: false
# NOTE: THIS IS EXAMPLE CODE; run_models IS NOT A FUNCTION AVAILABLE IN THE PACKAGE
mods <- list.files("simulation",pattern="\\.mod$",full.names = TRUE)
run_models(mods)
```

## Final thoughts

When simulating in NONMEM a large portion of the work is creating a dataset and rewrite a model to a simulation model. 
This package helps in both tasks. The sampling of parameters is done entirely within R. This makes it easy to control the kind of sampling that is necessary. 
Also, using this method it is easier to automate the creation of a simulation model. Overall the package reduces the number of manual tasks that are necessary 
making it less error prone.
Finally, the package provides functions to set-up and perform the simulations. For post-processing of the results like calculations and plotting no functionality is present. 
This is mainly because it is too dependent on the type of project, also there are many other R packages available to perform these tasks.

---
title: "Considerations for translating models"
format: 
  html: 
    toc: true
vignette: >
  %\VignetteIndexEntry{Considerations for translating models}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

Translating model syntax from one language to another can become quite complex. This package will try to do this as good as possible, however there
might be times that things just do not work out of the box. The most important things to take into account are listed here.
If you find some situations that are not handled correctly or have some ideas how it could work please file an issue on github. 

__Model types__

There are different types of models that can be used in NONMEM. Currently the package mainly supports models that are written in differential equations (e.g \$DES block). Models written without differential equations but still user defined (e.g. \$PRED block) can be translated but likely need further inspection. For these types of models, the `deSolve` package is not really suited. The package will translate it to an R function but it is advised to simplify the result and use base R for simulations. For `rxode2` and `mrgsolve` there are more options for simulating these types of models. Likely less adaptations are necessary, but it is advised to read the additional documentation regarding these models once translated for [rxode2](https://nlmixr2.github.io/rxode2/articles/rxode2-model-types.html) and/or [mrgsolve](https://mrgsolve.org/docs/articles/extra/dollar-pred.html).

Furthermore within NONMEM there are multiple ADVAN routines for common PK models. The package will not translate models defined using the general ADVANS (e.g. models without \$DES block). The main reason for this is that translating these types of models is generally fast and easy. Also the `rxode2` and `mrgsolve` packages both have options to handle these types of models. To simulate these types of models in `deSolve` or directly in base R, the package provide template models (see `?tmpl_model`).
When such a model is used in the `convert_nonmem` function, you would typically get the results for the blocks that can be translated, but the model will likely not compile. IT can be helpful though to translate parts that can easily be added to the the default models.

__Lag time, bio-availability, residual error and infusions__

Because both `rxode2` and `mrgsolve` are focused towards pharmacometric models, all specifics regarding lag time, bio-availability, etc. should be handled correctly. However when working with `deSolve` there might be some considerations that should be take into account. When a bio-availability parameter is available, you should check if this parameter should be adjust in the model or should be added to the dosing part. Similar for lag time, this can be added in the dosing part by delaying all given doses by the lag time. Residual error can be added manually quite easily within the model or afterwards. With respect to IV infusions, this is something that should be taken into account in both the event dataset and within the model. The event dataset should define a rate of the infusion while in the model an additional compartment should be included. Some examples on how to handle these specifics when working with `deSolve` are given below:

```{r eval=FALSE}
# residual error and bio-availability can be added in the des_func e.g.
CP  <- (A2/V) * F1 * (1 + rnorm(length(A2),0,sqrt(0.01)))
# lag time should be adapted in the events data frame e.g.
events      <- dose_func(cmt=1,value=300,ndose=3,tau=24)
events$time <- events$time + pars['TLAG']
# infusions should be handled in the event and are supported in dose_func
# see template models how this can be adapted in the model part (?tmpl_model)
dose_func(cmt=4,value=300,ndose=3,tau=24, tinf=2)
```


__Specific NONMEM coding__

Within NONMEM there are different ways of coding certain blocks. For instance there are many ways of defining initial estimates for $\theta$ and $\omega$ values. The package supports most common coding trough the wonderful `NMdata` package but maybe not all. It is therefore advised not to use very specific coding here (e.g. defining $\theta$ as `(value)xn` to repeat estimates). For $\omega$ values, blocks are supported but using the 'block same' structure to code inter-occasion variability might need some double checking within the simulations. Also structures defining correlations instead of covariance will likely need adaptations.

Within NONMEM it is also possible to write abbreviated FORTRAN code (e.g. including things like `COM(n)` and/or `$ABBREVIATED`). Additionally, the usage of protected functions in R could lead to possible issues in the translated model (e.g. `PROTECT` option in `$ABBREVIATED`). 
This part is currently not supported by the package. In some cases the code is commented out in the translated models, however in certain cases it might be necessary to deleted/rewrite before translating the model.

The most important reserved keywords are taken into account. However given the amount of reserved keywords (not only in NONMEM but in the simulation packages as well), it could be that certain keywords are not supported. Please file an issue if you stumble upon an example that does not work.

__General__

Please be aware that the package reduce the need of manual translating. It is however advised to always carefully go through your simulation model to see if everything works as expected. Some parts are difficult or nearly impossible to translate. However if you have the idea that things can be implemented or do not work as expected you can always file an issue on github and I will try to handle it as good as possible.

---
title: "Simulations using R"
format: 
  html: 
    toc: true
vignette: >
  %\VignetteIndexEntry{Simulations using R}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

## Introduction

This vignette describes how pharmacometric models can be simulated in R by using the `amp.sim` package. In the field of pharmacometrics, models are often 
estimated using the NONMEM software tool. However, simulating such models is more often done in R. This method is commonly preferred because an 
entire exercise can be done in one environment. There might also be advantages with respect to efficiency (object sizes and speed).

When simulating in R there are different packages available to do so. Although there are more, the `amp.sim` package support the following regularly used packages, 
each having their own pros and cons:

- `deSolve` package: Widely used general solver package. Can be slow when models are not compiled and does not have specific features for PKPD modeling. 
  For simple models this package can be a good choice
- `rxode2` package: Package specified towards PKPD modeling. Simulations are fast and is the backbone for the estimation package `nlmixr2`. 
  For models that are estimated and simulated solely in R this can be a good choice
- `nlmixr2` package: This is is a similar method as the previous, however the syntax is created using the `nonmem2rx` package. 
  This will create a model not only suitable for simulations but also directly for estimation using the `nlmixr2` package.
- `mrgsolve` package: Package specified towards PKPD model. Simulations are also fast. 
  Syntax for models is highly comparable to NONMEM syntax. When comparing with NONMEM syntax this can be good choice

This vignette highlights how the different packages can be used and provide some examples for different type of simulations.

## Model translation

A main feature of the package is the translation of NONMEM models to one of the frameworks discussed above.
Within this translation step, the model itself is translated and a starting point for controlling the simulations is provided. 
To perform the translation, the `convert_nonmem` function can be used. This function will translate the model as good as possible and 
create all applicable code to perform an initial simulation:

```{r}
#| eval: false
library(amp.sim)
convert_nonmem("run1.mod", "example1", type_return = "deSolve")
# convert_nonmem("run1.mod", "example2", type_return = "rxode2")
# For rxode2/nlmixr2 it is advised to let the nonmem2rx package
# do the translation, added value is that the model can also be use for fitting! 
convert_nonmem("run1.mod", "example2", type_return = "nonmem2rx")
convert_nonmem("run1.mod", "example3", type_return = "mrgsolve")
```

When the function is called, by default a file including the model will be generated (e.g. example1.r) and a separate file to control the simulation (e.g. example1_control.r). 
The control file can usually be directly submitted in R to perform a simple simulation and plot the result.
In case final estimates from a NONMEM run should be used for the simulations, a NONMEM ext file can be provided to the function. 

Be aware that the function tries to translate a NONMEM model as good as possible. There might however be cases that manual adaptations are necessary, 
simply because they are not supported by the package that performs the simulation. Depending on the package being used for simulation, 
the following items might need additional attention:

- Handling of (time-varying) covariates 
- Usage lag time and bio-availability (mainly applicable for deSolve)
- Handling of mixture models 
- Handling of inter-occasion variability
- Handling of NONMEM reserved keywords or syntax

For more information on this see [Considerations for translating models](Vign.04.options_restrictions.html).

## Simulation of typical subject

For the remainder of the vignette, a translation to an `mrgsolve` model is used. The main principles however holds for the other packages as well.
The step below shows the original model and the result of the translated model:

```{r}
#| eval: true
#| echo: false
library(amp.sim)
ret <- convert_nonmem(system.file("example_models/PK.1CMT.ORAL.COV.mod",package = "amp.sim"), 
                      paste0(tempdir(),"/example"), overwrite = TRUE,
                      type_return = "mrgsolve",control="string",mod_return = "CP")
mdl <- readLines(paste0(tempdir(),"/example.cpp"))
ret[grepl("mread",ret)] <- "mod <- mread(\"example.cpp\")"
```

```{r}
#| eval: true
#| echo: false
#| class-output: fortranfree
ret <- readLines(system.file("example_models/PK.1CMT.ORAL.COV.mod",package = "amp.sim"))
cat(ret, sep="\n")
```


```{r}
#| eval: false
library(amp.sim)
convert_nonmem(system.file("example_models/PK.1CMT.ORAL.COV.mod",package = "amp.sim"), 
               "example", type_return = "mrgsolve", mod_return = "CP")
```
```{r}
#| eval: true
#| echo: false
#| class-output: cpp
#cat(ret,sep="\n")
cat(mdl,sep="\n")
```

When we run the control script (without adaptations) and plot the results:

```{r}
#| echo: false
#| fig-width: 6
#| message: false
library(mrgsolve)
library(ggplot2)
mod  <- mread(paste0(tempdir(),"/example.cpp"))
evnt <- ev(amt = 100, ii = 24, addl = 1)
out  <- mod |> ev(evnt) |> mrgsim(end = 48, delta = 0.1, nid=10)
ggplot(out@data,aes(time,CP)) + geom_line() + scale_y_log10() + theme_bw()
```

We see that in this case, that the simulations are empty. This is because the model has a covariate of weight. Because the translation does not take into account the dataset,
the package cannot set a valid value and is therefore set to -999. 
Below we can see the simple addition to get a result. Other adaptations showcased are, simulate a single subject without random effects (using `zer0_re`):

```{r}
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
library(ggplot2)
parm <- c(WEIGHT = 70)               
mod  <- param(mod,parm)   
out  <- zero_re(mod) |> ev(evnt) |> mrgsim(end = 48, delta = 0.1, nid=1)
ggplot(out@data,aes(time,CP)) + geom_line() + scale_y_log10() + theme_bw()
```

__Take into account that in the result above the initial estimates from the model are used. We could provide the ext file to use final estimates__

## Simulation of population

In most cases the simulation should be performed for more than 1 subject. There is some variation what the different subjects represent. It could be either inter-individual variability (IIV), uncertainty or a combination of both. This section shows how various combinations can be handled.

### Simulations including only IIV

The situation where you want to simulate with only IIV, is the easiest and is already integrated within the `mrgsolve` package. By default, when a NONMEM model is translated, the model matrices included in the model. This is also used by the `mrgsim` function to sample $\eta$ values for the model. As shown in the first example the original code was adapted to set IIV to 0. If we do not do this and set a number of individuals to simulate we can easily include IIV


```{r}
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
parm <- c(WEIGHT = 70)               
mod  <- param(mod,parm)   
out  <- mod |> ev(evnt) |> mrgsim(end = 48, delta = 0.1, nid=100)
ggplot(out@data,aes(time,CP, group=ID)) + geom_line(alpha=0.3) + 
  scale_y_log10() + theme_bw()
```

#### Simulations including only uncertainty

In case we only want to simulate with uncertainty, we need to disregard IIV in the same way as in the first example. Furthermore we need to sample $\theta$ values from the covariance matrix for the uncertainty. The `amp.sim` package provides a function to do so (`sample_par`). Once we have the sampled parameters, it is a matter to simulate each row of the data frame with the sampled parameters. In this example the `dplyr` package is used to accomplish this:

```{r}
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false

library(dplyr)
parm  <- c(WEIGHT = 70)
mod   <- param(mod,parm)   

# sample from uncertainty and apply simulations
extf  <- system.file("example_models/PK.1CMT.ORAL.COV.ext",package = "amp.sim")
covf  <- system.file("example_models/PK.1CMT.ORAL.COV.cov",package = "amp.sim")
parms <- sample_par(ext = extf, covmat = covf, nrepl = 100, uncert = TRUE) |> 
  rename_with(~ sub("S", "", .x))

dosim <- function(df){
  zero_re(mod) |> ev(evnt) |>
    param(c(unlist(df),parm)) |> 
    mrgsim_df(end = 48, delta = 0.1, nid=1) |>
    mutate(ID = unique(df$ID))  
}

out <- parms |> group_by(ID) |>
  group_map(~dosim(.x),.keep=TRUE) |> bind_rows()

ggplot(out,aes(time,CP,group=ID)) + geom_line(alpha=.3) + 
  scale_y_log10() + theme_bw()
```


#### Simulations including IIV and uncertainty

For larger clinical trial simulations it is often necessary to simulate including both IIV and uncertainty. This section describes a way to do so. Also running these types of simulation can be computationally intensive. 
To speed up the simulation an example is included that uses the `parallel` package, although various other frameworks could be used
to multi-thread the simulations.
This example combines the examples above where in this case 10 trials are simulated for 10 subjects each

```{r}
#| eval: false
parm  <- c(WEIGHT = 70)
extf  <- system.file("example_models/PK.1CMT.ORAL.COV.ext",package = "amp.sim")
covf  <- system.file("example_models/PK.1CMT.ORAL.COV.cov",package = "amp.sim")

parms <- sample_par(ext = extf, covmat = covf, nrepl=10, uncert=TRUE)
parms <- setNames(parms,sub("S","",names(parms)))

library(parallel)
cl  <- makeCluster(getOption("cl.cores", 7))
clusterExport(cl, c("evnt","parms","mod","parm"))
out <- parLapply(cl, 1:nrow(parms),function(x){
  library(mrgsolve)
  ret     <- mod |> ev(evnt) |> param(c(unlist(parms[x,]),parm)) |>
               mrgsim_df(end = 48, delta = 0.1, nid=10)
  ret$REP <- unique(parms$ID[x])
  ret
})
stopCluster(cl)

out <- do.call(rbind,out)
ggplot(out,aes(time,CP,group=ID)) + geom_line(alpha=.3) + 
  scale_y_log10() + facet_wrap("REP",labeller = "label_both") + 
  theme_bw()

```


```{r}
#| echo: false
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
# Perform actual sims without parallel, because do not want to compile it (on CRAN) with this package
parm  <- c(WEIGHT = 70)
parms <- sample_par(ext    = system.file("example_models/PK.1CMT.ORAL.COV.ext",package = "amp.sim"),
                    covmat = system.file("example_models/PK.1CMT.ORAL.COV.cov",package = "amp.sim"),
                    nrepl=10, uncert=TRUE)
parms <- setNames(parms,sub("S","",names(parms)))
out <- lapply(1:nrow(parms),function(x){
  ret     <- mod |> ev(evnt) |> param(c(unlist(parms[x,]),parm)) |>
               mrgsim_df(end = 48, delta = 0.1, nid=10)
  ret$REP <- unique(parms$ID[x])
  ret
})
out <- do.call(rbind,out)
ggplot(out,aes(time,CP,group=ID)) + geom_line(alpha=.3) + 
  scale_y_log10() + facet_wrap("REP",labeller = "label_both") + 
  theme_bw()

```


For running in parallel, the following steps are important:

1. Create a cluster object where the numbers of cores are specified (`makeCluster`)
2. Exporting of objects to the cluster, so each worker has access to all objects used in the calculations (`clusterExport`)
3. Performing a special kind of lapply for parallel processing (`parLapply`)
4. Let R know that the cluster calculations are done (`stopCluster`)

It is some extra work to set-up a 'parallel' script, but this can really pay-off with drastically decreasing run-times. 

Within the examples above, weight is kept constant for simplicity. Obtaining valid covariates can be handled in different ways. 
There are different ways of sampling values and provide this to the simulation functions. Please refer to the help pages of the different packages for additional guidance.

In the example above, the $\eta$ values are sampled within the model. It is also possible to perform all sampling within R using the `sample_par` function. In case a clinical trial simulation should be done using a combination of replicates and subjects, it is more convenient to use the `sample_sim` function instead. This function is a wrapper around `sample_par` but makes it easier to make a combined sampling dataset of replicates and subjects. 

### Using a template model

In case a closed form model should be simulated, the previous mentioned methods do not work. For these cases different template models are present in the `amp.sim` package. It includes 1 and 2 compartment models for IV, infusion and extra-vascular dosing, parameterized using rate constants or CL/V, and as analytical solution or differential equations. The `tmpl_model` function can be used without arguments to see what is available:

```{r}
tmpl_model()
```

A model can be provided to the function, in which case the simulation code can be added in the current script right after the function call (when working in Rstudio), within the console or as a string:

```{r}
#| eval: false
tmpl_model("ana1CMTivC.tmp")
```

When submitting the lines of code you will directly get an initial simulated curve:

```{r}
#| echo: true
#| fig-width: 6
#| fig-height: 4
#| message: false
#| warning: false
library(ggplot2)
ana1CMTiv <- function(Dose,pars,t,dur){
  C <- 1/pars['V']
  L <- pars['CL']/pars['V']
  ifelse(t <= dur,
    (Dose/dur) * C / L * (1-exp(-L * t)),
    (Dose/dur) * C / L * (1-exp(-L * dur)) * exp(-L * (t-dur))
  )
}
pars   <- c(CL=1,V=1)
times  <- seq(0,24,length.out=200)
out    <- mdose(Dose = 10, tau = 24, ndose = 1, t = times,
                func = ana1CMTiv, pars = pars, dur = 2)
ggplot(out,aes(time,y)) + geom_line() + theme_bw()

```

This method can be a lot faster for models with an analytical solution, though simulating multiple doses is a bit more of a hassle. For these situations the `mdose` helper function can be used to perform superposition.
Finally, a simple example is shown how these models can be rewritten for simulating a population:

```{r}
#| eval: false
samp <- data.frame(CL=rnorm(100,10,1),V=rnorm(100,5,0.5))
out <- lapply(1:nrow(samp), function(x){
  data.frame(mdose(Dose=10,tau=24,ndose=1,t=times,func=ana1CMTiv,
                   pars=unlist(samp[x,]),dur=2),ID=x)
})
out <- do.call(rbind,out)
```


### Final thoughts

Simulating entirely in R can be quite some work. The `amp.sim` can help in reducing the amount of work to rewrite everything to R, and makes it easier to perform a simple simulation. Because R is different from NONMEM there are important things to take into account. When the simulations become larger it can also become more difficult to perform the simulation in R and the run-times can get out of hand. But the usage of template models (mainly the ones with an analytical solution) and the `parallel` package can help in optimizing performance.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.r
\name{pos_clpar}
\alias{pos_clpar}
\title{Get position of closing parenthesis}
\usage{
pos_clpar(x)
}
\arguments{
\item{x}{character string for which the parenthesis should be searched}
}
\value{
a numeric with the position of the closing parenthesis (will be -1 if no match is present)
}
\description{
This function gets the position of the closing parenthesis after the first opening one
}
\examples{

tst <- "IF (test == A(1)) a(1)=(1*5)/2"
pos_clpar(tst)
substring(tst,1,pos_clpar(tst))

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmlistblock.r
\name{nmlistblock}
\alias{nmlistblock}
\title{convert a block to a list}
\usage{
nmlistblock(dollmodel)
}
\arguments{
\item{dollmodel}{list with each item being a separate dollar block, usually obtain from \code{\link{get_nmblock}}}
}
\value{
a list with the structured code
}
\description{
This function will convert a NONMEM block to a list including the type and separate formula parts
}
\examples{

nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
lst   <- get_nmblock(nmmod, block = "PROB")
nmlistblock(lst)

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conv_pow.r
\name{conv_pow}
\alias{conv_pow}
\title{Convert infix power notation to prefix notation}
\usage{
conv_pow(x)
}
\arguments{
\item{x}{character vector with the formulas to convert}
}
\value{
a vector with the transformed power notations
}
\description{
This function will transform the power notation that can be used in R to the one needed by mrgsolve
}
\examples{

conv_pow("y = par1*(par2/par3)^xy + a - par4**(2/par5)")
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simdata.r
\name{simdata}
\alias{simdata}
\title{Create a simulation dataset for NONMEM simulation}
\usage{
simdata(time, dosetime, doseheight, addl, ii, rate = NA, numid = 5, ...)
}
\arguments{
\item{time}{a vector with all sampling times to be used}

\item{dosetime}{a vector with the different dosing times}

\item{doseheight}{a vector with the different dosing heights (to be added to AMT)}

\item{addl}{a vector with the additional dose levels (must be same length as doseheight)}

\item{ii}{a vector with the interdose interval (must be same length as doseheight)}

\item{rate}{a vector with the dosing rate (must be same length as doseheight)}

\item{numid}{a vector with the number of IDs to be created}

\item{...}{additional variables to be added to the resulting dataset}
}
\value{
a dataframe that can be used for NONMEM simulations
}
\description{
This function creates a simulation dataset including information
for dosing and sampling. The function is setup to return a data frame
to be used within NONMEM or other frameworks
}
\examples{

# Include additional variables
sim1 <- simdata(seq(0,24,1),0.5,100,10,12,NA,2, WEIGHT=70, ETA=0)

# unequal dosing scheme
sim2 <- simdata(seq(0,24,1), dosetime = c(0.5,1), doseheight = c(100,200),
                addl = c(10,5), ii = c(12,24), numid = 2)

# Directly create a sequence of different dose levels
sim3 <- lapply(seq(25,60,5),function(x){
  simdata(time=1:12,dosetime=0,doseheight=x,addl=139,ii=120,rate=0,numid=10)
}) 
sim3 <- do.call(rbind,sim3)

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.r
\name{dput2}
\alias{dput2}
\title{Wrapper function around dput to provide more options}
\usage{
dput2(x, comment = FALSE, obj = NULL, collapse = NULL, ...)
}
\arguments{
\item{x}{An object passed to \code{dput}}

\item{comment}{logical indicating if comment characters should be prepended to results}

\item{obj}{character of length one indicating the name of the object that should be prepended to result}

\item{collapse}{character of length one with the collapse character. If provided the result will be pasted with this collapse character}

\item{...}{additional arguments passed to dput}
}
\value{
a vector with the result from dput
}
\description{
This function wraps around the dput function and provide more options to save result
in a vector
}
\examples{

dput2(setNames(runif(5),letters[1:5]))

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod2shiny.r
\name{mod2shiny}
\alias{mod2shiny}
\title{Creates a basic simulation app for a given R model}
\usage{
mod2shiny(
  parvector,
  modfile,
  evnt,
  init = NULL,
  naming = NULL,
  apptitle = "Shiny app title",
  outloc = ".",
  omega = NULL,
  sigma = NULL,
  delloc = FALSE,
  framework = "deSolve",
  logo = paste0(system.file(package = "amp.sim"), "/logo.png"),
  times = NULL
)
}
\arguments{
\item{parvector}{named vector with the model parameters, should contain all parameters used by the model}

\item{modfile}{A script with the model defined in it, it is assumed that this file is created using \code{\link[amp.sim]{convert_nonmem}}}

\item{evnt}{dataframe with the events used by the model (this is saved as rds together with the app)}

\item{init}{vector with the compartment initialization (only applicable for deSolve framework)}

\item{naming}{named vector with the names in parvector and the new values to use within the shiny app}

\item{apptitle}{string with the title to be used for the app}

\item{outloc}{character with location where the resulting shiny app should be saved}

\item{omega}{vector with the omega matrix for the model (only applicable for rxode2 framework)}

\item{sigma}{vector with the sigma matrix for the model (only applicable for rxode2 framework)}

\item{delloc}{logical indicating if the location should be deleted first}

\item{framework}{character indicating the simulation framework that was used, currently "deSolve", "rxode2", "nonmem2rx" and "mrgsolve" are supported}

\item{logo}{character with a png of the company logo added in the header of the shiny app}

\item{times}{character vector with the times to simulate or set to NULL to use a default time vector}
}
\value{
creates necessary app files for a shiny simulation app
}
\description{
This function fills in a default template for simulations within shiny
}
\details{
This function creates a default shiny app that can be used as a starting point for further
development. There are already some basic features available but it is intended to be adapted when used
for production. For the automatic creation of an app it is assumed that the model is created using the \link{convert_nonmem}
function. Although it is not strictly necessary, the information provided to this function will be in the correct format
when this function is used. Some of the arguments in this function are only necessary in case a certain conversion framework
is used. In general a single subject is simulated, but be aware that for the deSolve and rxode2 framework OMEGA/ETA is implemented.
}
\examples{

nmmod <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
outf  <- tempfile()
convert_nonmem(nmmod, out=outf, verbose = FALSE)
prm   <- c(THETA1 = 0.3, THETA2 = 2, THETA3 = 5)
evnt  <- mrgsolve::ev(amt = 100, ii = 24, addl = 1)
nams  <- c(THETA1 = "KA (1/h)", THETA2 = "CL (l/h)", THETA3 = "V (l)") 
mod2shiny(prm, modfile= paste0(outf,".cpp"), evnt = evnt,
          naming = nams, framework = "mrgsolve", outloc=tempdir())

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/amp.sim-package.R
\docType{package}
\name{amp.sim-package}
\alias{amp.sim}
\alias{amp.sim-package}
\title{amp.sim: Flexible Simulation Utilities for Pharmacometric Modeling}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The goal of 'amp.sim' is to transform 'NONMEM' models into R syntax so they can be used for simulations using the 'deSolve', 'nlmixr2' or 'mrgsolve' package. Additionally, functionality is included to aid simulations performed directly in 'NONMEM' and to automatically create shiny apps for simulation models.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://leidenadvancedpkpd.github.io/amp.sim/}
  \item \url{https://github.com/LeidenAdvancedPKPD/amp.sim}
  \item Report bugs at \url{https://github.com/LeidenAdvancedPKPD/amp.sim/issues}
}

}
\author{
\strong{Maintainer}: Richard Hooijmaijers \email{richardhooijmaijers@gmail.com} [copyright holder]

Other contributors:
\itemize{
  \item LAPP Consultants \email{info@lapp.nl} [funder, copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings2df.r
\name{settings2df}
\alias{settings2df}
\title{Add settings to dataframe}
\usage{
settings2df(savedsims, leaveout = c("go", "updOpts", "sett", "refr", "tabsel"))
}
\arguments{
\item{savedsims}{reactiveValue that contains the saved simulations}

\item{leaveout}{character vector with the the elements that should be left out the dataframe}
}
\value{
a dataframe with settings
}
\description{
This function adds settings or input elements to a dataframe for displaying in app
}
\examples{

if(requireNamespace("tidyr")){
  sim1 <- list(THETA1  = 0.5, THETA2 = 1,
               alllabs =c("THETA1\%=\%DUMMY1","THETA2\%=\%DUMMY2"))
  sim2 <- list(THETA1  = 0.9, THETA2 = 1.5)
  settings2df(list(sim1 = sim1, sim2 = sim2))
} 

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_nmsimmodel.r
\name{make_nmsimmodel}
\alias{make_nmsimmodel}
\title{Create a simulation model from original model}
\usage{
make_nmsimmodel(omod, smod, data, subprobs = 1, table = NULL, sigma_ext = NULL)
}
\arguments{
\item{omod}{character string for the original model}

\item{smod}{character string for the simulation model (for writing the model to disk)}

\item{data}{character string for the input dataset for the simulation}

\item{subprobs}{numeric indicating the number of subproblems in simulation model}

\item{table}{character of length 1 with the items for the dollar table block (if null it will use items in input dataset)}

\item{sigma_ext}{character with the name of the ext file which includes the sigma value (in case residual error is coded in the dollar sigma block)}
}
\value{
a simulation model (file) is created
}
\description{
This function creates a simulation model from an original NONMEM model where dollar blocks are replaced and/or
commented based on input file and other simulation functions within package
}
\details{
The function will adapt an original model in such a way that it can be used directly for simulations
the assumptions are that an input dataset is created using \link{sample_par} and \link{simdata}.
This means that all THETA and ETA values are available in the simulation dataset as respectively STHETAN and SETAN (simulated THETA/ETA n).
Furthermore it is assumed that the input dataset is a csv file.
The function will first comment all applicable dollar blocks. Then a OMEGA is added with 0 FIX and the INPUT, DATA and
TABLE blocks are appended with information based on the input dataset. Finally all THETAs and ETAs are replaced with
the items in the dataset and the simulation model is written to  disk.
}
\examples{

nmmod   <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
dat     <- simdata(0:24, dosetime = 0, doseheight = 10, addl = 2, ii = 24, 
                   numid = 50, STHETA1= 1, STHETA2 = 2, STHETA3 = 1,
                   SETA1 = 0, SETA2 = 0)
tmp_out <- tempfile(fileext = ".csv")
mod_out <- tempfile(fileext = ".mod")

write.csv(dat,file=tmp_out, na=".", quote=FALSE, row.names = FALSE)
make_nmsimmodel(nmmod, mod_out, data=tmp_out)
readLines(mod_out) |> head(n=15) |> cat(sep="\n")

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split_sim.r
\name{split_sim}
\alias{split_sim}
\title{Split a large NONMEM simulation in chunks}
\usage{
split_sim(data, model, locout, splitby = "ID", numout = 4)
}
\arguments{
\item{data}{character string for input dataset or a dataframe with the simulation data (see details)}

\item{model}{character string for the simulation model}

\item{locout}{character string for the location of the split input dataset and models}

\item{splitby}{character string with the variable in the data to be split on (if this is not ID it could lead to unexpected results)}

\item{numout}{the number of 'equal length' outputs to be created.}
}
\value{
split CSV and models files are written to disk
}
\description{
This function splits a large simulation dataset and accompanying model in chunks
to prevent memory problems within the simulation or to enable simulating over multiple cores
}
\details{
In general the data can be a character string defining the input dataset (csv file) or a dataframe
it is proposed to use a dataframe as in many cases the splitting should take place for a large problem. In case
a dataframe is used (e.g. from loading an Rdata object)  it is not necessary to import a huge csv file.
}
\examples{

nmmod   <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
dat     <- simdata(0:24, dosetime = 0, doseheight = 10, addl = 2, ii = 24, 
                   numid = 50, STHETA1= 1, STHETA2 = 2, STHETA3 = 1,
                   SETA1 = 0, SETA2 = 0)
tmp_out <- tempfile(fileext = ".csv")
mod_out <- tempfile(fileext = ".mod")

write.csv(dat,file=tmp_out, na=".", quote=FALSE, row.names = FALSE)
make_nmsimmodel(nmmod, mod_out, data=tmp_out)

split_sim(data = tmp_out, model = mod_out, locout=tempdir())
list.files(tempdir(), pattern="\\\\.mod$")

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_param.r
\name{get_param}
\alias{get_param}
\title{Get parameter values from model or ext file}
\usage{
get_param(model, lstblock, ext = NULL, addparam = TRUE)
}
\arguments{
\item{model}{character vector with the model content}

\item{lstblock}{list with each item being a separate structured dollar block, usually obtain from \code{\link{nmlistblock}}}

\item{ext}{character with the name of the NONMEM ext file (if not provided estimates are read directly from the list block)}

\item{addparam}{logical indicating if the function should try to add parameters (besides THETAs and the ones defined in covariates)
the additional parameters are always returned so it can be used for warnings}
}
\value{
a list with parameters, names and matrices
}
\description{
This function gets the parameter values from a NONMEM model or ext file including naming and additional
variables if present
}
\examples{

mod    <- system.file("example_models","PK.1CMT.ORAL.COV.mod", package = "amp.sim")
mdll   <- get_nmblock(mod,block=c("PK","DES"))
mdlls  <- nmlistblock(mdll)
get_param(mod, mdlls)

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/par_delete.r
\name{par_delete}
\alias{par_delete}
\title{Delete parenthesis including a numeric within a vector}
\usage{
par_delete(vect, excfun = TRUE)
}
\arguments{
\item{vect}{the vector that should be scanned for parenthesis}

\item{excfun}{exclude functions from deleting parenthesis (taken from the globally defined function list in the package)}
}
\value{
a vector with the stripped parenthesis
}
\description{
This function deletes all parenthesis from a character vector that has a numeric value inside.
This is applicable as THETA(n), ETA(n), DADT(n) is not always allowed in R coding
}
\examples{

par_delete(c("LOG(1)","ETA(1)","EXP(2)/ETA(3)+THETA(4)"))
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_est.r
\name{get_est}
\alias{get_est}
\title{Get model estimates from NONMEM ext or model file}
\usage{
get_est(from)
}
\arguments{
\item{from}{the model or ext file (or data.frame/model text string from results object) to be read in to obtain estimates.
extension or class of object of file determines the actions to be taken}
}
\value{
a list with theta, eta and omega values
}
\description{
This function gets the estimates (THETA, ETA and OMEGA) from a NONMEM ext or model file to be used
within the simulations in R. The model file is included as option as the names of THETAs can be
obtained in case this is set as comment in the model file
}
\details{
the function will return a list with theta, eta, omega and naming of theta values. In case a model is used as input, the values represent
the initial values from a model. In case the ext file is used, the final estimates are taken. The eta values
are all set to 0. The omega values are returned as a matrix so it can be used for sampling (e.g. using mvrnorm).
naming of thetas is taken from the model comments in the THETA block or in case an ext file is used naming is
set to THETA1:n.
In case the model is used as input, there are some assumptions within the function on how the model is coded. For the
omega block the value of omega must always be placed on a separate line (e.g. $OMEGA 0.1 is not permitted as 0.1 should be placed
on the next line. Also covariance within the omega block should be placed on the same line separated by spaces (e.g. for a BLOCK(2)
the first line should state variance eta1 and the second line should state covariance eta1, eta2 followed by variance eta2).
For the theta block, it is assumed that in case lower and upper boundaries are available they are separated by commas.
}
\examples{

# get the initial estimates from the model or final estimates from ext file
mod  <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
ext  <- system.file("example_models","PK.1CMT.ORAL.ext", package = "amp.sim")
get_est(mod)
get_est(ext)

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_nmsyntax.r
\name{convert_nmsyntax}
\alias{convert_nmsyntax}
\title{Convert NONMEM specific syntax to R syntax}
\usage{
convert_nmsyntax(x, type = "mrgsolve")
}
\arguments{
\item{x}{character vector with the syntax to be converted}

\item{type}{character with the type of syntax to convert to (currently "deSolve" and "mrgsolve" and "rxode2" are supported)}
}
\value{
character vector with the converted syntax
}
\description{
This function converts a NONMEM syntax to R syntax, mainly for functions and operators
}
\examples{

convert_nmsyntax("IF(VAR.GT.0) VAR2 = PHI(1)")
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_parmvar.r
\name{get_parmvar}
\alias{get_parmvar}
\title{Get the variables within a NONMEM model that should be passed to the simulation model}
\usage{
get_parmvar(lstblock, returnall = FALSE)
}
\arguments{
\item{lstblock}{list with each item being a separate structured dollar block, usually obtain from \code{\link{nmlistblock}}}

\item{returnall}{logical indicating if all variables should be returned or just the ones that are not defined}
}
\value{
a vector with model variables for the simulation model
}
\description{
This function go through all formulas and control flows to check if a parameter is created ad-hoc
or taken from the input file. This is important as these variables need to be set for simulation
}
\examples{

mod    <- system.file("example_models","PK.1CMT.ORAL.COV.mod", package = "amp.sim")
mdll   <- get_nmblock(mod,block=c("PK","DES"))
mdlls  <- nmlistblock(mdll)
get_parmvar(mdlls)

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdose.r
\name{mdose}
\alias{mdose}
\title{Performs multiple dosing in case of analytical solution of model}
\usage{
mdose(Dose, tau, ndose, t, func, ...)
}
\arguments{
\item{Dose}{numeric vector with the dosing height}

\item{tau}{numeric vector with the tau of dosing}

\item{ndose}{numeric vector with the number of doses}

\item{t}{numeric vector with the time-points that should be outputted}

\item{func}{name of the function for which multiple dosing should be applied}

\item{...}{arguments for func}
}
\value{
a data frame with the superimposed results
}
\description{
This function takes a function that defines a model in analytical solution and performs multiple dosing for it
}
\details{
This function will create a list that can be used to perform superposition
which is necessary in case of an analytical solution in a multiple dose setting.
The function will check if there is an overlap in arguments and will use the arguments
given to \code{mdose} for the function given in \code{func} if applicable (e.g. it is
likely that \code{func} has an argument for Dose, in this case it will use the Dose argument
provide in \code{mdose})
The function can have any number of arguments that can be passed using "...". However there
should at least be an argument \code{t} which is the time vector for which simulations are
necessary.
}
\examples{

ana1CMTiv <- function(Dose,pars,t){
  Dose * pars['C'] * exp(-pars['L']*t)
}
res <- mdose(10, tau = 24, ndose = 5, t = 0:240, 
             func = ana1CMTiv, pars = c(L=.01,C=5))
plot(res$time,res$y, type="l")

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_validation.r
\name{model_validation}
\alias{model_validation}
\title{Validates NONMEM estimation model with an mrgsolve simulation model}
\usage{
model_validation(
  nmtable,
  simmodel,
  rounding = 4,
  comppred = "CP",
  out = NULL,
  ...
)
}
\arguments{
\item{nmtable}{either a character with a file or a data frame including the NONMEM table output}

\item{simmodel}{character with the file including the mrgsolve model}

\item{rounding}{numeric with the rounding applied for comparing}

\item{comppred}{character with the variable in mrgsolve model that should be compared with PRED variable in NONMEM}

\item{out}{character with the name of the output to create}

\item{...}{additional arguments passed through to \code{mrgsim} function}
}
\value{
a file with a PDF report is returned
}
\description{
This function uses the estimates from a NONMEM run and compares this with the results of a simulation
run. This function is inspired by a blog post from mrgsolve and mainly looks at the differences in population predictions
}
\details{
For a correct comparison, the nmtable should include all variables related to dosing (e.g. AMT/CMT/EVID).
The simulation model should be available as a separate file that can be read in using \code{mrgsolve::mread}.
To use the function, the packages \code{R3port}, \code{ggplot2}, \code{mrgsolve} and \code{dplyr} should be installed.
Be aware that no variables are renamed in $TABLE in the NONMEM control stream (e.g. AMT2=AMT). This can have unexpected results when comparing.
}
\examples{

\donttest{
  res  <- model_validation(system.file("testfiles/compareParfile",package="amp.sim"),
                           system.file("testfiles/compareModel.cpp",package="amp.sim"),
                           out=NULL)
}
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaying.r
\name{overlaying}
\alias{overlaying}
\title{Performs overlaying of simulations}
\usage{
overlaying(out, input, savedsims = NULL)
}
\arguments{
\item{out}{dataframe with the results from a simulation to be appended for overlaying}

\item{input}{list with the input elements from a shiny app}

\item{savedsims}{reactiveValue or list that contains the saved simulations}
}
\value{
a list with a dataframe with the appended simulations and settings
}
\description{
This function basically appends the simulation output and add this to a reactiveValue
}
\examples{
if(requireNamespace("shiny")){
  out   <- data.frame(time=0:4, A1=rnorm(5), numsim=1)
  input <- shiny::reactiveValues(CL=2, KA=0.3, updOpts="appsim")
  overlayres <- overlaying(out,input)
  savedsims  <- list(res = overlayres$results, sett = overlayres$settings)
  overlayres <- overlaying(input=input,out=out,savedsims=savedsims)
  head(overlayres$result)
}

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_nonmem.r
\name{convert_nonmem}
\alias{convert_nonmem}
\title{Convert NONMEM model to R syntax}
\usage{
convert_nonmem(
  model,
  out,
  ext = NULL,
  mod_return = NULL,
  type_return = "mrgsolve",
  overwrite = FALSE,
  control = "file",
  verbose = TRUE
)
}
\arguments{
\item{model}{character with the model file to be read in and converted}

\item{out}{character with the name of the output file without a file extension}

\item{ext}{character with the name of the NONMEM ext file (if not provided estimates are read directly from control stream)}

\item{mod_return}{a character vector indicating which items should be returned from the model function. For more information see details}

\item{type_return}{character indicating the type of model that should be created. Currently "deSolve", "rxode2", "nonmem2rx" and "mrgsolve" are accepted}

\item{overwrite}{logical indicating if the output model should be overwritten}

\item{control}{character indicating how the model control code should be returned. Currently "file", "console", "string", "script" and "model" (only for rxode2/DeSolve) are accepted}

\item{verbose}{logical indicating if additional information is written to the console}
}
\value{
a converted file is generated and a message is returned
}
\description{
This function converts a NONMEM model to syntax useable in R simulations. Currently deSolve, rxode2 (nonmem2rx) and mrgsolve are available syntaxes to use
Additionally a code to control the simulations is created to directly test the simulations
}
\details{
For the mod_return argument, the additional variables are added to the output in case the type_return is either mrgsolve or deSolve.
}
\examples{

mod  <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
outf <- tempfile()

convert_nonmem(mod, outf, verbose = FALSE)
readLines(paste0(outf,".cpp")) |> head(n=15) |> cat(sep="\n")

convert_nonmem(mod, outf, verbose = FALSE, type_return = "nonmem2rx") |> suppressMessages()
readLines(paste0(outf,".r")) |> head(n=15) |> cat(sep="\n")

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tmpl_model.r
\name{tmpl_model}
\alias{tmpl_model}
\title{Get coding for template models}
\usage{
tmpl_model(tmpl, ret = "console")
}
\arguments{
\item{tmpl}{character with template}

\item{ret}{character indicating how the result should be returned. Currently "console", "string" and "script" are accepted}
}
\value{
model syntax is returned either to console or script (within Rstudio) or a character string
}
\description{
This function returns the code for some template models
}
\details{
There are templates available for 1-2 CMT PK models for IV/bolus/oral administration
as both analytical as closed form and parameterized with constants or as CL/V.
To see which templates are available in the package
run the function without arguments
}
\examples{

tmpl_model()
tmpl_model("ana1CMTbolusK.tmp")

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cut_equal.r
\name{cut_equal}
\alias{cut_equal}
\title{cut function for (appr.) equal intervals}
\usage{
cut_equal(x, n, type = 1, ntries = 1000)
}
\arguments{
\item{x}{the vector that should be cut into equal bins}

\item{n}{the number of bins that should be used}

\item{type}{the type of algorithm to be used (see details)}

\item{ntries}{number of samples/tries for type 2}
}
\value{
a character vector with the categories/bins
}
\description{
This function uses cut to make categories based on approximately equal number of bins
}
\details{
Generating equal bins can be quite difficult. There are multiple ways of
assessing if bins are equal. This function provides a method based on quantiles
(type=1) or a method based on sampling proposed by M. Ruppert (type=2). In general
type 1 is faster but less accurate, while type 2 is slower and more accurate
(in case of a reasonable ntries)
}
\examples{

 table(cut_equal(1:20,5))
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dose_func.r
\name{dose_func}
\alias{dose_func}
\title{Create data.frame with dosing for usage in simulation with deSolve}
\usage{
dose_func(cmt, value, tinf, tau, ndose, times)
}
\arguments{
\item{cmt}{number of compartment or differential equation where the dosing should be given}

\item{value}{the value of the dosing that should be given}

\item{tinf}{in case value is set an infusion is assumed with tinf as infusion time}

\item{tau}{dosing interval to be used}

\item{ndose}{number of doses to be used}

\item{times}{In case tau and ndose cannot be used (unequal dosing), this parameter can be
used to set times of dosing (e.g c(0,24,168))}
}
\value{
a data frame that can be used as an event dataset within deSolve
}
\description{
This function creates a data.frame that can be used for events of a deSolve simulation
}
\examples{

 dose_func(8,100,tau=48,ndose=5,tinf=2)
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_par.r
\name{sample_par}
\alias{sample_par}
\title{Sample model parameters from a multivariate normal distribution}
\usage{
sample_par(
  ext,
  covmat = NULL,
  bootstrap = NULL,
  seed = NULL,
  nrepl = 10,
  inc_theta = TRUE,
  inc_eta = FALSE,
  verbose = FALSE,
  dropfixed = FALSE,
  uncert = FALSE,
  restheta = NULL
)
}
\arguments{
\item{ext}{character string with location of ext file with final model parameters}

\item{covmat}{character string with location of cov file with covariance matrix or data.frame of cov file}

\item{bootstrap}{character string with location of ext files from bootrstrap or vector with ext files from bootstrap
(relevant in case ext files should be excluded because of minimization issues)}

\item{seed}{a numeric with the seed number used in set.seed to enable reproducibility, when not provided
the seed from the global environment will be used (e.g. using \link[base:Random]{base::set.seed})}

\item{nrepl}{numeric with the number of replicates to sample}

\item{inc_theta}{logical indicating if THETAs should be added to result}

\item{inc_eta}{logical indicating if ETAs should be added to result}

\item{verbose}{logical indicating if additional information should be added to result (e.g. OMEGA/SIGMA values)}

\item{dropfixed}{logical indicating if parameters that are fixed should be dropped (can only be done in case covmat is provided)}

\item{uncert}{logical indicating if the uncertainty should be sampled}

\item{restheta}{character with the theta that describes residual error (e.g. "THETA3") in case uncertainty is sampled
this parameter will be set to the population value}
}
\value{
a dataframe with sampled values
}
\description{
This function samples model parameters from NONMEMs covariate matrix (\emph{.cov) and final parameter
estimates (}.ext). Furthermore etas can also be sampled where eta blocks are taken into account (see details)
}
\details{
In general the function can be used to sample from covariance matrix so the different parameters can be
added to a simulation dataset or model to enable uncertainty simulations. In most cases it is not necessary to
include OMEGAs or SIGMAs in these type of simulations. It can be convenient to add ETAs in the simulation dataset
to perform a simulation where no $THETA or $OMEGA information is necessary. In case inc_eta is TRUE, the ETAs
from the ext files are used and placed in a matrix to take into account covariance or
'BLOCKS'. A matrix from the ext file is constructed based on the naming of OMEGA values (e.g. OMEGA.2.1. will be
added to row 2, column 1 and column 1, row 2). The matrix is used as Sigma for the mvrnorm function with mu=0.
}
\examples{

ext <- system.file("example_models","PK.1CMT.ORAL.COV.ext", package = "amp.sim")
cov <- system.file("example_models","PK.1CMT.ORAL.COV.cov", package = "amp.sim")
sample_par(ext, inc_eta = TRUE, nrepl = 5)
sample_par(ext, cov, uncert = TRUE, nrepl = 5)

}
\seealso{
\link[MASS:mvrnorm]{MASS::mvrnorm}
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_inits.r
\name{get_inits}
\alias{get_inits}
\title{get the initial states for the differential equations}
\usage{
get_inits(lstblock)
}
\arguments{
\item{lstblock}{list with each item being a separate structured dollar block, usually obtain from \code{\link{nmlistblock}}}
}
\value{
a named vector with the state values
}
\description{
This function will extracts the initial states for the differential equations from a NONMEM model
}
\examples{

mod    <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
mdll   <- get_nmblock(mod,block=c("PK","DES"))
mdlls  <- nmlistblock(mdll)
get_inits(mdlls)

}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_nmblock.r
\name{get_nmblock}
\alias{get_nmblock}
\title{Get information from dollar blocks inside NONMEM control streams}
\usage{
get_nmblock(model, block, ret = "content", omitbn = TRUE)
}
\arguments{
\item{model}{character vector of length 1 with filename of model, in case length is greater than 1 it is assumed to be a vector with model code}

\item{block}{character vector with names of the model blocks. Take into account that grep is used with respect to partial matching}

\item{ret}{character with the type of return value can be either "content" or "index"}

\item{omitbn}{logical indicating if the name of the block should be omitted when return (has only effect if ret="content")}
}
\value{
a list with either a numeric vector with the indices or a character vector with the content of the dollar blocks
}
\description{
This function returns a list with indices or content of dollar blocks
}
\examples{

mod  <- system.file("example_models","PK.1CMT.ORAL.mod", package = "amp.sim")
get_nmblock(mod,"OMEGA")
}
\author{
Richard Hooijmaijers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_sim.r
\name{sample_sim}
\alias{sample_sim}
\title{Sample model parameters for a simulation}
\usage{
sample_sim(nrepl = 2, nsub = 3, type = "noIIV", ...)
}
\arguments{
\item{nrepl}{Number of replicates for the simulation}

\item{nsub}{Number of subjects for the simulation}

\item{type}{character with the type of simulation to perform (see details)}

\item{...}{Additional arguments passed to \link{sample_par}
mainly for passing information for ext and cov files}
}
\value{
a dataframe with sampled values
}
\description{
This function samples parameters for a simulation. It wraps around the \link{sample_par} function on the background
}
\details{
This function is a high level function wrapper for the \code{sample_par} function specified
for different types of simulations that might occur. Currently the following situations can be sampled:
\itemize{
\item \code{noIIV}: Sample without uncertainty and without IIV
\item \code{sameIIV}: Sample without uncertainty and with the same IIV values within subjects
\item \code{varIIV}: Sample without uncertainty and with different IIV values within subjects
\item \code{unc_noIIV}: Sample with uncertainty and without IIV
\item \code{unc_sameIIV}: Sample with uncertainty and and with the same IIV values within subjects
\item \code{unc_varIIV}: Sample with uncertainty and with different IIV values within subjects
}

In all the cases above where uncertainty is sampled, this is done only for THETA values.
}
\examples{

ext <- system.file("example_models","PK.1CMT.ORAL.COV.ext", package = "amp.sim")
cov <- system.file("example_models","PK.1CMT.ORAL.COV.cov", package = "amp.sim")
sample_sim(nrepl=2,nsub=3,type="unc_varIIV", ext=ext,cov=cov)
sample_sim(nrepl=2,nsub=3,type="sameIIV", ext=ext)

}
\author{
Richard Hooijmaijers
}
